/**
 * Collapsable div, since I can't get bulma-accordion to work
 * @moduleName Accordion 
 * @snippet Accordion|constructor
Accordion({
    title: ''
})
 */
bento.define('debug/html/accordion', [
    'bento',
    'bento/utils',
    'debug/html'
], function (
    Bento,
    Utils,
    Html
) {
    'use strict';
    var id = 0;
    var Accordion = function (settings) {
        var isToggled = false;
        var accordionId = id++;
        var description = settings.description;
        var showTrash = settings.showTrash || settings.canRemove;
        var body = Html.make('div', 'card-content', 'accordion-' + accordionId, {
            style: {
                display: 'none'
            }
        });
        var onClick = function () {
            isToggled = !isToggled;
            body.style.display = isToggled ? 'block' : 'none';
            arrow.className = isToggled ? 'fa fa-angle-down' : 'fa fa-angle-right';
        };
        var arrow = Html.make('i', isToggled ? 'fa fa-angle-down' : 'fa fa-angle-right', '', {});
        var clickable = Html.make('a', '', '', {
            href: '#accordion-' + accordionId
        }).append(
            Html.make('span', 'icon is-small').append(arrow)
        );
        var trashIcon = Html.make('a', 'is-pulled-right is-relative', '', {
            onclick: function () {
                // remove self
                Html.remove(container);
            }
        }).append(Html.make('i', 'fas fa-trash', '', {}));
        var title = Html.make('span', '', null, {
            style: 'margin-left: 4px',
            innerHtml: settings.title,
        });
        var footerText = Html.make('p', 'card-footer-item', '', {
            innerHtml: description,
            style: 'align-items: flex-start; justify-content: flex-start; padding: 0px 0px 0px 32px;'
        });
        var footer = Html.make('footer', 'card-footer', '', {
            style: 'border-top: none; font-size: 12px; justify-content: normal;'
        }).append(footerText);
        var header = Html.make('header', 'card-header-title', null, {
            style: 'display: block;',
            onclick: onClick
        }).append([
            clickable,
            title
        ]);
        var container = Html.make('div', 'card', null, {}).append([
            header,
            body
        ]);
        if (description) {
            container.append(footer);
        }

        if (settings.isToggled) {
            onClick();
        }
        if (showTrash) {
            header.append(trashIcon);
        }

        return new Object({
            container: container,
            body: body,
            title: title,
            append: function (child) {
                body.append(child);
                return this;
            },
            attach: function (child) {
                body.attach(child);
                return this;
            },
            addDescription: function (text) {
                if (!description) {
                    container.append(footer);
                }
                description = text;
                footerText.innerHTML = text;
            }
        });
    };
    return Accordion;
});