/**
 * IAP debug test, using atomic-plugins-inapps-...
 * @moduleName IAPTest
 */
bento.define('debug/tests/iap', [
    'bento',
    'bento/utils',
    'modules/store',
    'debug/html',
    'ui/iapoverlay'
], function (
    Bento,
    Utils,
    Store,
    Html,
    IapOverlay
) {
    'use strict';
    var Test = function (settings) {
        return [{
            // simplest IAP: global variable needs to exists
            name: 'Plugin exists',
            run: function (htmlContainer, onSuccess, onFail) {
                if (window.Cocoon && window.Cocoon.InApp) {
                    onSuccess('PASS');
                } else {
                    onFail('FAIL: missing global variable window.Cocoon.InApp');
                }
            }
        }, {
            name: 'Can purchase',
            run: function (htmlContainer, onSuccess, onFail) {
                if (!window.Cocoon || !window.Cocoon.InApp) {
                    onFail('FAIL: missing global variable window.Cocoon.InApp');
                    return;
                }
                if (window.Cocoon.InApp.canPurchase()) {
                    onSuccess('PASS');
                } else {
                    onFail('FAIL: window.Cocoon.InApp.canPurchase() returns false');
                }
            }
        }, {
            name: 'Get products',
            run: function (htmlContainer, onSuccess, onFail) {
                if (!window.Cocoon || !window.Cocoon.InApp) {
                    onFail('FAIL: missing global variable window.Cocoon.InApp');
                    loading.removeSelf();
                    return;
                }
                var loading = Html('<p>Waiting for store...<i class="fas fa-circle-notch fa-spin"></i></p>');
                var attempts = 10;
                var getProducts = function () {
                    var productsCache = Store.getCachedProducts();
                    var ids = [];
                    Utils.forEach(productsCache, function (product) {
                        ids.push(product.productId);
                    });
                    if (productsCache.length) {
                        onSuccess('PASS: loaded ' + ids.join(', '));
                    } else {
                        onFail('FAIL: 0 products retrieved (assuming there are IAP products)');
                    }
                    loading.removeSelf();
                };
                var checkLoaded = function () {
                    // wait 10 seconds and try again
                    window.setTimeout(function () {
                        if (attempts === 0 && !Store.isLoaded()) {
                            onFail('FAIL: Store loading timed out');
                            loading.removeSelf();
                        } else if (attempts > 0 && !Store.isLoaded()) {
                            attempts -= 1;
                            loading.innerHtml = 'Waiting for store... ' + attempts + ' <i class="fas fa-circle-notch fa-spin"></i></p>';
                            checkLoaded();
                        } else {
                            getProducts();
                        }
                    }, 1000);
                };

                if (!Store.isLoaded()) {
                    checkLoaded();
                } else {
                    getProducts();
                }
                htmlContainer.append(loading);
            }
        }, {
            name: 'Make purchase',
            run: function (htmlContainer, onSuccess, onFail) {
                if (!settings.testProduct) {
                    onFail('FAIL: IAP test not properly initialized');
                    return;
                }
                var isBuying = false;
                var info = Html('<p>Product to buy: ' + settings.testProduct + '</p>');
                var button = Html('<a class="level button is-primary">Purchase</a>', {
                    onclick: function () {
                        if (isBuying) {
                            // disable button
                            return;
                        }
                        isBuying = true;
                        button.innerHtml = '<i class="fas fa-circle-notch fa-spin"></i>';
                        IapOverlay.purchase({
                            id: settings.testProduct,
                            isConsumable: true,
                            resolveAutomatically: true,
                            onPurchaseComplete: function (success, id, purchaseInfo) {
                                if (success && id === settings.testProduct) {
                                    onSuccess('PASS (note: in-game logic not tested)');
                                } else {
                                    onFail('FAIL: purchase was not successful');
                                }
                                button.removeSelf();
                                info.removeSelf();
                            }
                        });
                    }
                });
                htmlContainer.attach(info);
                htmlContainer.attach(button);
            }
        }];
    };
    return Test;
});