/**
 * Tenjin debug test
 * @moduleName TenjinTest
 */
bento.define('debug/tests/tenjin', [
    'bento',
    'bento/utils',
    'modules/tenjin',
    'debug/html'
], function (
    Bento,
    Utils,
    Tenjin,
    Html
) {
    'use strict';
    var Test = function (settings) {
        return [{
            // simplest tenjin: global variable needs to exists
            name: 'Plugin exists',
            run: function (htmlContainer, onSuccess, onFail) {
                if (window.tenjin) {
                    onSuccess('PASS');
                } else {
                    onFail('FAIL: missing global variable window.tenjin');
                }
            }
        }, {
            // debug event: requires setup on Tenjin, first test will always fail (callback can be set up after debug event is sent)
            name: 'Send debug event',
            run: function (htmlContainer, onSuccess, onFail) {
                var testValue = Date.now(); // send a unique value
                var loading = Html('<p>Sent event...<i class="fas fa-circle-notch fa-spin"></i></p>');
                var buttonCheck = Html('<a class="level button is-primary">Check event</a>', {
                    onclick: function () {
                        loading.innerHtml = 'Checking...<i class="fas fa-circle-notch fa-spin"></i>';
                        // remove self and temp fail text
                        buttonCheck.removeSelf();
                        tempFailText.removeSelf();
                        // retry (deliberately use 5sec timeout, the event can take a while to 
                        // come around, spamming the button my lead you to think it's not working)
                        checkValue(5000);
                    }
                });
                var tempFailText = Html('p', {
                    style: 'color: red;',
                    innerHtml: 'FAIL'
                });
                var retrieveValue = function (failureCallback) {
                    var xhr = new window.XMLHttpRequest();
                    xhr.open('GET', 'https://wjp7krc4ml.execute-api.us-east-1.amazonaws.com/latest/get-record?value=' + testValue);
                    xhr.onreadystatechange = function () {
                        if (xhr.readyState === 4) {
                            if (xhr.status >= 200 && xhr.status < 300) {
                                var json;
                                try {
                                    json = JSON.parse(xhr.responseText);
                                    if (json.Count) {
                                        // onSuccess('PASS: ' + JSON.stringify(json.Items[0]));
                                        onSuccess('PASS: Custom event found');
                                    } else {
                                        failureCallback('FAIL: no records of custom event, try again. (Is the debug callback set up in Tenjin?)');
                                    }
                                } catch (e) {
                                    failureCallback('FAIL: ' + e);
                                }
                            } else {
                                failureCallback('FAIL: ' + xhr.responseText);
                            }
                            loading.removeSelf();
                        }
                    };
                    xhr.ontimeout = function (e) {
                        onFail('FAIL: HTTP request timeout');
                        loading.removeSelf();
                    };
                    xhr.send();
                    loading.innerHtml = 'Checking callback...<i class="fas fa-circle-notch fa-spin"></i>';
                };
                var checkValue = function (timeout) {
                    htmlContainer.attach(loading);
                    window.setTimeout(function () {
                        retrieveValue(function (msg) {
                            // attach button and let developer retry
                            tempFailText.innerHtml = msg + '<br>';
                            htmlContainer.attach(tempFailText);
                            htmlContainer.attach(buttonCheck);
                        });
                    }, timeout);
                };
                if (!window.tenjin) {
                    onFail('FAIL: missing global variable window.tenjin');
                    return;
                }
                Tenjin.sendEventAndValue('debug', testValue);


                // wait 30 seconds to retrieve from database
                checkValue(30000);
            }
        }];
    };
    return Test;
});