/**
 * IronSrc debug test
 * @moduleName IronSrcTest
 */
bento.define('debug/tests/toutiao', [
    'bento',
    'bento/utils',
    'debug/html',
    'modules/toutiao'
], function (
    Bento,
    Utils,
    Html,
    Toutiao
) {
    'use strict';
    var Test = function (settings) {
        return [{
            // simplest: global variable needs to exists
            name: 'Plugin exists',
            run: function (htmlContainer, onSuccess, onFail) {
                if (window.cordova && window.cordova.plugins && window.cordova.plugins.toutiao) {
                    onSuccess('PASS');
                } else {
                    onFail('FAIL: Missing global variable window.plugins.toutiao');
                }
            }
        }, {
            // can view a rewarded ad
            name: 'Can view a rewarded video ad',
            run: function (htmlContainer, onSuccess, onFail) {
                // append a button so the user can wait until a rewarded video is available 
                var button = Html('<a class="level button is-primary">Loading video...<i class="fas fa-circle-notch fa-spin"></i></a>', {
                    onclick: function () {
                        if (hasRewarded) {
                            Toutiao.showRewarded(function () {
                                onSuccess('PASS: video watched');
                                button.removeSelf();
                            }, function (e) {
                                onFail('FAIL: could not watch video ' + e);
                                button.removeSelf();
                            });
                        } else {
                            window.alert('No video has been loaded (yet).');
                        }
                    }
                });
                var hasRewarded = false;
                var checkRewarded = function () {
                    // occasionally check for a rewarded video
                    if (Toutiao.canShowRewarded()) {
                        hasRewarded = true;
                        button.innerHtml = 'Watch video';
                    } else {
                        window.setTimeout(checkRewarded, 1000);
                    }
                };
                htmlContainer.append(button);
                checkRewarded();
            }
        }, {
            // can view a rewarded ad
            name: 'Can view an fullscreen video ad',
            run: function (htmlContainer, onSuccess, onFail) {
                // append a button so the user can wait until a rewarded video is available 
                var button = Html('<a class="level button is-primary">Loading video...<i class="fas fa-circle-notch fa-spin"></i></a>', {
                    onclick: function () {
                        if (hasInterstitial) {
                            Toutiao.showFullScreenVideo(function () {
                                onSuccess('PASS: video watched');
                                button.removeSelf();
                            }, function (e) {
                                onFail('FAIL: could not watch video ' + e);
                                button.removeSelf();
                            });
                        } else {
                            window.alert('No video has been loaded (yet).');
                        }
                    }
                });
                var hasInterstitial = false;
                var checkInterstitial = function () {
                    // occasionally check for a rewarded video
                    Toutiao.canShowFullScreenVideo(function () {
                        hasInterstitial = true;
                        button.innerHtml = 'Watch video';
                    }, function (e) {
                        window.setTimeout(checkInterstitial, 1000);
                    });
                };
                htmlContainer.append(button);
                checkInterstitial();
            }
        }];
    };
    return Test;
});