/**
 * Module description
 * @moduleName Bullet
 * @snippet Bullet.snippet
Bullet({
    position: new Vector2(0, 0)
})
 */
bento.define('entities/bullet', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'components/movable',
    'globals',
    'components/culler',
    'components/spritecontainer',
    'entities/particle'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Movable,
    Globals,
    Culler,
    SpriteContainer,
    Particle
) {
    'use strict';
    return function (settings) {

        // --- PARAMETERS ---
        var camera;
        var position = settings.position;
        var source = settings.source;
        var shootDirection = settings.shootDirection || "left";

        // --- VARS ---
        var xVel = (shootDirection === "left") ? -0.5 : 0.5;


        // --- COMPONENTS ---
        var movable = new Movable({
            velocity: new Vector2(xVel, 0),
            ignore: [source],
            stopOnCollide: false,
            canCollideWith: ['solids', 'hazards', 'breakables'],
            onCollision: function (collisionData) {
                new Particle({
                    z: Globals.layers.effects,
                    spriteSheet: 'effects/hit',
                    position: entity.position.clone(),
                    scale: Globals.pixelScaleV
                });
                entity.removeSelf();
                Utils.forEach(collisionData.entities, function (entity, i, l, breakLoop) {
                    if (entity.name === "bullet") {
                        entity.removeSelf();
                    }
                    if (entity.name === "breakableWall") {
                        entity.break();
                    }
                });
            }
        });
        var sprite = new SpriteContainer({
            spriteSheet: 'cannon/cannonball',
            scale: new Vector2(Globals.pixelScale, Globals.pixelScale),
            visible: false
        });
        var behaviour = {
            name: 'behaviour',
            draw: function (data) {
                var s = sprite;
                s.visible = true;
                s.alpha = 0.5;
                for (var i = 0; i < 5; i++) {
                    s.alpha -= 0.1;
                    s.position.x -= movable.velocity.x * 2;
                    s.draw(data);
                }
                s.position.x = 0;
                s.alpha = 1;
                s.draw(data);
                s.visible = false;
            }
        };

        // --- ENTITY ---
        var entity = new Entity({
            z: Globals.layers.active,
            name: 'bullet',
            family: ['hazards'],
            position: position,
            scale: new Vector2(1, 1),
            boundingBox: new Rectangle(-7, -7, 14, 14),
            components: [
                sprite,
                behaviour,
                new Culler({
                    height: 8
                }),
                movable
            ]
        });
        return entity;
    };
});