/**
 * Module description
 * @moduleName ModuleName
 * @snippet ModuleName.snippet
ModuleName({
    position: new Vector2(0, 0)
})
 */
bento.define('entities/endarrow', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'components/spritecontainer',
    'modules/skinmanager',
    'modules/savemanager',
    'entities/particle',
    'entities/effects/confettiburst',
    'entities/effects/confettistring',
    'components/gamecounter'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    SpriteContainer,
    SkinManager,
    SaveManager,
    Particle,
    ConfettiBurst,
    ConfettiString,
    GameCounter
) {
    'use strict';
    return function (settings) {

        // --- VARS ---
        var gameController = Bento.objects.get("gameController");
        var plantSkins = SkinManager.getAllLockedPlantSkins();
        var isBean = (SaveManager.load('superProgress') >= Globals.superProgressSegments - 1 && plantSkins.length > 0) || (SaveManager.load('superProgress') === Globals.superProgressChest - 1 && !SaveManager.load('hasInitialBean'));
        var isChest = (SaveManager.load('superProgress') === Globals.superProgressChest - 1 && SaveManager.load('hasInitialBean'));
        var isSpecialStage = gameController.getIsSpecialStage();
        var isCollectible = !isSpecialStage && (isBean || isChest);
        var position = settings.position;
        var canPulse = isCollectible;

        // --- FUNCTIONS ---
        var pulseLoop = function () {
            new Tween({ in: 90,
                onComplete: function () {
                    if (canPulse) {
                        new Particle({
                            z: Globals.layers.active + 1,
                            imageName: 'ui/fx/bloom-halo',
                            position: entity.position.add(collectableSpriteContainer.position),
                            scale: Globals.pixelScaleV.scalarMultiply(1.5),
                            removeAfterTime: 45,
                            alpha: 1,
                            originRelative: new Vector2(0.5, 0.5),
                            removeEffect: 'scalefade'
                        });
                        new Particle({
                            z: Globals.layers.active + 1,
                            imageName: 'ui/fx/bloom-halo',
                            position: entity.position.add(collectableSpriteContainer.position),
                            scale: Globals.pixelScaleV.scalarMultiply(2.25),
                            removeAfterTime: 30,
                            alpha: 0.5,
                            originRelative: new Vector2(0.5, 0.5),
                            removeEffect: 'scalefade'
                        });
                        pulseLoop();
                    }
                }
            });
        };
        if (canPulse) {
            pulseLoop();
        }


        // --- COMPONENTS ---
        var beanSprite = new Sprite({
            spriteSheet: 'beans/' + (plantSkins[0] || 'daisy')
        });
        beanSprite.setCurrentSpeed(0);
        var chestSprite = new Sprite({
            spriteSheet: 'ui/small-chest'
        });
        var collectableSpriteContainer = new Entity({
            name: 'collectableSpriteContainer',
            position: new Vector2(0, 14),
            scale: Globals.pixelScaleV.scalarMultiply(0.75)
        });
        if (isCollectible) {
            if (isBean) {
                collectableSpriteContainer.attach(beanSprite);
            }
            if (isChest) {
                collectableSpriteContainer.attach(chestSprite);
            }
        }

        var arrowSprite = new SpriteContainer({
            imageName: 'arrow',
            originRelative: new Vector2(0.5, 0),
            scale: Globals.pixelScaleV.clone()
        });
        var behavior = {
            name: 'behaviorComponent',
            update: function (data) {
                arrowSprite.position.y = ((isCollectible) ? 24 : 0) + Math.abs(Math.sin(entity.timer * 0.075) * 8);
                if (isCollectible) {
                    collectableSpriteContainer.position.x = Math.sin(entity.timer * 0.013) * 3;
                    collectableSpriteContainer.position.y = 14 + Math.sin(entity.timer * 0.027) * 3;
                    collectableSpriteContainer.rotation = Math.sin(entity.timer * 0.035) * 0.1;
                }
            },
            onWin: function () {
                if (isBean) {
                    //bean effects
                    Bento.audio.playSound('sfx_candy_1');
                    collectableSpriteContainer.visible = false;
                    new Particle({
                        z: Globals.layers.effects,
                        spriteSheet: 'effects/coin',
                        position: entity.position.clone().add(new Vector2(0, 8)),
                        alpha: 1,
                        scale: Globals.pixelScaleV.scalarMultiply(2),
                        rotation: 0,
                        velocity: new Vector2(0, 0),
                        gravity: 0.05
                    });
                    new Tween({ in: 8,
                        onComplete: function () {
                            new Particle({
                                z: Globals.layers.tiles - 1,
                                imageName: 'ui/fx/bloom-halo',
                                position: entity.position.clone().add(new Vector2(0, 8)),
                                scale: Globals.pixelScaleV.scalarMultiply(1.2),
                                removeAfterTime: 30,
                                originRelative: new Vector2(0.5, 0.5),
                                removeEffect: 'scalefade'
                            });
                        }
                    });
                } else if (isChest) {
                    //chest effects
                    Bento.audio.playSound('sfx_candy_1');
                    collectableSpriteContainer.visible = false;
                    new Particle({
                        z: Globals.layers.effects,
                        spriteSheet: 'effects/coin',
                        position: entity.position.clone().add(new Vector2(0, 8)),
                        alpha: 1,
                        scale: Globals.pixelScaleV.scalarMultiply(2),
                        rotation: 0,
                        velocity: new Vector2(0, 0),
                        gravity: 0.05
                    });
                    new Tween({ in: 8,
                        onComplete: function () {
                            new Particle({
                                z: Globals.layers.tiles - 1,
                                imageName: 'ui/fx/bloom-halo',
                                position: entity.position.clone().add(new Vector2(0, 8)),
                                scale: Globals.pixelScaleV.scalarMultiply(1.2),
                                removeAfterTime: 30,
                                originRelative: new Vector2(0.5, 0.5),
                                removeEffect: 'scalefade'
                            });
                        }
                    });
                } else {
                    //standard effects
                    new Particle({
                        z: Globals.layers.effects,
                        spriteSheet: 'effects/coin',
                        position: entity.position.clone(),
                        alpha: 1,
                        scale: Globals.pixelScaleV.scalarMultiply(2),
                        rotation: 0,
                        velocity: new Vector2(0, 0),
                        gravity: 0.05
                    });
                    new Particle({
                        z: Globals.layers.effects2,
                        imageName: 'ui/fx/bloom-halo',
                        position: entity.position.clone(),
                        scale: Globals.pixelScaleV.scalarMultiply(0.4),
                        removeAfterTime: 30,
                        originRelative: new Vector2(0.5, 0.5),
                        removeEffect: 'scalefade'
                    });
                }

                //flash screen
                var background = Bento.objects.get('background');
                if (background) {
                    background.flash();
                }

                //stop pulse
                canPulse = false;

                if (isSpecialStage) {
                    gameController.giveCoins(5);
                    Bento.audio.stopSound('sfx_coin_5');
                    Bento.audio.playSound('sfx_coin_5');
                    // make text particle
                    if (Globals.showCounters) {
                        new Particle({
                            z: Globals.layers.effects,
                            sprite: new GameCounter({
                                value: '+5',
                                imageName: 'addcoincounter'
                            }),
                            originRelative: new Vector2(0.5, 0.5),
                            position: entity.position.clone(),
                            rotation: 0,
                            rotationRate: Utils.getRandomRangeFloat(-0.02, 0.02),
                            velocity: new Vector2(Utils.getRandomRangeFloat(-0.5, 0.5), -1.5),
                            scale: Globals.pixelScaleV.scalarMultiply(0.45),
                            acceleration: new Vector2(0, 0.04),
                            friction: 1,
                            removeAfterTime: 60,
                            removeEffect: 'flicker'
                        });
                        for (var i = 0; i < 5; i++) {
                            Bento.objects.attach(new Particle({
                                spriteSheet: 'coin',
                                position: entity.position.clone(),
                                alpha: 1,
                                scale: Globals.pixelScaleV.scalarMultiply(0.75),
                                rotation: Utils.getRandomRangeFloat(-1, 1),
                                rotationRate: Utils.getRandomRangeFloat(-0.1, 0.1),
                                velocity: new Vector2(Utils.getRandomRangeFloat(-1, 1), Utils.getRandomRangeFloat(0, 3)),
                                acceleration: new Vector2(0, 0.065),
                                friction: 1,
                                removeAfterTime: 60,
                                dontAttach: true,
                                z: Globals.layers.active + 0.1
                            }));
                        }
                    }
                }

                //create confetti burst
                for (var i = 10 - 1; i >= 0; i--) {
                    new ConfettiString({
                        position: entity.position.add(new Vector2(Utils.getRandomRangeFloat(-8, 8), Utils.getRandomRangeFloat(-4, 4))),
                        velocity: new Vector2(0, Utils.getRandomRange(-2, -5)).rotateDegree(Utils.getRandomRangeFloat(90, 270)),
                        friction: 0.95
                    });

                }
                for (var i = 30 - 1; i >= 0; i--) {
                    new ConfettiBurst({
                        position: entity.position.add(new Vector2(Utils.getRandomRangeFloat(-8, 8), Utils.getRandomRangeFloat(-4, 4))),
                        velocity: new Vector2(Utils.getRandomRangeFloat(-4, 4), Utils.getRandomRangeFloat(0, 2)),
                        friction: 0.95,
                        particleScale: 0.04 + Utils.getRandomFloat(0.03)
                    });
                }
            }
        };


        // --- ENTITY ---
        var entity = new Entity({
            z: Globals.layers.active - 0.1,
            name: 'endarrow',
            position: position,
            boundingBox: new Rectangle(-12, 0, 24, 20),
            updateWhenPaused: 0,
            float: false,
            components: [
                arrowSprite,
                (isCollectible) ? collectableSpriteContainer : {
                    name: 'blank'
                },
                behavior
            ]
        }).extend({
            onWin: behavior.onWin
        });
        return entity;
    };
});