/**
 * Module description
 * @moduleName GameController
 * @snippet GameController.snippet
GameController({
    position: new Vector2(0, 0)
})
 */
bento.define('entities/gamecontroller', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'entities/levelgenerator',
    'entities/textup',
    'entities/player',
    'globals',
    'ui/mainmenuscreen',
    'entities/solid',
    'components/spritecontainer',
    'entities/revivecloud',
    'ui/revivescreen',
    'modules/ads',
    'ui/screentransition',
    'ui/gamecompletescreen',
    'modules/tenjin',
    'modules/skinmanager',
    'ui/tutorialscreen',
    'ui/pottutorialscreen',
    'modules/gameanalytics',
    'ui/planttutorialscreen',
    'modules/savemanager',
    'entities/background',
    'entities/introcutscene',
    'ui/screentransition2',
    'modules/vipmanager',
    'components/backbutton',
    'ui/pausescreen',
    'modules/levelmanager',
    'entities/effects/confettiburst',
    'entities/effects/confettistring',
    'ui/coindialog',
    'ui/specialtext',
    'modules/cloudsavemanager',
    'ui/dialog',
    'modules/localization',
    'ui/doublecoindialog'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    LevelGenerator,
    TextUp,
    Player,
    Globals,
    MainMenuScreen,
    Solid,
    SpriteContainer,
    ReviveCloud,
    ReviveScreen,
    Ads,
    ScreenTransition,
    GameCompleteScreen,
    Tenjin,
    SkinManager,
    TutorialScreen,
    PotTutorialScreen,
    GameAnalytics,
    PlantTutorialScreen,
    SaveManager,
    Background,
    IntroCutscene,
    ScreenTransition2,
    VipManager,
    BackButton,
    PauseScreen,
    LevelManager,
    ConfettiBurst,
    ConfettiString,
    CoinDialog,
    SpecialText,
    CloudSaveManager,
    Dialog,
    Localization,
    DoubleCoinDialog
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var player;
        var playerSpawn;
        var currentRevivePoint = new Vector2(0, 100000000);
        var camera;
        var gui;

        var thisLevel = LevelManager.getCurrentLevel();
        var levelProgressCount = LevelManager.getLevelProgressCount();
        var superProgress = SaveManager.load('superProgress');
        var levelSkin = Utils.isDefined(Globals.forcedWorldSkin) ? Globals.forcedWorldSkin : 1 + ((Math.floor((thisLevel - 1) / 10)) % Globals.worldCount);
        var levelVariant = 1 + ((thisLevel - 1) % 2);
        var isSpecialStage = (Globals.gameState === Globals.gameStates.specialStage); // an overide for most functionality to go into special stage
        var isPlaying = false;
        var canScrollCamera = false;
        var nextLevelPosition = new Vector2(viewport.width * 0.5, viewport.height);
        var startPlayerY = 0;
        var levelPercent = 0;
        var endPlayerY = 0;
        var coinsCollected = 0;
        var coinsToCashIn = 0;
        var freeRespawns = 0;
        var deathCount = 0;

        if (levelProgressCount < 5) {
            freeRespawns = 1;
        }
        if (thisLevel === 7) {
            freeRespawns = 1;
        }
        Globals.canAdRevive = true;

        // --- FUNCTIONS ---
        var startLevel = function () {
            if (!isPlaying) {
                // start game
                if (!window.ISCHINA) {
                    Globals.attemptsSinceAd++;
                }
                isPlaying = true;
                Bento.audio.playSound('sfx_start');

                if (getIsSpecialStage()) {
                    //we're on a bonus stage level
                    SaveManager.save('levelsSinceSpecialStage', 0);

                    Bento.audio.playMusic('music/bgm_gameplay_fever_loop', true);
                    Globals.mainMusicVolume = 0.5;
                    Bento.audio.setVolume(Globals.mainMusicVolume, 'music/bgm_gameplay_fever_loop');
                } else {
                    new Tween({
                        from: Globals.mainMusicVolume,
                        to: 0.5,
                        in: 60,
                        ease: 'linear',
                        onUpdate: function (v, t) {
                            Globals.mainMusicVolume = v;
                            Bento.audio.setVolume(v, 'music/bgm_gameplay_loop');
                        }
                    });
                }

                // enable camera control
                canScrollCamera = true;
                // enable the player
                player.setEnabled(true);
                // zoom camera to default scale 
                new Tween({
                    from: camera.getViewScale(),
                    to: 0.875,
                    in: 60,
                    delay: 0,
                    applyOnDelay: 0,
                    ease: 'easeOutExpo',
                    onUpdate: function (v, t) {
                        camera.setViewScale(v);
                    }
                });
                //start analytics
                EventSystem.fire('GameAnalytics-addProgressionEvent', {
                    progressionStatus: GameAnalytics.EGAProgressionStatus.Start,
                    progression01: "level_" + thisLevel
                });
                EventSystem.fire("FirebaseAnalytics-logEvent", {
                    name: "level_start",
                    properties: {
                        level: thisLevel
                    }
                });
                if (thisLevel === 1 && !getIsSpecialStage()) {
                    EventSystem.fire("FirebaseAnalytics-setScreenName", "gameplay_tutorial");
                } else {
                    EventSystem.fire("FirebaseAnalytics-setScreenName", "gameplay");
                }
            }
        };

        var onFeverChanged = function (starting) {
            if (getIsSpecialStage()) {
                return;
            }
            if (starting) {
                new Tween({
                    from: Globals.mainMusicVolume,
                    to: 0,
                    in: 30,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        Globals.mainMusicVolume = v;
                        Bento.audio.setVolume(v, 'music/bgm_gameplay_loop');
                    },
                    onComplete: function () {
                        Bento.audio.stopMusic('music/bgm_gameplay_loop');
                    }
                });

                Bento.audio.stopMusic('music/bgm_gameplay_fever_loop');
                Bento.audio.playMusic('music/bgm_gameplay_fever_loop', true);
                Bento.audio.setVolume(Globals.feverMusicVolume, 'music/bgm_gameplay_loop');
                new Tween({
                    from: Globals.feverMusicVolume,
                    to: 0.66,
                    in: 30,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        Globals.feverMusicVolume = v;
                        Bento.audio.setVolume(v, 'music/bgm_gameplay_fever_loop');
                    }
                });
            } else {
                Bento.audio.stopMusic('music/bgm_gameplay_loop');
                Bento.audio.playMusic('music/bgm_gameplay_loop', true);
                new Tween({
                    from: Globals.mainMusicVolume,
                    to: 0.5,
                    in: 30,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        Globals.mainMusicVolume = v;
                        Bento.audio.setVolume(v, 'music/bgm_gameplay_loop');
                    }
                });
                new Tween({
                    from: Globals.feverMusicVolume,
                    to: 0,
                    in: 30,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        Globals.feverMusicVolume = v;
                        Bento.audio.setVolume(v, 'music/bgm_gameplay_fever_loop');
                    },
                    onComplete: function () {
                        Bento.audio.stopMusic('music/bgm_gameplay_fever_loop');
                    }
                });
            }
        };

        var startSpecialStage = function () {
            // disable the player
            player.setEnabled(false);
            // zoom in camera
            camera.setViewScale(0.66);
            //set camera position to 
            var cameraPosition = new Vector2(viewport.width * 0.5, player.position.y - 64);
            camera.setTargetPosition(cameraPosition);
            camera.position = cameraPosition.clone();

            //animated intro
            var drawEntity = player.getDrawEntity2();
            player.z = Globals.layers.behindtiles;
            drawEntity.scale = new Vector2(0.5, 0.5);
            drawEntity.position.y = drawEntity.position.y + 32;
            new Tween({ in: 30,
                onComplete: function () {
                    new Tween({
                        from: new Vector2(0.5, 0.5),
                        to: new Vector2(1, 1),
                        in: 60,
                        ease: 'easeOutBack',
                        onUpdate: function (v, t) {
                            drawEntity.scale = v;
                        }
                    });
                    Bento.audio.playSound('sfx_start');
                    new Tween({
                        from: drawEntity.position.y,
                        to: drawEntity.position.y - 32,
                        in: 45,
                        ease: 'easeOutBack',
                        onUpdate: function (v, t) {
                            drawEntity.position.y = v;
                        }
                    });
                    new Tween({ in: 30,
                        onComplete: function () {
                            player.z = Globals.layers.active;
                            player.snapHead();
                            Bento.audio.stopSound('sfx_grow_head');
                            Bento.audio.playSound('sfx_grow_head');
                            new Tween({ in: 30,
                                onComplete: function () {
                                    player.snapHead();
                                    Bento.audio.stopSound('sfx_grow_head');
                                    Bento.audio.playSound('sfx_grow_head');
                                }
                            });
                            new Tween({ in: 50,
                                onComplete: function () {
                                    player.snapHead();
                                    Bento.audio.stopSound('sfx_grow_head');
                                    Bento.audio.playSound('sfx_grow_head');
                                }
                            });
                            Bento.audio.setVolume(0.25, 'sfx_burst');
                            new Tween({ in: 70,
                                onComplete: function () {
                                    //begin level
                                    startLevel();

                                    //fire poppers
                                    Bento.audio.playSound('sfx_burst');

                                    //make text
                                    new SpecialText({});

                                    // fade in special stage effects
                                    var background = Bento.objects.get('background');
                                    new Tween({
                                        from: 0,
                                        to: 1,
                                        in: 30,
                                        ease: 'linear',
                                        onUpdate: function (v, t) {
                                            background.setSpecialIntensity(v);
                                        }
                                    });

                                    //create confetti burst
                                    for (var i = 10 - 1; i >= 0; i--) {
                                        new ConfettiString({
                                            position: player.position.add(new Vector2(Utils.getRandomRangeFloat(-8, 8), Utils.getRandomRangeFloat(-4, 4))),
                                            velocity: new Vector2(0, Utils.getRandomRange(-2, -5)).rotateDegree(Utils.getRandomRangeFloat(-90, 90)),
                                            friction: 0.95
                                        });

                                    }
                                    for (i = 30 - 1; i >= 0; i--) {
                                        new ConfettiBurst({
                                            position: player.position.add(new Vector2(Utils.getRandomRangeFloat(-8, 8), Utils.getRandomRangeFloat(-4, 4))),
                                            velocity: new Vector2(Utils.getRandomRangeFloat(-4, 4), Utils.getRandomRangeFloat(0, -4)),
                                            friction: 0.95,
                                            particleScale: 0.04 + Utils.getRandomFloat(0.03)
                                        });
                                    }

                                }
                            });
                        }
                    });
                }
            });
        };

        var makeMainMenu = function () {
            // make main menu
            var thisMenu = new MainMenuScreen({});
            // disable the player
            player.setEnabled(false);
            // zoom in camera
            camera.setViewScale(0.5);
            //set camera position to 
            var cameraPosition = new Vector2(viewport.width * 0.5, player.position.y - 24);
            camera.setTargetPosition(cameraPosition);
            camera.position = cameraPosition.clone();
            return thisMenu;
        };

        var revivePlayer = function () {
            player.respawnAt(currentRevivePoint.clone());
            var cloud = new ReviveCloud({
                position: currentRevivePoint.clone()
            });
            Bento.objects.attach(cloud);
        };

        var checkInterstitials = function (complete, didWin) {
            var placement = (didWin) ? 'onLevelCompleteA' : 'onLevelFailA';
            if (
                ((Globals.ISCHINA && thisLevel > 4) || (thisLevel !== 3)) &&
                Globals.attemptsSinceAd >= Globals.maxAttemptsBetweenInterstitials &&
                SaveManager.load('showForcedAds') &&
                !VipManager.isVip()
            ) {
                if (Utils.isDev()) {
                    window.alert("This is an Interstital Ad!");
                    Bento.input.resetPointers();
                    Globals.attemptsSinceAd = 0;
                    complete();
                } else {
                    Ads.showInterstitial(function () {
                        Globals.attemptsSinceAd = 0;
                        complete();
                    }, function (e) {
                        complete();
                    }, placement);
                }
            } else {
                complete();
            }
        };

        var getIsSpecialStage = function () {
            return isSpecialStage;
        };

        var onLevelComplete = function () {
            //we are now not playing
            isPlaying = false;
            // lock camera
            canScrollCamera = false;

            Bento.audio.playSound('sfx_land');

            //cash in coins left to cash in
            Utils.giveCoins(coinsToCashIn, 'ingame', 'coinCollected');
            coinsToCashIn = 0;

            if (getIsSpecialStage()) {
                //hide gui on win
                gui.hide();
                Bento.objects.get('specialText', function (specialText) {
                    specialText.removeSelf();
                });

                var background = Bento.objects.get('background');
                new Tween({
                    from: 1,
                    to: 0,
                    in: 30,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        background.setSpecialIntensity(v);
                    }
                });

                // fade music
                new Tween({
                    from: Globals.mainMusicVolume,
                    to: 0.25,
                    in: 30,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        Globals.mainMusicVolume = v;
                        Bento.audio.setVolume(v, 'music/bgm_gameplay_loop');
                    }
                });
                new Tween({
                    from: Globals.mainMusicVolume,
                    to: 0.25,
                    in: 30,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        Globals.mainMusicVolume = v;
                        Bento.audio.setVolume(v, 'music/bgm_gameplay_fever_loop');
                    }
                });

                // back to the main menu
                Globals.gameState = Globals.gameStates.menu;

                new Tween({ in: 30,
                    onComplete: function () {
                        new DoubleCoinDialog({
                            numberOfCoins: coinsCollected
                        });
                    }
                });
                new Tween({ in: 45,
                    onComplete: function () {
                        new Tween({
                            from: Globals.mainMusicVolume,
                            to: 0,
                            in: 15,
                            ease: 'linear',
                            onUpdate: function (v, t) {
                                Globals.mainMusicVolume = v;
                                Bento.audio.setVolume(v, 'music/bgm_gameplay_fever_loop');
                            }
                        });
                        new ScreenTransition({
                            destination: 'screens/main',
                            position: new Vector2(viewport.width * 0.5, viewport.height * 0.5 + 64),
                            onScreenChange: function (complete) {
                                checkInterstitials(complete, true);
                            }
                        });
                    }
                });
            } else {

                // fade music
                new Tween({
                    from: Globals.mainMusicVolume,
                    to: 0.25,
                    in: 15,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        Globals.mainMusicVolume = v;
                        Bento.audio.setVolume(v, 'music/bgm_gameplay_loop');
                    }
                });
                new Tween({
                    from: Globals.mainMusicVolume,
                    to: 0.25,
                    in: 15,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        Globals.mainMusicVolume = v;
                        Bento.audio.setVolume(v, 'music/bgm_gameplay_fever_loop');
                    }
                });

                //bump globals
                Globals.levelsThisSession++;
                Globals.levelsSinceVIP++;
                SaveManager.add('levelsSinceSpecialStage', 1);
                if (window.ISCHINA) {
                    Globals.attemptsSinceAd++;
                }

                //save collected coins
                saveCollectedCoins();

                //unlock next level
                LevelManager.levelComplete();

                //send analytics
                EventSystem.fire('GameAnalytics-addProgressionEvent', {
                    progressionStatus: GameAnalytics.EGAProgressionStatus.Complete,
                    progression01: "level_" + thisLevel
                });
                EventSystem.fire('GameAnalytics-addDesignEvent', {
                    eventId: "progression:coinsCollected:" + thisLevel,
                    value: coinsCollected
                });
                EventSystem.fire('GameAnalytics-addDesignEvent', {
                    eventId: "progression:deathCount:" + thisLevel,
                    value: deathCount
                });
                EventSystem.fire('GameAnalytics-addDesignEvent', {
                    eventId: "progression:levelCompleted:" + thisLevel
                });
                EventSystem.fire('GameAnalytics-addDesignEvent', {
                    eventId: "RewardedAdsCount:Level:" + thisLevel,
                    value: Bento.saveState.load('watchedAds', 0)
                });
                EventSystem.fire("FirebaseAnalytics-logEvent", {
                    name: "level_complete",
                    properties: {
                        level: thisLevel,
                        coinsCollected: coinsCollected,
                        deathCount: deathCount
                    }
                });

                if (LevelManager.getBeatenLevels() === 10) {
                    EventSystem.fire('Tenjin', {
                        name: "10_games_completed"
                    });
                }
                if (LevelManager.getBeatenLevels() === 50) {
                    EventSystem.fire('Tenjin', {
                        name: "50_games_completed"
                    });
                }
                if (LevelManager.getBeatenLevels() === 100) {
                    EventSystem.fire('Tenjin', {
                        name: "100_games_completed"
                    });
                }

                // do we unlock a skin this level? unlock it
                var unlockedSkin = Utils.unlockLevelSkins();
                //are we about to get the bean tutorial
                var doInitialBean = (superProgress === 2 && !SaveManager.load('hasInitialBean'));

                //create the game complete screen
                new GameCompleteScreen({
                    thisLevel: thisLevel,
                    unlockedSkin: unlockedSkin,
                    freeBean: doInitialBean,
                    onClose: function (goToSpecialStage) {
                        if (doInitialBean && superProgress < Globals.superProgressSegments - 1) {
                            superProgress++;
                            SaveManager.save('superProgress', superProgress);
                        }
                        if (goToSpecialStage) {
                            //WOO SPECIAL STAGE TIME
                            Globals.gameState = Globals.gameStates.specialStage;
                            new ScreenTransition({
                                destination: 'screens/main',
                                imageName: 'transition_star',
                                position: new Vector2(viewport.width * 0.5, viewport.height * 0.5 + 64)
                            });
                            new Tween({
                                from: Globals.mainMusicVolume,
                                to: 0,
                                in: 15,
                                ease: 'linear',
                                onUpdate: function (v, t) {
                                    Globals.mainMusicVolume = v;
                                    Bento.audio.setVolume(v, 'music/bgm_gameplay_loop');
                                }
                            });
                        } else {
                            // back to the main menu
                            Globals.gameState = Globals.gameStates.menu;
                            new ScreenTransition({
                                destination: 'screens/main',
                                position: new Vector2(viewport.width * 0.5, viewport.height * 0.5 + 64),
                                onScreenChange: checkInterstitials
                            });
                            new Tween({
                                from: Globals.mainMusicVolume,
                                to: 0,
                                in: 15,
                                ease: 'linear',
                                onUpdate: function (v, t) {
                                    Globals.mainMusicVolume = v;
                                    Bento.audio.setVolume(v, 'music/bgm_gameplay_loop');
                                }
                            });
                        }
                    }
                });
                // up the super progress
                if (!doInitialBean && superProgress < Globals.superProgressSegments - 1) {
                    superProgress++;
                    SaveManager.save('superProgress', superProgress);
                }
            }
        };


        var onGameOver = function () {
            //cash in coins left to cash in
            Utils.giveCoins(coinsToCashIn, 'ingame', 'coinCollected');
            coinsToCashIn = 0;
            saveCollectedCoins();

            // send analytics
            EventSystem.fire('GameAnalytics-addDesignEvent', {
                eventId: "progression:coinsCollected:" + thisLevel,
                value: coinsCollected
            });
            EventSystem.fire('GameAnalytics-addProgressionEvent', {
                progressionStatus: GameAnalytics.EGAProgressionStatus.Fail,
                progression01: "level_" + thisLevel,
                score: Globals.currentScore
            });
            EventSystem.fire("FirebaseAnalytics-logEvent", {
                name: "level_fail",
                properties: {
                    level: thisLevel
                }
            });

            //make game over screen
            new GameCompleteScreen({
                currentLevel: thisLevel,
                failed: true,
                onClose: function () {
                    // back to the main menu
                    Globals.gameState = Globals.gameStates.menu;
                    new ScreenTransition({
                        destination: 'screens/main',
                        position: new Vector2(viewport.width * 0.5, viewport.height * 0.5 + 64),
                        onScreenChange: function (complete) {
                            checkInterstitials(complete, false);
                        }
                    });
                }
            });
        };

        // die
        var onDeath = function () {
            deathCount++;
            new Tween({ in: (Globals.longerDeathDelay) ? 180 : 60,
                onComplete: function () {
                    if (levelProgressCount <= 2 || freeRespawns > 0 || getIsSpecialStage()) {
                        new Tween({ in: 30,
                            onComplete: function () {
                                revivePlayer();
                                freeRespawns--;
                            }
                        });
                        return;
                    }

                    //cash in coins left to cash in
                    Utils.giveCoins(coinsToCashIn, 'ingame', 'coinCollected');
                    coinsToCashIn = 0;
                    saveCollectedCoins();

                    //make revive screen
                    new ReviveScreen({
                        deathCount: deathCount,
                        onComplete: function () {
                            Globals.canAdRevive = false;
                            Globals.attemptsSinceAd = 0;
                            new Tween({ in: 60,
                                onComplete: function () {
                                    revivePlayer();
                                }
                            });
                        },
                        onSkipped: function () {
                            onGameOver();
                        }
                    });
                }
            });
        };

        var markedCoins = [];
        var saveCollectedCoins = function () {
            if (!isSpecialStage && (thisLevel < LevelManager.getInfiniteModeLevel())) {
                console.log('Saving Coins!');
                var collectedCoinIdentifiers = SaveManager.load('collectedCoinIdentifiers');
                collectedCoinIdentifiers = collectedCoinIdentifiers.concat(markedCoins);
                SaveManager.save('collectedCoinIdentifiers', collectedCoinIdentifiers);
            }
            markedCoins = [];
        };

        // --- COMPONENTS ---
        var behaviour = {
            name: 'behaviourComponent',
            start: function () {
                // collecting things we need
                if (!camera) {
                    camera = Bento.objects.get('camera');
                    if (!camera) {
                        return;
                    }
                }

                EventSystem.on('onFeverChanged', onFeverChanged);

                // reset stuff
                Globals.currentScore = 0;

                // set the game state from stuff
                if (thisLevel === 1 && !getIsSpecialStage()) {
                    Globals.gameState = Globals.gameStates.tutorial;
                    if (!SaveManager.load('watchedCutscene')) {
                        Globals.gameState = Globals.gameStates.cutscene;
                    }
                }

                //make screen transition wait for all clear
                ScreenTransition.canTweenOut = false;

                // make level
                entity.attach(new LevelGenerator({
                    level: thisLevel,
                    startPosition: nextLevelPosition.clone(),
                    levelSkin: levelSkin,
                    hasSuper: ((thisLevel) % 3) === 0,
                    isSpecialStage: getIsSpecialStage(),
                    onComplete: function () {
                        Bento.objects.get('playerSpawn', function (thisPlayerSpawn) {
                            startPlayerY = thisPlayerSpawn.position.y;
                            playerSpawn = thisPlayerSpawn;
                        });
                        Bento.objects.get('endarrow', function (endarrow) {
                            endPlayerY = endarrow.position.y;
                        });

                        player = new Player({
                            position: playerSpawn.position.clone(),
                            potSkin: SkinManager.getCurrentPotSkin(),
                            plantSkin: SkinManager.getCurrentPlantSkin(),
                            doScore: !getIsSpecialStage()
                        });
                        Bento.objects.attach(player);

                        Bento.audio.stopMusic('music/bgm_gameplay_loop');
                        Bento.audio.stopMusic('music/bgm_gameplay_fever_loop');
                        if (!getIsSpecialStage()) {
                            Bento.audio.playMusic('music/bgm_gameplay_loop', true);
                            Bento.audio.setVolume(Globals.mainMusicVolume, 'music/bgm_gameplay_loop');
                            new Tween({
                                from: Globals.mainMusicVolume,
                                to: 0.25,
                                in: 15,
                                ease: 'linear',
                                onUpdate: function (v, t) {
                                    Globals.mainMusicVolume = v;
                                    Bento.audio.setVolume(v, 'music/bgm_gameplay_loop');
                                }
                            });
                        }

                        // spawn stuff specific from game state
                        switch (Globals.gameState) {
                        case Globals.gameStates.cutscene:
                            // make main menu
                            Bento.objects.attach(new IntroCutscene({}));
                            break;
                        case Globals.gameStates.tutorial:
                            // make main menu
                            Bento.objects.attach(new TutorialScreen({}));
                            startLevel();
                            break;
                        case Globals.gameStates.menu:
                            // make main menu
                            makeMainMenu();
                            break;
                        case Globals.gameStates.game:
                            startLevel();
                            break;
                        case Globals.gameStates.specialStage:
                            startSpecialStage();
                            break;
                        }

                        // give transition the all clear
                        ScreenTransition.canTweenOut = true;
                    }
                }));

                // --- BACKGROUND ---
                var background = new Background({
                    levelSkin: levelSkin,
                    levelVariant: levelVariant
                });
                Bento.objects.attach(background);

                EventSystem.on('onLevelComplete', onLevelComplete);
                EventSystem.on('onDeath', onDeath);
            },
            update: function (data) {
                // check we have everything
                if (!player) {
                    return;
                }
                if (!camera) {
                    return;
                }

                if (!gui) {
                    gui = Bento.objects.get('gameGUI');
                }

                // get player head position
                var playerHeadPosition = player.getHeadPosition();
                var reviveCheckPosition = new Vector2(playerHeadPosition.x, Math.min(camera.position.y, playerHeadPosition.y));

                // keep track of player progress
                if (isPlaying) {
                    levelPercent = Utils.clamp(0, (camera.position.y - startPlayerY) / (endPlayerY - startPlayerY), 1);
                }


                // keep track of respawn point while alive
                if (!player.getIsDead()) {
                    var revivePoints = Bento.objects.getByFamily('revivePoints');
                    Utils.forEach(revivePoints, function (revivePoint, i, l, breakLoop) {
                        //has to be above the last to change
                        if (revivePoint.position.y <= currentRevivePoint.y) {
                            if (revivePoint.position.distance(reviveCheckPosition) <= currentRevivePoint.distance(reviveCheckPosition)) {
                                currentRevivePoint = revivePoint.position.clone();
                            }
                        }
                    });
                }


                // kill player when they fall off the screen
                if (isPlaying && !player.getIsDead() && playerHeadPosition.y > viewport.y + viewport.height + 8) {
                    player.die();
                }

                // move camera
                if (canScrollCamera && camera) {
                    if (playerHeadPosition.y < camera.position.y + (SkinManager.getCurrentPlantSkin() === 'growbot' ? 16 : 32)) {
                        camera.setTargetPosition(new Vector2(
                            viewport.width * 0.5,
                            playerHeadPosition.y - (SkinManager.getCurrentPlantSkin() === 'growbot' ? 16 : 32)
                        ));
                    }
                    // move camera up slowly in special stage
                    if (getIsSpecialStage()) {
                        camera.setTargetPosition(new Vector2(
                            viewport.width * 0.5,
                            camera.getTargetPosition().y - 0.6
                        ));
                    }
                    //set max height
                    camera.setTargetPosition(new Vector2(camera.getTargetPosition().x, Math.max(camera.getTargetPosition().y, endPlayerY)));
                }
            },
            destroy: function () {
                EventSystem.off('onFeverChanged', onFeverChanged);
                EventSystem.off('onLevelComplete', onLevelComplete);
                EventSystem.off('onDeath', onDeath);
            }
        };

        // cheats for devs
        if (Utils.isDev()) {
            window.goToLevel = function (level) {
                var levelProgress = SaveManager.load('levelProgress');
                for (var i = SaveManager.load('levelProgressCount'); i < Math.min(level, LevelManager.getLevelCount()); i++) {
                    if (levelProgress["lvl_" + i] === "locked") {
                        levelProgress["lvl_" + i] = "unlocked";
                        console.log('LevelManager: Level ' + i + ' Unlocked!');
                    }
                }
                SaveManager.save('levelProgress', levelProgress);
                Globals.gameState = Globals.gameStates.menu;
                LevelManager.changeLevel(level || thisLevel);
            };
            window.skipLevel = function (skipscore) {
                Globals.currentScore = skipscore || Globals.currentScore;
                onLevelComplete();
                return "Level " + thisLevel + " Skipped!";
            };
            window.killPlayer = function () {
                player.die();
                return "Player Killed!";
            };
        }

        // --- ENTITY ---
        var entity = new Entity({
            name: 'gameController',
            position: new Vector2(0, 0),
            float: true,
            components: [
                behaviour,
                new BackButton({
                    onPressed: function (data) {
                        if (isPlaying) {
                            new PauseScreen({});
                        }
                    }
                })
            ]
        }).extend({
            startLevel: startLevel,
            setIsPlaying: function (newIsPlaying) {
                isPlaying = newIsPlaying;
            },
            getIsSpecialStage: getIsSpecialStage,
            setCanScrollCamera: function (newCanScroll) {
                canScrollCamera = newCanScroll;
            },
            giveCoins: function (amount) {
                if (gui) {
                    var tracker = gui.getCoinTracker();
                    tracker.updateCoinValue(tracker.getCoinValue() + amount);
                }
                coinsCollected += amount;
                coinsToCashIn += amount;
            },
            markCoinCollected: function (coinIdentifier) {
                if (markedCoins.indexOf(coinIdentifier) === -1) {
                    markedCoins.push(coinIdentifier);
                }
            },
            getCoinsCollected: function () {
                return coinsCollected;
            },
            giveScore: function (amount) {
                Globals.currentScore += amount;
            },
            getLevelPercent: function () {
                return levelPercent;
            }
        });
        return entity;
    };
});