/**
 * Module description
 * @moduleName SuperChest
 * @snippet SuperChest.snippet
SuperChest({
    position: new Vector2(0, 0)
})
 */
bento.define('entities/superchest', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'components/culler',
    'components/spritecontainer',
    'entities/particle',
    'components/movable'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    Culler,
    SpriteContainer,
    Particle,
    Movable
) {
    'use strict';
    return function (settings) {
        // --- PARAMETERS ---
        var startPosition = settings.position;
        var canPulse = true;
        var canLoop = true;

        var pulseLoop = function () {
            new Tween({ in: 60,
                onComplete: function () {
                    if (canPulse) {
                        new Particle({
                            z: Globals.layers.tiles - 1,
                            imageName: 'ui/fx/bloom-halo',
                            position: entity.position.add(chestSpriteContainer.position),
                            scale: Globals.pixelScaleV.scalarMultiply(0.9),
                            removeAfterTime: 20,
                            originRelative: new Vector2(0.5, 0.5),
                            removeEffect: 'scalefade'
                        });
                    }
                    if (canLoop) {
                        pulseLoop();
                    }
                }
            });
        };

        // --- ENTITY ---
        var chestSpriteContainer = new SpriteContainer({
            imageName: 'chest',
            originRelative: new Vector2(0.5, 0.5),
            scale: Globals.pixelScaleV.clone()
        });

        var entity = new Entity({
            z: Globals.layers.active,
            name: 'chest',
            family: ['chests'],
            position: startPosition.clone(),
            boundingBox: new Rectangle(-12, -9, 24, 18),
            components: [
                chestSpriteContainer,
                new Culler({
                    height: 12
                }), {
                    name: "effectBehaviour",
                    start: function () {
                        pulseLoop();
                    },
                    update: function () {
                        chestSpriteContainer.scale.x = (1 + Math.sin((entity.ticker) * 0.1) * 0.1) * Globals.pixelScale * 0.8;
                        chestSpriteContainer.scale.y = (1 + Math.sin((entity.ticker + 23) * 0.137) * 0.1) * Globals.pixelScale * 0.8;
                        chestSpriteContainer.rotation = Math.sin((entity.ticker + 77) * 0.07) * 0.1;
                        chestSpriteContainer.position.x = Math.sin((entity.ticker + 56) * 0.02) * 2;
                        chestSpriteContainer.position.y = Math.sin((entity.ticker + 19) * 0.04) * 2;
                    },
                    destroy: function (data) {
                        canLoop = false;
                        canPulse = false;
                        if (!Globals.showEffects) {
                            return;
                        }
                        new Particle({
                            z: Globals.layers.effects,
                            spriteSheet: 'effects/coin',
                            position: entity.position.clone(),
                            alpha: 1,
                            scale: Globals.pixelScaleV.scalarMultiply(1.5),
                            rotation: 0,
                            velocity: new Vector2(0, 0),
                            gravity: 0.05
                        });
                        new Tween({ in: 8,
                            onComplete: function () {
                                new Particle({
                                    z: Globals.layers.tiles - 1,
                                    imageName: 'ui/fx/bloom-halo',
                                    position: entity.position.clone(),
                                    scale: Globals.pixelScaleV.scalarMultiply(0.9),
                                    removeAfterTime: 10,
                                    originRelative: new Vector2(0.5, 0.5),
                                    removeEffect: 'scalefade'
                                });
                            }
                        });
                    },

                    draw: function (data) {
                        if (Globals.debug) {
                            var bb = entity.boundingBox;
                            data.renderer.fillRect([1, 0.75, 0, 0.5], bb.x, bb.y, bb.width, bb.height);
                        }
                    }
                }
            ]
        });
        return entity;
    };
});