/**
 * AB test manager
 * @moduleName ABTest
 */
bento.define('modules/abtest', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'modules/savemanager'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    SaveManager
) {
    'use strict';

    /*
        --- DO NOT FORGET TO ADD THESE KEYS (as well as with 'Set' on the end, though that'll change soon) TO SAVEMANAGER ---
    */
    var AB1Key = 'ABUXOptimizations1';
    var AB1_DimensionA = "ABUXOptimizations1_Original";
    var AB1_DimensionB = "ABUXOptimizations1_New";
    var AB1;

    var AB2Key = '';
    var AB2_DimensionA = "";
    var AB2_DimensionB = "";
    var AB2;

    var ABTest = {
        init: function (onComplete) {
            // set A/B tests
            // AB - 1
            if (AB1Key) {
                if (!SaveManager.load(AB1Key + 'Set')) {
                    AB1 = ["A", "B"][Utils.getRandom(2)];
                    SaveManager.save(AB1Key, AB1);
                    SaveManager.save(AB1Key + 'Set', true);
                } else {
                    AB1 = SaveManager.load(AB1Key);
                }
                //set dimensions for tests
                // AB - 1
                var dimension01 = "";
                if (AB1 === "A") {
                    dimension01 = AB1_DimensionA;
                }
                if (AB1 === "B") {
                    dimension01 = AB1_DimensionB;
                }
                EventSystem.fire('GameAnalytics-setCustomDimension01', dimension01);
                EventSystem.fire("FirebaseAnalytics-setUserProperty", {
                    property: "ABTest1",
                    value: dimension01
                });
                console.log(AB1Key + ':', AB1, dimension01);
            }


            // AB - 2
            if (AB2Key) {
                if (!SaveManager.load(AB2Key + 'Set')) {
                    AB2 = ["A", "B"][Utils.getRandom(2)];
                    SaveManager.save(AB2Key, AB2);
                    SaveManager.save(AB2Key + 'Set', true);
                } else {
                    AB2 = SaveManager.load(AB2Key);
                }

                // AB - 2
                var dimension02 = "";
                if (AB2 === "A") {
                    dimension02 = AB2_DimensionA;
                }
                if (AB2 === "B") {
                    dimension02 = AB2_DimensionB;
                }
                EventSystem.fire('GameAnalytics-setCustomDimension02', dimension02);
                EventSystem.fire("FirebaseAnalytics-setUserProperty", {
                    property: "ABTest2",
                    value: dimension02
                });
                console.log(AB2Key + ':', AB2, dimension02);
            }

            // set id if unset
            if (!SaveManager.load('identifier')) {
                var userId;
                // use default methods
                if (window.device) {
                    userId = 'dvc_' + window.device.uuid;
                } else if (navigator.isCocoonJS && window.Cocoon && window.Cocoon.Device) {
                    userId = 'ccd_' + window.Cocoon.Device.getDeviceId();
                } else {
                    // last resort: randomize number
                    userId = 'rnd_' + (1 + Utils.getRandom(100000000));
                }
                if (Utils.isDev()) {
                    userId = 'dev';
                }
                SaveManager.save('identifier', userId);
            }


            if (onComplete) {
                onComplete();
            }
        },
        getAB1: function () {
            return AB1;
        },
        getAB2: function () {
            return AB2;
        },
        getAB1Key: function () {
            return AB1Key;
        },
        getAB2Key: function () {
            return AB2Key;
        },
        getCustomDimensions01: function () {
            return [AB1_DimensionA, AB1_DimensionB];
        },
        getCustomDimensions02: function () {
            return [AB2_DimensionA, AB2_DimensionB];
        }
    };

    return ABTest;
});