/**
 * Module description
 * @moduleName ByteDanceDeepConversion
 * @snippet ByteDanceDeepConversion.snippet
ByteDanceDeepConversion({})
 */
bento.define('modules/bytedancedeepconversion', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween
) {
    'use strict';
    // ------ FUNCTIONS ------
    var nothing = function () {
        // do nothing
    };
    var error = function (err) {
        if (!err) {
            return;
        }
        console.log('ByteDance DeepConversion Error: ', err);
    };

    var ByteDanceDeepConversionModule = {
        initialize: nothing,
        registerEvent: nothing,
        purchaseEvent: nothing
    };
    var ByteDanceDeepConversion;
    if (window.cordova && window.cordova.plugins && window.cordova.plugins.BDDeepConversion) {
        ByteDanceDeepConversion = window.cordova.plugins.BDDeepConversion;

        ByteDanceDeepConversionModule.initialize = function (options) {
            ByteDanceDeepConversion.initSDK({
                    appID: options.appID,
                    appName: options.appName,
                    channel: options.channel
                },
                function () {
                    console.log('Deep Conversion SDK initialized!');
                },
                function (e) {
                    console.log('Deep Conversion SDK initialization failed:', e);
                }
            );
        };

        ByteDanceDeepConversionModule.registerEvent = function (options, success, failure) {
            ByteDanceDeepConversion.registerEvent({
                    method: options.method,
                    isSuccess: options.isSuccess
                },
                success,
                failure
            );
        };

        ByteDanceDeepConversionModule.purchaseEvent = function (options, success, failure) {
            ByteDanceDeepConversion.purchaseEvent({
                    contentType: options.contentType,
                    contentName: options.contentName,
                    contentID: options.contentID,
                    contentNumber: options.contentNumber,
                    paymentChannel: options.paymentChannel,
                    currency: options.currency,
                    currency_amount: options.currency_amount,
                    isSuccess: options.isSuccess
                },
                success,
                failure
            );
        };
    }

    return ByteDanceDeepConversionModule;
});