/**
 * Module description
 * @moduleName CloudSaveManager
 * @snippet CloudSaveManager.snippet
CloudSaveManager({
    position: new Vector2(0, 0)
})
 */
bento.define('modules/cloudsavemanager', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'modules/cloudsave',
    'modules/savemanager',
    'modules/skinmanager',
    'modules/levelmanager',
    'globals'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    CloudSave,
    SaveManager,
    SkinManager,
    LevelManager,
    Globals
) {
    'use strict';

    var verbose = true;

    var doSave = function (onComplete) {
        CloudSave.saveToCloud(function (saveSuccess, errorMsg) {
            if (!saveSuccess) {
                console.log('CloudSave: FAILED - ' + errorMsg);
            } else {
                console.log('CloudSave: SUCCESS');
            }
            onComplete(saveSuccess);
        });
    };

    var doLoad = function (onComplete) {
        CloudSave.loadFromCloud(function (loadSuccess, errorMsg) {
            if (!loadSuccess) {
                console.log('CloudLoad: FAILED - ' + errorMsg);
            } else {
                console.log('CloudLoad: SUCCESS');
            }
            onComplete(loadSuccess);
        });
    };

    var CloudSaveManager = {};
    CloudSaveManager.checkCloudNewer = function (onComplete) {
        CloudSave.hasAnySave(function (success, data, cloudSave) {
            if (success) {
                if (verbose && cloudSave) {
                    console.log('CLOUDSAVE FOUND: ' + JSON.stringify(cloudSave));
                }
                if (!cloudSave || cloudSave['levelsBeaten'] > SaveManager.load('levelsBeaten')) { // the cloud save is old, save over it
                    onComplete(true);
                } else {
                    onComplete(false);
                }
            } else {
                onComplete(false);
            }
        });
    };
    CloudSaveManager.save = function (onComplete, force) {
        if (force) {
            doSave(onComplete);
        } else {
            CloudSaveManager.checkCloudNewer(function (isNewer) {
                if (isNewer) {
                    console.log('CloudSave: FAILED - Cloud is newer than local');
                    onComplete(false);
                } else {
                    doSave(onComplete);
                }
            });
        }
    };
    CloudSaveManager.load = function (onComplete, force) {
        if (force) {
            doLoad(onComplete);
        } else {
            CloudSaveManager.checkCloudNewer(function (isNewer) {
                if (isNewer) {
                    doLoad(onComplete);
                } else {
                    console.log('CloudLoad: FAILED - Cloud is older than local');
                    onComplete(false);
                }
            });
        }
    };
    CloudSaveManager.resetup = function () {
        SkinManager.initialize();
        LevelManager.initialize();
        Globals.gameState = Globals.gameStates.menu;
        Bento.screens.show('screens/main');
    };

    return CloudSaveManager;
});