/**
 * Module description
 * @moduleName DailyRewardManager
 * @snippet DailyRewardManager.snippet
DailyRewardManager({
    position: new Vector2(0, 0)
})
 */
bento.define('modules/dailyrewardmanager', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'modules/timemanager',
    'modules/localization',
    'modules/skinmanager'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    TimeManager,
    Localization,
    SkinManager
) {
    'use strict';
    // --- VARS ---
    var maxStreak = 7;
    var today;
    var lastDayCollected;
    var dayStreak;
    var weekName;
    var canCollectReward = false;

    var weeksRewards;
    var todaysReward = {
        id: null,
        name: '',
        value: null,
        canDouble: false
    };

    // --- REWARDS ---
    var weekOrder;
    var rewards;

    // --- FUNCTIONS ---

    var migrateSave = function () {
        var weekStreak = Bento.saveState.load('dailyLoginWeekStreak', 0);
        var completeWeeks = Bento.saveState.load('dailyLoginCompletedWeeks', []);
        if (weekStreak > 0 && completeWeeks.indexOf("week_1") === -1) {
            completeWeeks.push("week_1");
        }
        if (weekStreak > 1 && completeWeeks.indexOf("week_2") === -1) {
            completeWeeks.push("week_2");
        }
        Bento.saveState.save('dailyLoginCompletedWeeks', completeWeeks);
        Bento.saveState.save('dailyLoginLastWeekName', Bento.saveState.load('dailyLoginLastWeekName', "none"));
    };

    var getThisWeekName = function () {
        var options = [];
        var completeWeeks = Bento.saveState.load('dailyLoginCompletedWeeks', []);
        Utils.forEach(weekOrder.slice(), function (item, i, l, breakLoop) {
            if (completeWeeks.indexOf(item) === -1) {
                options.push(item);
            }
        });
        return options[0] || 'default';
    };

    var checkDailyReward = function () {
        today = Math.floor(TimeManager.getCurrentLocalTime() / 1000 / 60 / 60 / 24);
        lastDayCollected = Bento.saveState.load('dailyLoginLastDayCollected', 0);
        dayStreak = Bento.saveState.load('dailyLoginDayStreak', 0);
        migrateSave();
        weekName = getThisWeekName();

        // is this a new day
        if (today > lastDayCollected) {
            // we can collect our reward
            canCollectReward = true;
            // if we reached the max last collection, reset
            if (dayStreak >= maxStreak) {
                //reset the day streak
                dayStreak = 0;
                Bento.saveState.save('dailyLoginCurrentStreak', dayStreak);

                //complete this week
                var completeWeeks = Bento.saveState.load('dailyLoginCompletedWeeks', []);
                if (completeWeeks.indexOf(weekName) === -1) {
                    completeWeeks.push(weekName);
                }
                Bento.saveState.save('dailyLoginCompletedWeeks', completeWeeks);
                //get new week
                weekName = getThisWeekName();
            }
        }

        //if we just changed week
        if (weekName !== Bento.saveState.load('dailyLoginLastWeekName', "none")) {
            //reset the day streak
            dayStreak = 0;
            canCollectReward = true;
            Bento.saveState.save('dailyLoginCurrentStreak', dayStreak);
        }

        //get todays reward
        weeksRewards = rewards[weekName];
        todaysReward = weeksRewards[dayStreak + 1] || todaysReward;

        //save stuff
        Bento.saveState.save('dailyLoginLastWeekName', weekName);

        //print stuff
        console.log('Daily Reward Initialized! can we collect a reward ', canCollectReward);
        console.log('Current week:' + weekName);

        //cancel event
        EventSystem.off('serverTimeUpdated', checkDailyReward);
    };

    // --- MODULE ---
    var DailyRewardManager = {
        initialize: function () {
            //set up rewards now
            weekOrder = [
                "week_1",
                "week_2",
                "halloween"
            ];
            rewards = {
                "default": {
                    "1": {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "25"),
                        value: 25,
                        canDouble: true
                    },
                    "2": {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "25"),
                        value: 25,
                        canDouble: true
                    },
                    "3": {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "35"),
                        value: 35,
                        canDouble: true
                    },
                    "4": {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "35"),
                        value: 35,
                        canDouble: true
                    },
                    "5": {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "50"),
                        value: 50,
                        canDouble: true
                    },
                    "6": {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "50"),
                        value: 50,
                        canDouble: true
                    },
                    "7": {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "100"),
                        value: 100,
                        canDouble: true
                    },
                },
                "halloween": {
                    "1": {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "25"),
                        value: 25,
                        canDouble: true
                    },
                    "2": SkinManager.isPotSkinUnlocked("trickytreat") ? {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "25"),
                        value: 25,
                        canDouble: true
                    } : {
                        id: 'pot',
                        name: Localization.getText('freePot'),
                        value: "trickytreat",
                        canDouble: false
                    },
                    "3": {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "35"),
                        value: 35,
                        canDouble: true
                    },
                    "4": SkinManager.isPotSkinUnlocked("cauldron") ? {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "35"),
                        value: 35,
                        canDouble: true
                    } : {
                        id: 'pot',
                        name: Localization.getText('freePot'),
                        value: "cauldron",
                        canDouble: false
                    },
                    "5": {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "50"),
                        value: 50,
                        canDouble: true
                    },
                    "6": {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "50"),
                        value: 50,
                        canDouble: true
                    },
                    "7": SkinManager.isPlantSkinUnlocked("spooki") ? {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "100"),
                        value: 100,
                        canDouble: true
                    } : {
                        id: 'plant',
                        name: Localization.getText('spooki-plant'),
                        value: "spooki",
                        canDouble: false
                    }
                },
                "week_1": {
                    "1": {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "25"),
                        value: 25,
                        canDouble: true
                    },
                    "2": SkinManager.isPotSkinUnlocked("lemontea") ? {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "25"),
                        value: 25,
                        canDouble: true
                    } : {
                        id: 'pot',
                        name: Localization.getText('freePot'),
                        value: "lemontea",
                        canDouble: false
                    },
                    "3": {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "35"),
                        value: 35,
                        canDouble: true
                    },
                    "4": SkinManager.isPotSkinUnlocked("bubbletea") ? {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "35"),
                        value: 35,
                        canDouble: true
                    } : {
                        id: 'pot',
                        name: Localization.getText('freePot'),
                        value: "bubbletea",
                        canDouble: false
                    },
                    "5": {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "50"),
                        value: 50,
                        canDouble: true
                    },
                    "6": {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "50"),
                        value: 50,
                        canDouble: true
                    },
                    "7": SkinManager.isPlantSkinUnlocked("potato") ? {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "100"),
                        value: 100,
                        canDouble: true
                    } : {
                        id: 'plant',
                        name: Localization.getText('potato-plant'),
                        value: "potato",
                        canDouble: false
                    }
                },
                "week_2": {
                    "1": {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "25"),
                        value: 25,
                        canDouble: true
                    },
                    "2": SkinManager.isPotSkinUnlocked("medicine") ? {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "25"),
                        value: 25,
                        canDouble: true
                    } : {
                        id: 'pot',
                        name: Localization.getText('freePot'),
                        value: "medicine",
                        canDouble: false
                    },
                    "3": {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "35"),
                        value: 35,
                        canDouble: true
                    },
                    "4": SkinManager.isPotSkinUnlocked("soymilk") ? {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "35"),
                        value: 35,
                        canDouble: true
                    } : {
                        id: 'pot',
                        name: Localization.getText('freePot'),
                        value: "soymilk",
                        canDouble: false
                    },
                    "5": {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "50"),
                        value: 50,
                        canDouble: true
                    },
                    "6": {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "50"),
                        value: 50,
                        canDouble: true
                    },
                    "7": SkinManager.isPlantSkinUnlocked("tomato") ? {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "100"),
                        value: 100,
                        canDouble: true
                    } : {
                        id: 'plant',
                        name: Localization.getText('tomato-plant'),
                        value: "tomato",
                        canDouble: false
                    }
                }
            };

            // do we have server time?
            if (TimeManager.hasUpdatedTime) {
                // yes? initialize
                checkDailyReward();
            } else {
                // no? wait for it
                EventSystem.on('serverTimeUpdated', checkDailyReward);
            }
        },
        canCollectReward: function () {
            return canCollectReward;
        },
        getWeekName: function () {
            return weekName;
        },
        getWeeksRewards: function () {
            return weeksRewards;
        },
        getTodaysReward: function () {
            return todaysReward;
        },
        getDayStreak: function () {
            return dayStreak;
        },
        collect: function (handleReward) {
            if (canCollectReward) {
                //we can't collect any more
                canCollectReward = false;

                //save that we collected today
                lastDayCollected = today;
                Bento.saveState.save('dailyLoginLastDayCollected', lastDayCollected);

                //up our streak
                dayStreak++;
                Bento.saveState.save('dailyLoginDayStreak', dayStreak);

                //create dialog for reward
                if (handleReward) {
                    handleReward(todaysReward);
                }

                // print stuff
                console.log('Daily Reward Collected!');
            }
        }

    };
    return DailyRewardManager;
});