/**
 * Component description
 * @moduleName FirebaseAnalytics
 * @snippet FirebaseAnalytics.logEvent
FirebaseAnalytics.logEvent("event_name",{
    property_name: "serializable_value"
});

 * @snippet FirebaseAnalytics.logEvent | EventSystem
EventSystem.fire("FirebaseAnalytics-logEvent", {
    name: "event_name",
    properties: {
        property_name: "serializable_value"
    }
});

 * @snippet FirebaseAnalytics.setScreenName
FirebaseAnalytics.setScreenName("screen_name");

 * @snippet FirebaseAnalytics.setScreenName | EventSystem
EventSystem.fire("FirebaseAnalytics-setScreenName", "screen_name");

 * @snippet FirebaseAnalytics.setUserId
FirebaseAnalytics.setUserId("user_id");

 * @snippet FirebaseAnalytics.setUserId | EventSystem
EventSystem.fire("FirebaseAnalytics-setUserId", "user_id");

 * @snippet FirebaseAnalytics.setUserProperty
FirebaseAnalytics.setUserProperty("property_name", "serializable_value");

 * @snippet FirebaseAnalytics.setUserProperty | EventSystem
EventSystem.fire("FirebaseAnalytics-setUserProperty", {
    property: "property_name",
    value: "serializable_value"
});
 */
bento.define('modules/firebaseanalytics', [
    'bento',
    'bento/eventsystem',
    'bento/utils',
], function (
    Bento,
    EventSystem,
    Utils
) {
    'use strict';
    var nothing = function () {
        //nothing     
    };
    var enabled = true;
    var FirebasePlugin = window.FirebasePlugin;

    //empty module
    var FirebaseAnalytics = {
        logEvent: nothing,
        setScreenName: nothing,
        setUserId: nothing,
        setUserProperty: nothing,
        init: nothing,
        setEnabled: function (newEnabled) {
            enabled = newEnabled;
        }
    };

    //return the empty module if plugin is missing
    if (!FirebasePlugin) {
        return FirebaseAnalytics;
    }

    // replace module methods since we have the plugin
    FirebaseAnalytics.logEvent = function (eventName, eventProperties) {
        FirebasePlugin.logEvent(eventName, eventProperties);
    };
    FirebaseAnalytics.setScreenName = function (screenName) {
        FirebasePlugin.setScreenName(screenName);
    };
    FirebaseAnalytics.setUserId = function (userId) {
        FirebasePlugin.setUserId(userId);
    };
    FirebaseAnalytics.setUserProperty = function (propertyName, propertyValue) {
        FirebasePlugin.setUserProperty(propertyName, propertyValue);
    };
    FirebaseAnalytics.init = function () {
        //register events
        EventSystem.on('FirebaseAnalytics-logEvent', function (data) {
            if (enabled) {
                FirebaseAnalytics.logEvent(data.name, data.properties);
            }
        });
        EventSystem.on('FirebaseAnalytics-setScreenName', function (data) {
            if (enabled) {
                FirebaseAnalytics.setScreenName(data);
            }
        });
        EventSystem.on('FirebaseAnalytics-setUserId', function (data) {
            if (enabled) {
                FirebaseAnalytics.setUserId(data);
            }
        });
        EventSystem.on('FirebaseAnalytics-setUserProperty', function (data) {
            if (enabled) {
                FirebaseAnalytics.setUserProperty(data.property, data.value);
            }
        });
        EventSystem.on('FirebaseAnalytics-setEnabled', function (data) {
            if (enabled) {
                FirebaseAnalytics.setEnabled(data);
            }
        });

        // console log
        console.log("Firebase Initialised!");
    };
    return FirebaseAnalytics;
});