/**
 * Module description
 * @moduleName LevelManager
 * @snippet LevelManager.snippet
LevelManager({})
 */
bento.define('modules/levelmanager', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'modules/savemanager',
    'ui/screentransition'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    SaveManager,
    ScreenTransition
) {
    'use strict';
    var levelCount = 1;
    var LevelManager = {
        initialize: function (options) {
            options = options || {};
            levelCount = options.levelCount || levelCount;

            //create blank save file if there is no data
            var currentLevel = SaveManager.load('currentLevel');
            var levelProgress = SaveManager.load('levelProgress');
            var levelProgressCount = SaveManager.load('levelProgressCount');
            levelProgress["lvl_1"] = levelProgress["lvl_1"] || "unlocked";
            for (var i = 0; i < levelCount || 0; i++) {
                levelProgress["lvl_" + (i + 1)] = levelProgress["lvl_" + (i + 1)] || "locked";
            }

            //set the minimum infinite mode level
            SaveManager.save('infiniteModeLevel', Math.max(SaveManager.load('infiniteModeLevel'), (levelCount + 1)));

            // not migrated progress from pre-levelmanager times, so migrate
            if (!SaveManager.load('hasMigratedLevelProgress')) {

                // unlock infinite mode
                if (currentLevel > levelCount) {
                    SaveManager.save('infiniteModeUnlocked', true);
                }

                //migrate current level to levelProgressData, it can't be higher than progresslevels
                for (i = 0; i < Math.min(currentLevel, levelCount); i++) {
                    levelProgress["lvl_" + (i + 1)] = "beaten";
                    levelProgressCount++;
                }
                // make only the current level unlocked
                levelProgress["lvl_" + Math.min(currentLevel, levelCount + 1)] = "unlocked";
                console.log('LevelManager: Migrated currentLevel to levelProgress');

                // migrate beatenlevels to levelProgress can't be lower than currentLevel -1
                SaveManager.save('levelsBeaten', SaveManager.load('currentLevel') - 1);
                console.log('LevelManager: Migrated currentLevel to levelsBeaten');

                // migration completed
                SaveManager.save('hasMigratedLevelProgress', true);
            }

            // currentLevel is in the middle of the locked levels (likely happens after new levels have been added)
            if (currentLevel <= levelCount) {
                if (levelProgress["lvl_" + currentLevel] === "locked") {
                    console.log('LevelManager: currentLevel is in the middle of locked levels, unlocking necessary new levels up to this point!');
                    for (var i = levelProgressCount; i <= currentLevel; i++) {
                        if (levelProgress["lvl_" + i] === "locked") {
                            levelProgress["lvl_" + i] = "unlocked";
                            console.log('LevelManager: Level ' + i + ' Unlocked!');
                        }
                    }
                }
            }

            SaveManager.save('levelProgress', levelProgress);
            SaveManager.save('levelProgressCount', levelProgressCount);

            console.log('LevelManager: currentLevel: ', SaveManager.load('currentLevel'));
            console.log('LevelManager: levelProgress: ', SaveManager.load('levelProgress'));
            console.log('LevelManager: levelProgressCount: ', SaveManager.load('levelProgressCount'));
            console.log('LevelManager: levelsBeaten: ', SaveManager.load('levelsBeaten'));
            console.log('LevelManager Initialized!');
        },
        getLevelProgress: function () {
            return SaveManager.load('levelProgress');
        },
        getLevelProgressCount: function () {
            return SaveManager.load('levelProgressCount');
        },
        getCurrentLevel: function () {
            return SaveManager.load('currentLevel');
        },
        getBeatenLevels: function () {
            return SaveManager.load('levelsBeaten');
        },
        getLevelState: function (level) {
            return SaveManager.load('levelProgress')["lvl_" + level];
        },
        getLevelCount: function () {
            return levelCount;
        },
        getInfiniteModeLevel: function () {
            return SaveManager.load('infiniteModeLevel');
        },
        levelComplete: function () {
            // get stuff
            var currentLevel = SaveManager.load('currentLevel');
            var levelProgress = SaveManager.load('levelProgress');
            var levelProgressCount = SaveManager.load('levelProgressCount');

            console.log('Level ' + currentLevel + ' Complete!');

            // if this level was locked or unlocked
            if (currentLevel <= levelCount) {

                if (levelProgress["lvl_" + currentLevel] === "unlocked") {
                    // it is now beaten
                    levelProgress["lvl_" + currentLevel] = "beaten";
                    // unlock the next level 
                    levelProgress["lvl_" + (currentLevel + 1)] = "unlocked";
                    SaveManager.add('levelProgressCount', 1);
                    console.log('LevelManager: Level ' + (currentLevel + 1) + ' Unlocked!');
                }

                // if we just beat a 'locked' level, unlock all the levels up to it
                if (levelProgress["lvl_" + currentLevel] === "locked") {
                    for (var i = levelProgressCount; i <= (currentLevel + 1); i++) {
                        if (levelProgress["lvl_" + i] === "locked") {
                            levelProgress["lvl_" + i] = "unlocked";
                            console.log('LevelManager: Level ' + i + ' Unlocked!');
                        }
                    }
                }
            }
            if (currentLevel === levelCount) {
                SaveManager.save('infiniteModeUnlocked', true);
            }
            if (currentLevel > levelCount) {
                SaveManager.add('infiniteModeLevel', 1);
            }
            SaveManager.save('levelProgress', levelProgress);

            SaveManager.add('currentLevel', 1);
            SaveManager.add('levelsBeaten', 1);
        },
        changeLevel: function (level, transitionPosition) {
            //changeLevel
            SaveManager.save('currentLevel', level);

            //unlocke this level
            var levelProgress = SaveManager.load('levelProgress');
            if (levelProgress["lvl_" + level] === "locked") {
                levelProgress["lvl_" + level] = "unlocked";
            }
            SaveManager.save('levelProgress', levelProgress);

            new ScreenTransition({
                destination: 'screens/main',
                position: transitionPosition
            });
        }
    };
    return LevelManager;
});