/**
 * Component description
 * @moduleName BytedanceAds
 * @snippet BytedanceAds.snippet
 */
bento.define('modules/toutiao', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween
) {
    'use strict';
    // ------ FUNCTIONS ------
    var nothing = function () {
        // do nothing
    };
    var error = function (err) {
        if (!err) {
            return;
        }
        console.log('Toutiao Ads Error: ' + err);
    };
    var prepareFullScreenVideoLoop = function () {
        var tween = new Entity({
            updateWhenPaused: 10000,
            name: 'tween',
            global: true,
            components: [
                new Tween.TweenBehavior({ in: 600,
                    updateWhenPaused: 10000,
                    onComplete: function () {
                        ToutiaoModule.prepareFullScreenVideo(nothing, prepareFullScreenVideoLoop);
                    }
                })
            ]
        });
        Bento.objects.attach(tween);
    };
    var prepareRewardedLoop = function () {
        var tween = new Entity({
            updateWhenPaused: 10000,
            name: 'tween',
            global: true,
            components: [
                new Tween.TweenBehavior({ in: 600,
                    updateWhenPaused: 10000,
                    onComplete: function () {
                        ToutiaoModule.prepareRewarded(nothing, prepareRewardedLoop);
                    }
                })
            ]
        });
        Bento.objects.attach(tween);
    };

    // --- VARS ---
    var hasFullScreenVideoAd = false;
    var isProcessingFullScreenVideo = false;
    var fullscreenVideoCodeID;
    var autoLoadFullScreenVideoAds = true;
    var fullscreenVideoLoadCallback;
    var fullscreenVideoLoadFailedCallback;
    var fullscreenVideoCompleteCallback;

    var hasRewardedAd = false;
    var isProcessingRewarded = false;
    var rewardedCodeID;
    var autoLoadRewardedAds = true;
    var rewardedLoadCallback;
    var rewardedLoadFailedCallback;
    var rewardedCompleteCallback;

    // --- MODULE ---
    var ToutiaoModule = {
        initialize: nothing,
        prepareFullScreenVideo: nothing,
        canShowFullScreenVideo: function () {
            return Utils.isDev();
        },
        showFullScreenVideo: nothing,
        prepareRewarded: nothing,
        canShowRewarded: function () {
            return Utils.isDev();
        },
        showRewarded: nothing
    };
    // Check if we can access iron source, if we can, get it ready
    if (window.cordova && window.cordova.plugins && window.cordova.plugins.toutiao) {
        var ToutiaoPlugin = window.cordova.plugins.toutiao;

        ToutiaoModule = {
            initialize: function (options, success, failure) {
                if (Utils.isDefined(options.autoLoadInteractionAds)) {
                    autoLoadFullScreenVideoAds = options.autoLoadFullScreenVideoAds;
                }
                if (Utils.isDefined(options.autoLoadRewardedAds)) {
                    autoLoadRewardedAds = options.autoLoadRewardedAds;
                }
                if (!options.appID) {
                    Utils.log('Toutiao Ads Initialization failed!', 'No App ID!');
                    return;
                }
                if (!options.appName) {
                    Utils.log('Toutiao Ads Initialization failed!', 'No App Name!');
                    return;
                }
                fullscreenVideoCodeID = options.fullscreenVideoCodeID;
                rewardedCodeID = options.rewardedCodeID;
                ToutiaoPlugin.initSDK({
                    appID: options.appID,
                    appName: options.appName,
                    textureView: true,
                    showNotificationBar: true,
                    debug: true,
                    showPageOnScreenLock: true,
                    titleBarTheme: 1
                }, function () {
                    console.log('Toutiao Ads Initialized!');
                    if (success) {
                        success();
                    }
                    if (autoLoadFullScreenVideoAds) {
                        prepareFullScreenVideoLoop();
                    }
                    if (autoLoadRewardedAds) {
                        prepareRewardedLoop();
                    }
                }, function (e) {
                    Utils.log('Toutiao Ads Initialization failed! Error: ', e);
                    if (failure) {
                        failure(e);
                    }
                });
            },
            prepareFullScreenVideo: function (success, failure) {
                console.log("Toutiao Ads: Preparing FullScreen Ad!");
                var screenSize = Utils.getScreenSize();
                var AdSize = new Vector2(screenSize.width, screenSize.height);

                fullscreenVideoLoadCallback = success;
                fullscreenVideoLoadFailedCallback = failure;
                ToutiaoPlugin.loadFullScreenVideoAd({
                    codeID: fullscreenVideoCodeID,
                    imageWidth: AdSize.x,
                    imageHeight: AdSize.y,
                    autoShow: true
                }, nothing, error);
            },
            canShowFullScreenVideo: function (yes, no) {
                if (hasFullScreenVideoAd && !isProcessingRewarded) {
                    if (yes) {
                        yes();
                    }
                } else {
                    if (no) {
                        no();
                    }
                }
            },
            showFullScreenVideo: function (onComplete, failure) {
                var onFailure = function (e) {
                    isProcessingFullScreenVideo = false;
                    if (failure) {
                        failure(e);
                    } else {
                        error(e);
                    }
                };

                if (hasFullScreenVideoAd && !isProcessingFullScreenVideo) {
                    isProcessingFullScreenVideo = true;
                    fullscreenVideoCompleteCallback = onComplete;
                    ToutiaoPlugin.showFullScreenVideoAd(nothing, onFailure);
                } else {
                    onFailure("No ad, or already watching an ad.");
                }
            },
            prepareRewarded: function (success, failure) {
                console.log("Toutiao Ads: Preparing Rewarded Ad!");
                var screenSize = Utils.getScreenSize();
                var AdSize = new Vector2(screenSize.width, screenSize.height);

                rewardedLoadCallback = success;
                rewardedLoadFailedCallback = failure;
                ToutiaoPlugin.loadRewardVideoAd({
                    codeID: rewardedCodeID,
                    imageWidth: AdSize.x,
                    imageHeight: AdSize.y,
                    rewardName: '',
                    amount: 0,
                    autoShow: false
                }, nothing, error);
            },
            canShowRewarded: function () {
                return hasRewardedAd && !isProcessingRewarded;
            },
            showRewarded: function (onComplete, failure) {
                var onFailure = function (e) {
                    isProcessingRewarded = false;
                    if (failure) {
                        failure(e);
                    } else {
                        error(e);
                    }
                };

                if (hasRewardedAd && !isProcessingRewarded) {
                    isProcessingRewarded = true;
                    rewardedCompleteCallback = onComplete;
                    ToutiaoPlugin.showRewardVideoAd(nothing, onFailure);
                }
            },
        };
    }

    //interaction callbacks
    document.addEventListener('onFullScreenVideoLoaded', function (data) {
        hasFullScreenVideoAd = true;
        if (fullscreenVideoLoadCallback) {
            fullscreenVideoLoadCallback(data);
        }
    }, false);
    document.addEventListener('onFullScreenVideoError', function (data) {
        hasFullScreenVideoAd = false;
        if (fullscreenVideoLoadFailedCallback) {
            fullscreenVideoLoadFailedCallback(data);
        }
    }, false);
    //Interstitial ad close Listener
    document.addEventListener('onFullScreenVideoClose', function (data) {
        // If we're waiting for a close
        if (isProcessingFullScreenVideo) {
            isProcessingFullScreenVideo = false;
            // fire success
            if (fullscreenVideoCompleteCallback) {
                fullscreenVideoCompleteCallback();
            }
        }
        hasFullScreenVideoAd = false;
        // prepare next interstitial
        if (autoLoadFullScreenVideoAds) {
            prepareFullScreenVideoLoop();
        }
    }, false);

    //rewarded callbacks
    document.addEventListener('onRewardVideoLoaded', function (data) {
        hasRewardedAd = true;
        if (rewardedLoadCallback) {
            rewardedLoadCallback(data);
        }
    }, false);
    document.addEventListener('onRewardVideoError', function (data) {
        switch (data.code) {
        case '20001': // ad failed to load!!!
            hasRewardedAd = false;
            if (rewardedLoadFailedCallback) {
                rewardedLoadFailedCallback(data);
                // prepare next rewardec
                if (autoLoadRewardedAds) {
                    prepareRewardedLoop();
                }
            }
            break;
        }
    }, false);
    //Interstitial ad close Listener
    document.addEventListener('onRewardVideoClose', function (data) {
        // if we're waiting for a close event
        if (isProcessingRewarded) {
            isProcessingRewarded = false;
        }
        hasRewardedAd = false;
        // prepare next rewarded
        if (autoLoadRewardedAds) {
            prepareRewardedLoop();
        }
    }, false);
    //Interstitial ad close Listener
    document.addEventListener('onRewardVideoComplete', function (data) {
        // if we're waiting for a close event
        if (isProcessingRewarded) {
            isProcessingRewarded = false;
            if (rewardedCompleteCallback) {
                rewardedCompleteCallback();
            }
        }
    }, false);


    return ToutiaoModule;
});