/**
 * Module description
 * @moduleName BonusWheel
 * @snippet BonusWheel.snippet
BonusWheel({
    position: new Vector2(0, 0)
})
 */
bento.define('ui/bonuswheel', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'components/spritecontainer',
    'entities/effects/sparkleparticle',
    'modules/skinmanager',
    'components/gamecounter',
    'modules/localization',
    'modules/taptic'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    SpriteContainer,
    Sparkle,
    SkinManager,
    GameCounter,
    Localization,
    TapticEngine
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var twoPi = Math.PI * 2;
        var degToRad = Math.PI / 180;

        // --- VARS ---
        var segments = 6;
        var thisSegment = 0;
        var lastSegment = 0;
        var angle = 0;
        var spinSpeed = 0.3;
        var blurLength = 2;
        var blurPasses = 2;
        var pointerTween;

        var adPots = SkinManager.getWheelPots();
        var adPot1;
        var adPot2;

        if (adPots.length > 0) {
            adPot1 = SkinManager.getNextWheelPot();
        }
        if (adPots.length > 1) {
            adPot2 = SkinManager.getNextWheelPot();
            while (adPot1 === adPot2) {
                adPot2 = SkinManager.getNextWheelPot();
            }
        }

        var items = [{
                id: 'coins~50',
                name: Localization.getText('XCoins').replace('{COUNT}', "50"),
                graphic: new Entity({
                    name: 'coins50',
                    components: [
                        new SpriteContainer({
                            imageName: 'ui/big_coin',
                            originRelative: new Vector2(0.5, 0.5),
                            scale: Globals.pixelScaleV.scalarMultiply(0.3)
                        }),
                        new GameCounter({
                            value: '+50',
                            imageName: 'addcoincounter',
                            scale: Globals.pixelScaleUIV.scalarMultiply(0.5),
                            position: new Vector2(0, 20)
                        })
                    ]
                })
            }, {
                id: 'coins~150',
                name: Localization.getText('XCoins').replace('{COUNT}', "150"),
                graphic: new Entity({
                    name: 'coins150',
                    components: [
                        new SpriteContainer({
                            imageName: 'ui/big_coin',
                            originRelative: new Vector2(0.5, 0.5),
                            scale: Globals.pixelScaleV.scalarMultiply(0.35)
                        }),
                        new GameCounter({
                            value: '+150',
                            imageName: 'addcoincounter',
                            scale: Globals.pixelScaleUIV.scalarMultiply(0.5),
                            position: new Vector2(0, 20)
                        })
                    ]
                })
            },
            // if we have an ad pot show it
            (adPot1) ? {
                id: 'pot~' + adPot1,
                name: Localization.getText('freePot'),
                graphic: new Entity({
                    name: adPot1,
                    components: [
                        new SpriteContainer({
                            spriteSheet: 'boxes/' + adPot1,
                            scale: Globals.pixelScaleV.scalarMultiply(1.5)
                        })
                    ]
                })
            } : {
                id: 'coins~75',
                name: Localization.getText('XCoins').replace('{COUNT}', "75"),
                graphic: new Entity({
                    name: 'coins75',
                    components: [
                        new SpriteContainer({
                            imageName: 'ui/big_coin',
                            originRelative: new Vector2(0.5, 0.5),
                            scale: Globals.pixelScaleV.scalarMultiply(0.325)
                        }),
                        new GameCounter({
                            value: '+75',
                            imageName: 'addcoincounter',
                            scale: Globals.pixelScaleUIV.scalarMultiply(0.5),
                            position: new Vector2(0, 20)
                        })
                    ]
                })
            }, {
                id: 'coins~100',
                name: Localization.getText('XCoins').replace('{COUNT}', "100"),
                graphic: new Entity({
                    name: 'coins100',
                    components: [
                        new SpriteContainer({
                            imageName: 'ui/big_coin',
                            originRelative: new Vector2(0.5, 0.5),
                            scale: Globals.pixelScaleV.scalarMultiply(0.325)
                        }),
                        new GameCounter({
                            value: '+100',
                            imageName: 'addcoincounter',
                            scale: Globals.pixelScaleUIV.scalarMultiply(0.5),
                            position: new Vector2(0, 20)
                        })
                    ]
                })
            }, {
                id: 'coins~200',
                name: Localization.getText('XCoins').replace('{COUNT}', "200"),
                graphic: new Entity({
                    name: 'coins200',
                    components: [
                        new SpriteContainer({
                            imageName: 'ui/big_coin',
                            originRelative: new Vector2(0.5, 0.5),
                            scale: Globals.pixelScaleV.scalarMultiply(0.4)
                        }),
                        new GameCounter({
                            value: '+200',
                            imageName: 'addcoincounter',
                            scale: Globals.pixelScaleUIV.scalarMultiply(0.5),
                            position: new Vector2(0, 22)
                        })
                    ]
                })
            },
            // if we have an ad pot here, show it
            (adPot2) ? {
                id: 'pot~' + adPot2,
                name: Localization.getText('freePot'),
                graphic: new Entity({
                    name: adPot2,
                    components: [
                        new SpriteContainer({
                            spriteSheet: 'boxes/' + adPot2,
                            scale: Globals.pixelScaleV.scalarMultiply(1.5)
                        })
                    ]
                })
            } : {
                id: 'coins~75',
                name: Localization.getText('XCoins').replace('{COUNT}', "75"),
                graphic: new Entity({
                    name: 'coins75',
                    components: [
                        new SpriteContainer({
                            imageName: 'ui/big_coin',
                            originRelative: new Vector2(0.5, 0.5),
                            scale: Globals.pixelScaleV.scalarMultiply(0.325)
                        }),
                        new GameCounter({
                            value: '+75',
                            imageName: 'addcoincounter',
                            scale: Globals.pixelScaleUIV.scalarMultiply(0.5),
                            position: new Vector2(0, 20)
                        })
                    ]
                })
            },
        ];

        // --- FUNCTIONS ---
        var spin = function (forcedSegment, onComplete) {
            spinSpeed = 0.3;
            // what segment are we aiming for
            var targetSegment = (forcedSegment === null || forcedSegment === undefined) ? Utils.getRandom(segments) : forcedSegment;
            // calculate total distance to rotate
            var totalAngle = (((targetSegment + (segments * 3)) * (twoPi / segments)) - (angle % twoPi)) + Utils.getRandomFloat(degToRad * 15);
            // calculate the deceleration arte to stop at that distance
            var decelerationRate = (spinSpeed * spinSpeed) / (2 * totalAngle);
            //calculate how long it will take
            var decelerationTime = spinSpeed / decelerationRate;

            //decelerate
            new Tween({ in: decelerationTime,
                ignoreGameSpeed: true,
                onUpdate: function (v, t) {
                    spinSpeed -= decelerationRate;
                },
                onComplete: function () {
                    //stop
                    spinSpeed = 0;
                    Bento.audio.stopSound('sfx_wheelsuccess');
                    Bento.audio.playSound('sfx_wheelsuccess');
                    // complete
                    if (onComplete) {
                        onComplete(items[thisSegment]);
                    }
                }
            });
        };
        var WheelItem = function (item) {
            var thisWheelItem = new Entity({
                name: item.id,
                position: new Vector2(0, 0),
                components: [
                    new Entity({
                        name: 'itemHolder',
                        position: new Vector2(0, -56),
                        components: [
                            item.graphic
                        ]
                    })
                ]
            });
            return thisWheelItem;
        };

        // --- COMPONENTS ---
        var spinBehaviour = {
            name: 'spinBehaviour',
            update: function () {
                angle += spinSpeed;
                thisSegment = Math.floor((angle + (30 * degToRad)) / (twoPi / segments)) % segments;
                if (entity.visible && lastSegment !== thisSegment) {
                    if (pointerTween) {
                        pointerTween.stop();
                    }
                    pointerTween = new Tween({
                        from: -0.5,
                        to: 0,
                        in: 15,
                        ease: 'easeOutExpo',
                        onUpdate: function (v, t) {
                            pointer.rotation = v;
                        }
                    });
                    entity.attach(new Sparkle({
                        position: pointer.position.add(new Vector2(0, 8)),
                        dontAttach: true
                    }));
                    TapticEngine.impact({
                        style: 'light'
                    });
                    Bento.audio.stopSound('sfx_wheeltick');
                    Bento.audio.playSound('sfx_wheeltick');
                }
                lastSegment = thisSegment;
                wheel.rotation = angle;
                wheelItems.rotation = angle;
            }
        };

        // --- ENTITY ---
        var pointer = new SpriteContainer({
            imageName: 'ui/pointer',
            originRelative: new Vector2(0.5, 0),
            position: new Vector2(0, -88),
            scale: Globals.pixelScaleUIV
        });

        var wheel = new SpriteContainer({
            imageName: 'ui/wheel',
            originRelative: new Vector2(0.5, 0.5),
            scale: Globals.pixelScaleUIV
        });
        var wheelBlur = {
            name: 'angularBlur',
            draw: function (data) {
                var preAngle = wheelItems.rotation;
                wheel.rotation -= spinSpeed * blurLength;
                wheel.alpha = 1 / (blurLength * blurPasses);
                for (var i = 0; i < (blurLength * blurPasses); i++) {
                    wheel.rotation += (spinSpeed / blurPasses);
                    wheel.draw(data);
                }
                wheel.alpha = 1;
                wheel.rotation = preAngle;
            }
        };
        var wheelItems = new Entity({
            name: 'wheelItems'
        });
        var wheelItemsBlur = {
            name: 'angularBlur',
            draw: function (data) {
                var preAngle = wheelItems.rotation;
                wheelItems.rotation -= spinSpeed * blurLength;
                wheelItems.alpha = 1 / (blurLength * blurPasses);
                for (var i = 0; i < (blurLength * blurPasses); i++) {
                    wheelItems.rotation += (spinSpeed / blurPasses);
                    wheelItems.draw(data);
                }
                wheelItems.alpha = 1;
                wheelItems.rotation = preAngle;
            }
        };
        // attach graphics to wheel
        Utils.forEach(items, function (item, i, l, breakLoop) {
            var wheelItem = new WheelItem(item);
            wheelItem.rotation = -(twoPi / segments) * i;
            wheelItems.attach(wheelItem);
        });

        var entity = new Entity({
            name: 'wheelContainer',
            position: settings.position || new Vector2(0, 0),
            components: [
                wheel,
                wheelBlur,
                wheelItems,
                wheelItemsBlur,
                spinBehaviour,
                pointer,
            ]
        }).extend({
            spin: spin
        });
        return entity;
    };
});