/**
 * Module description
 * @moduleName DailyRewardDialog
 * @snippet DailyRewardDialog.snippet
DailyRewardDialog({
    onComplete: function () {}
})
 */
bento.define('ui/dailyrewarddialog', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'ui/dialog',
    'components/spritecontainer',
    'entities/particle',
    'modules/localization',
    'ui/confetti',
    'modules/skinmanager',
    'modules/dailyrewardmanager',
    'components/gamecounter',
    'modules/ads',
    'ui/coindialog',
    'ui/potunlockeddialog',
    'ui/plantunlockeddialog'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    Dialog,
    SpriteContainer,
    Particle,
    Localization,
    Confetti,
    SkinManager,
    DailyRewardManager,
    GameCounter,
    Ads,
    CoinDialog,
    PotUnlockedDialog,
    PlantUnlockedDialog
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();

        // --- VARS ---
        var watchingAd = false;
        var animating = false;
        var weeksRewards = DailyRewardManager.getWeeksRewards();
        var dayStreak = DailyRewardManager.getDayStreak();
        var todaysReward = DailyRewardManager.getTodaysReward();
        var canDoubleCollect = true;
        var doDoubleScreen = todaysReward.canDouble && Ads.canShowRewarded();
        var onComplete = settings.onComplete || function () {};
        var mainMenuScreen = Bento.objects.get('mainMenuScreen');
        var doHalloween = (DailyRewardManager.getWeekName() === "halloween");
        // --- FUNCTIONS ---
        var DayBox = function (options) {
            var day = options.day || 0;
            var reward = weeksRewards[day];
            var displayIcon = {
                name: 'blank'
            };

            if (reward && reward.id && reward.id) {
                switch (reward.id) {
                case "coins":
                    displayIcon = new Entity({
                        name: 'displayIcon',
                        position: new Vector2(0, 6),
                        components: [
                            new SpriteContainer({
                                imageName: 'ui/big_coin',
                                originRelative: new Vector2(0.5, 0.5),
                                position: new Vector2(0, -6),
                                scale: Globals.pixelScaleUIV.scalarMultiply(0.2)
                            }),
                            new GameCounter({
                                value: parseInt(reward.value),
                                imageName: 'addcoincounter',
                                position: new Vector2(0, 12),
                                scale: new Vector2(0.075, 0.075),
                                align: 'center',
                                updateWhenPaused: 0
                            })
                        ]
                    });
                    break;
                case "pot":
                    displayIcon = new Entity({
                        name: 'displayIcon',
                        position: new Vector2(0, 4),
                        components: [
                            new SpriteContainer({
                                spriteSheet: 'boxes/' + reward.value,
                                position: new Vector2(0, 0),
                                scale: Globals.pixelScaleV.scalarMultiply(1.125)
                            })
                        ]
                    });
                    break;
                case "plant":
                    var spriteContainer = new SpriteContainer({
                        spriteSheet: 'beans/' + reward.value,
                        position: new Vector2(0, 0),
                        scale: Globals.pixelScaleV.scalarMultiply(1.125)
                    });
                    spriteContainer.sprite.setCurrentSpeed(0);
                    displayIcon = new Entity({
                        name: 'displayIcon',
                        position: new Vector2(0, 4),
                        components: [
                            spriteContainer
                        ]
                    });
                    break;
                }
            }

            var dayBox = new Entity({
                name: 'dayBox',
                position: options.position || new Vector2(0, 0),
                scale: options.scale || new Vector2(1, 1),
                components: [
                    new SpriteContainer({
                        imageName: 'ui/dailyreward/daybox',
                        originRelative: new Vector2(0.5, 0.5),
                        position: new Vector2(0, 0),
                        scale: Globals.pixelScaleUIV
                    }),
                    (day === dayStreak + 1) ? new SpriteContainer({
                        imageName: 'ui/dailyreward/daybox_outline',
                        originRelative: new Vector2(0.5, 0.5),
                        position: new Vector2(0, 0),
                        scale: Globals.pixelScaleUIV
                    }) : {
                        name: "blank"
                    },
                    new Text({
                        fontSettings: Utils.getTextStyle('dailyRewardDay'),
                        position: new Vector2(0, -13),
                        text: Localization.getText('dayX').replace('{NUMBER}', day.toString()),
                        maxWidth: 108,
                        maxHeight: 24
                    }),
                    (day <= dayStreak) ? new SpriteContainer({
                        imageName: 'ui/icons/tick',
                        originRelative: new Vector2(0.5, 0.5),
                        position: new Vector2(0, 6),
                        scale: Globals.pixelScaleUIV
                    }) : displayIcon
                ]
            });
            return dayBox;
        };

        var close = function () {
            dialog.close();
            animating = true;
            new Tween({
                from: rewardBox.position.y,
                to: rewardBox.position.y + 160,
                in: 15,
                ease: 'linear',
                onUpdate: function (v, t) {
                    rewardBox.position.y = v;
                },
                onComplete: function () {
                    rewardBox.visible = false;
                }
            });
        };

        var handleReward = function (doDouble) {
            mainMenuScreen = Bento.objects.get('mainMenuScreen');
            if (!Utils.isDefined(todaysReward.value) || !Utils.isDefined(todaysReward.id)) {
                return;
            }
            switch (todaysReward.id) {
            case "coins":
                var thisValue = (doDouble) ? (todaysReward.value * 2) : (todaysReward.value);
                Utils.giveCoins(thisValue);
                new CoinDialog({
                    numberOfCoins: thisValue,
                    onComplete: close
                });
                break;
            case "pot":
                var thisPot = todaysReward.value;
                SkinManager.makePotSkinUnlocked(thisPot);
                new PotUnlockedDialog({
                    potSkin: thisPot,
                    onComplete: close
                });
                if (mainMenuScreen) {
                    mainMenuScreen.potSelector.generateSlots();
                    mainMenuScreen.potSelector.changeSkin(thisPot);
                }
                break;
            case "plant":
                var thisPlant = todaysReward.value;
                SkinManager.makePlantSkinUnlocked(thisPlant);
                new PlantUnlockedDialog({
                    plantSkin: thisPlant,
                    onComplete: close
                });
                if (mainMenuScreen) {
                    mainMenuScreen.plantSelector.generateSlots();
                    mainMenuScreen.plantSelector.changeSkin(thisPlant);
                }
                break;
            }
        };

        // --- COMPONENTS ---
        var mainRoot = new Entity({
            name: 'mainRoot',
            position: new Vector2(0, -12),
            components: [
                new Text({
                    position: new Vector2(0, -48),
                    scale: new Vector2(1, 1),
                    maxWidth: 96,
                    text: todaysReward.name || '',
                    fontSettings: Utils.getTextStyle('dailyReward'),
                    align: 'center',
                    textBaseline: 'middle',
                    linebreaks: true,
                    maxHeight: 24
                })
            ]
        });

        var spacing = new Vector2(44, 38);
        var thisDayBox;

        //days 1-6
        for (var i = 0; i < 6; i++) {
            thisDayBox = new DayBox({
                day: (i + 1),
                position: new Vector2(-spacing.x + (spacing.x * (i % 3)), (spacing.y * Math.floor(i / 3)) - 16)
            });
            mainRoot.attach(thisDayBox);
        }
        //day 7
        thisDayBox = new DayBox({
            day: 7,
            position: new Vector2(0, 66),
            scale: new Vector2(1.25, 1.25)
        });
        mainRoot.attach(thisDayBox);

        var collectPanel = new ClickButton({
            name: 'collectButton',
            alpha: 1,
            sfx: 'sfx_clickbutton',
            imageName: 'ui/greenbutton',
            frameCountX: 1,
            frameCountY: 3,
            position: new Vector2(0, -60),
            scale: Globals.pixelScaleUIV.clone().scalarMultiply(2),
            updateWhenPaused: 0,
            float: false,
            onClick: function () {
                if (watchingAd || animating) {
                    return;
                }
                DailyRewardManager.collect(function () {
                    handleReward(false);
                });
            },
            components: [
                new Text({
                    position: new Vector2(0, 4),
                    scale: new Vector2(0.75, 0.75).scalarMultiply(1 / Globals.pixelScaleUI),
                    maxWidth: 60,
                    text: Localization.getText('collect'),
                    fontSettings: Utils.getTextStyle('collectButton'),
                    fontColor: '#fbffbe',
                    align: 'center',
                    textBaseline: 'middle',
                    linebreaks: true,
                    maxHeight: 14
                })
            ]
        });
        var doubleButton = new ClickButton({
            name: 'doubleButton',
            alpha: 1,
            sfx: 'sfx_clickbutton',
            imageName: 'ui/orangebutton',
            frameCountX: 1,
            frameCountY: 3,
            position: new Vector2(0, -68),
            scale: Globals.pixelScaleUIV.scalarMultiply(2),
            updateWhenPaused: 0,
            float: false,
            onClick: function () {
                if (watchingAd || animating) {
                    return;
                }
                //double reward
                if (Utils.isDev()) {
                    window.alert("This is a Rewarded Ad!");
                    Bento.input.resetPointers();
                    Globals.attemptsSinceAd = 0;
                    DailyRewardManager.collect(function () {
                        handleReward(true);
                    });
                } else {
                    watchingAd = true;
                    canDoubleCollect = false;
                    doubleButton.setActive(false);
                    Ads.showRewarded(function () {
                        Globals.attemptsSinceAd = 0;
                        DailyRewardManager.collect(function () {
                            handleReward(true);
                        });
                        watchingAd = false;
                    }, function (e) {
                        canDoubleCollect = true;
                        doubleButton.setActive(Ads.canShowRewarded());
                        watchingAd = false;
                    }, "DoubleDailyReward");
                }
            },
            components: [
                new SpriteContainer({
                    imageName: 'ui/icons/vid',
                    originRelative: new Vector2(0.5, 0.5),
                    position: new Vector2(0, -24),
                    scale: new Vector2(1, 1).scalarMultiply(0.4)
                }),
                new Text({
                    position: new Vector2(0, 48),
                    scale: new Vector2(0.66, 0.66).scalarMultiply(1 / Globals.pixelScaleUI),
                    maxWidth: 96,
                    text: Localization.getText('getdouble'),
                    fontSettings: Utils.getTextStyle('respin'),
                    fontSize: 14,
                    fontColor: '#fbffbe',
                    align: 'center',
                    textBaseline: 'middle',
                    linebreaks: true,
                    maxHeight: 28
                })
            ]
        });
        var collectButton = new Entity({
            name: 'collectButton',
            position: new Vector2(0, -20),
            boundingBox: new Rectangle(-32, -15, 64, 40),
            components: [
                new Clickable({
                    onClick: function (data) {
                        if (canDoubleCollect && this.parent.active) {
                            if (watchingAd || animating) {
                                return;
                            }
                            collectButton.setActive(false);
                            doubleButton.setActive(false);
                            DailyRewardManager.collect(function () {
                                handleReward(false);
                            });
                        }
                    }
                }),
                new Text({
                    position: new Vector2(0, 0),
                    scale: new Vector2(1.55, 1.55),
                    maxWidth: 54,
                    text: Localization.getText('collect'),
                    fontSettings: Utils.getTextStyle('respin'),
                    fontSize: 14,
                    fontColor: '#fbffbe',
                    align: 'center',
                    textBaseline: 'middle',
                    linebreaks: true,
                    maxHeight: 28
                })
            ]
        }).extend({
            active: true,
            setActive: function (newActive) {
                collectButton.active = newActive;
            }
        });
        var doublePanel = new Entity({
            name: 'doublePanel',
            components: [
                doubleButton,
                collectButton
            ]
        });

        var rewardBox = new Entity({
            name: 'rewardBox',
            position: new Vector2(viewport.width * 0.5, viewport.height),
            visible: true,
            components: [
                (doDoubleScreen) ? doublePanel : collectPanel
            ]
        });

        // --- ENTITY ---
        var dialog = new Dialog({
            titleText: Localization.getText('dailyrewards'),
            type: '',
            components: [mainRoot],
            hidebuttons: true,
            attach: true,
            delay: 15,
            bgScale: new Vector2(1, 1.125),
            offset: new Vector2(0, -32)
        });
        dialog.attach(rewardBox);

        if (doHalloween) {
            dialog.bgAnchor.attach(new SpriteContainer({
                position: new Vector2(viewport.width * 0.2, viewport.height * 0.1),
                originRelative: new Vector2(0.5, 0.5),
                scale: new Vector2(0.4, 0.4),
                imageName: 'ui/spiderweb'
            }));
            dialog.bgAnchor.attach(new SpriteContainer({
                position: new Vector2(viewport.width * 0.8, viewport.height * 0.75),
                originRelative: new Vector2(0.4, 0.4),
                scale: new Vector2(0.4, 0.4),
                rotation: 0.0,
                imageName: 'ui/spiderweb'
            }));
        }

        animating = true;
        new Tween({
            from: rewardBox.position.y + 160,
            to: rewardBox.position.y,
            in: 30,
            ease: 'easeOutQuad',
            onUpdate: function (v, t) {
                rewardBox.position.y = v;
            },
            onComplete: function () {
                animating = false;
            }
        });
        if (doDoubleScreen) {
            doubleButton.setActive(Ads.canShowRewarded());
            collectButton.alpha = 0;
            canDoubleCollect = false;
            new Tween({
                from: collectButton.alpha,
                to: 1,
                in: 30,
                delay: 180,
                ease: 'linear',
                onUpdate: function (v, t) {
                    collectButton.alpha = v;
                },
                onComplete: function () {
                    canDoubleCollect = true;
                }
            });
        }

        return dialog;
    };
});