/**
 * Module description
 * @moduleName DoubleCoinDialog
 * @snippet DoubleCoinDialog.snippet
DoubleCoinDialog({
    numberOfCoins: 1
})
 */
bento.define('ui/doublecoindialog', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'ui/dialog',
    'components/spritecontainer',
    'entities/particle',
    'modules/localization',
    'components/gamecounter',
    'modules/ads'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    Dialog,
    SpriteContainer,
    Particle,
    Localization,
    GameCounter,
    Ads
) {
    'use strict';
    return function (settings) {

        // --- VARS ---
        var viewport = Bento.getViewport();
        var completed = false;
        var numberOfCoins = settings.numberOfCoins || 1;
        var onComplete = settings.onComplete || function () {};
        var watchingAd = false;
        var canDouble = true;

        // --- FUNCTIONS ---
        var openAnimation = function () {
            new Tween({ in: 5,
                onComplete: function (v, t) {
                    for (var i = 0; i < 15; i++) {
                        mainRoot.attach(new Particle({
                            imageName: 'sparkle',
                            position: new Vector2(0, 0),
                            alpha: 1,
                            scale: Globals.pixelScaleV.scalarMultiply(3),
                            rotation: Utils.getRandomRangeFloat(-2, 2),
                            rotationRate: 0,
                            velocity: new Vector2(Utils.getRandomRangeFloat(-3, 3), Utils.getRandomRangeFloat(-3, 0)),
                            acceleration: new Vector2(0, 0.025),
                            friction: 1,
                            removeAfterTime: 100,
                            removeEffect: 'scale',
                            dontAttach: true,
                            z: Globals.layers.modals + 0.1
                        }));
                    }
                }
            });
            new Tween({
                from: 1,
                to: 0.85,
                in: 20,
                delay: 30,
                ease: 'easeOutBack',
                onUpdate: function (v, t) {
                    coinSprite.scale = Globals.pixelScaleUIV.clone().scalarMultiply(v);
                },
                onComplete: function () {

                }
            });
            new Tween({
                from: 0,
                to: numberOfCoins,
                in: 30,
                ease: 'easeOutQuad',
                onUpdate: function (v, t) {
                    coinSprite.scale = Globals.pixelScaleUIV.clone().scalarMultiply(0 + (v / numberOfCoins));
                    coinCounter.setValue('+' + Math.round(v));
                },
                onComplete: function () {
                    dialog.showButtons();
                }
            });

            collectButton.alpha = 0;
            collectButton.setActive(false);
            new Tween({
                from: 0,
                to: 1,
                in: 30,
                delay: 120,
                ease: 'linear',
                onUpdate: function (v, t) {
                    collectButton.alpha = v;
                },
                onComplete: function () {
                    collectButton.setActive(true);
                }
            });

            Bento.audio.stopSound('sfx_coinburst2');
            Bento.audio.playSound('sfx_coinburst2');
        };
        var getStandard = function () {
            if (!completed) {
                completed = true;
                burstAnimation(numberOfCoins, false);
            }
        };
        var getDouble = function () {
            if (!completed) {
                completed = true;
                burstAnimation(numberOfCoins * 2, true);
                Utils.giveCoins(numberOfCoins);
            }
        };
        var burstAnimation = function (coins, double) {
            var doBursts = function () {
                //particles
                var bursts = 10;
                var burstsRemaining = bursts;
                var coinsPerBurst = (Math.min(coins, 100)) / bursts;
                var burstLoop = function () {
                    new Tween({ in: 5,
                        onComplete: function () {
                            for (i = 0; i < coinsPerBurst; i++) {
                                dialog.attach(new Particle({
                                    spriteSheet: 'coin',
                                    position: new Vector2(viewport.width * 0.5, viewport.height * 0.5).add(new Vector2(Utils.getRandomRange(-8, 8), Utils.getRandomRange(-8, 8))),
                                    alpha: 1,
                                    scale: Globals.pixelScaleV,
                                    rotation: (Math.PI * 4 * (burstsRemaining / bursts)) + Utils.getRandomRangeFloat(-0.5, 0.5),
                                    rotationRate: Utils.getRandomRangeFloat(-0.1, 0.1),
                                    velocity: new Vector2(0, Utils.getRandomRangeFloat(-3, 0)).rotateRadian((Math.PI * 4 * (burstsRemaining / bursts)) + Utils.getRandomRangeFloat(-0.5, 0.5)).add(new Vector2(0, -1.5)),
                                    acceleration: new Vector2(0, 0.1),
                                    friction: 1,
                                    removeAfterTime: 300,
                                    removeEffect: 'scale',
                                    dontAttach: true,
                                    z: Globals.layers.modals + 0.1
                                }));
                            }
                            for (var i = 0; i < coinsPerBurst; i++) {
                                dialog.attach(new Particle({
                                    imageName: 'sparkle',
                                    position: new Vector2(viewport.width * 0.5, viewport.height * 0.5),
                                    alpha: 1,
                                    scale: Globals.pixelScaleV.scalarMultiply(3),
                                    rotation: (Math.PI * 4 * (burstsRemaining / bursts)) + Utils.getRandomRangeFloat(-0.5, 0.5),
                                    rotationRate: 0,
                                    velocity: new Vector2(0, Utils.getRandomRangeFloat(-3, 0)).rotateRadian((Math.PI * 4 * (burstsRemaining / bursts)) + Utils.getRandomRangeFloat(-0.5, 0.5)).add(new Vector2(0, -1.5)),
                                    acceleration: new Vector2(0, 0.1),
                                    friction: 1,
                                    removeAfterTime: 100,
                                    removeEffect: 'scale',
                                    dontAttach: true,
                                    z: Globals.layers.modals + 0.1
                                }));
                            }
                            burstsRemaining--;
                            if (burstsRemaining > 0) {
                                burstLoop();
                            }
                        }
                    });
                };
                burstLoop();

                // bloom halo burst outwards
                new Tween({
                    from: 0,
                    to: 1,
                    in: 30,
                    ease: 'easeOutQuad',
                    onUpdate: function (v, t) {
                        bloomHalo.scale.x = bloomHalo.scale.y = 0.75 * v;
                        bloomHalo.alpha = (1 - v) * 0.6;
                    },
                });
                // bloom grow and shrink
                new Tween({
                    from: 0,
                    to: 0.555,
                    in: 45,
                    ease: 'easeOutQuad',
                    onUpdate: function (v, t) {
                        bloomHard.scale.x = v;
                        bloomHard.scale.y = v;
                    },
                    onComplete: function () {
                        new Tween({
                            from: 0.555,
                            to: 0,
                            in: 45,
                            ease: 'easeInCubic',
                            onUpdate: function (v, t) {
                                bloomHard.scale.x = v;
                                bloomHard.scale.y = v;
                            },
                            onComplete: function () {
                                dialog.close();
                            }
                        });
                    }
                });
            };

            doubleButton.setActive(false);
            new Tween({
                from: doubleButton.scale.clone(),
                to: new Vector2(0, 0),
                in: 15,
                ease: 'easeInBack',
                onUpdate: function (v, t) {
                    doubleButton.scale = v;
                }
            });
            collectButton.setActive(false);
            new Tween({
                from: 1,
                to: 0,
                in: 15,
                ease: 'linear',
                onUpdate: function (v, t) {
                    collectButton.alpha = v;
                }
            });

            new Tween({
                from: 1,
                to: 0,
                in: 5,
                delay: 0,
                ease: 'easeInQuad',
                onUpdate: function (v, t) {
                    coinSprite.scale = Globals.pixelScaleUIV.clone().scalarMultiply(v);
                },
                onComplete: function () {
                    coinSprite.removeSelf();
                }
            });
            new Tween({
                from: 1,
                to: 0,
                in: 5,
                delay: 0,
                ease: 'easeInQuad',
                onUpdate: function (v, t) {
                    counterBacker.scale = Globals.pixelScaleUIV.clone().scalarMultiply(v);
                },
                onComplete: function () {
                    counterBacker.removeSelf();
                }
            });
            new Tween({
                from: coinCounter.position.y,
                to: 12,
                in: 15,
                delay: 0,
                ease: 'easeOutBack',
                onUpdate: function (v, t) {
                    coinCounter.position.y = v;
                }
            });
            if (double) {
                new Tween({
                    from: coinCounter.scale.clone(),
                    to: coinCounter.scale.scalarMultiply(1.5),
                    in: 30,
                    delay: 0,
                    ease: 'easeOutBack',
                    onUpdate: function (v, t) {
                        coinCounter.scale = v;
                    }
                });
                var lastCoins = numberOfCoins;
                new Tween({
                    from: numberOfCoins,
                    to: coins,
                    in: 15,
                    delay: 0,
                    ease: 'easeOutBack',
                    onUpdate: function (v, t) {
                        if (v !== lastCoins) {
                            coinCounter.setValue('+' + Math.round(v));
                            lastCoins = v;
                        }
                    },
                    onComplete: function () {
                        doBursts();
                    }
                });
            } else {
                new Tween({
                    from: coinCounter.scale.clone(),
                    to: coinCounter.scale.scalarMultiply(1.1),
                    in: 30,
                    delay: 0,
                    ease: 'easeOutBack',
                    onUpdate: function (v, t) {
                        coinCounter.scale = v;
                    }
                });
                new Tween({ in: 15,
                    onComplete: function () {
                        doBursts();
                    }
                });
            }
        };

        // --- COMPONENTS ---
        var counterBacker = new SpriteContainer({
            imageName: 'ui/scoreholder',
            position: new Vector2(0, 0),
            originRelative: new Vector2(0.5, 0.5),
            alpha: 1,
            scale: Globals.pixelScaleUIV,
            rotation: 0
        });
        var coinSprite = new SpriteContainer({
            imageName: 'ui/big_coin',
            position: new Vector2(0, -20),
            originRelative: new Vector2(0.5, 0.5),
            alpha: 1,
            scale: Globals.pixelScaleUIV,
            rotation: 0
        });
        var coinCounter = new GameCounter({
            imageName: "ui/scorecounter",
            position: coinSprite.position.add(new Vector2(0, 48)),
            value: 0,
            scale: Globals.pixelScaleUIV
        });

        var doubleButton = new ClickButton({
            name: 'doubleButton',
            alpha: 1,
            sfx: 'sfx_clickbutton',
            imageName: 'ui/orangebutton',
            frameCountX: 1,
            frameCountY: 3,
            position: new Vector2(0, 68),
            scale: Globals.pixelScaleUIV.scalarMultiply(2),
            updateWhenPaused: 0,
            float: false,
            onClick: function () {
                if (!canDouble || watchingAd) {
                    return;
                }
                //double reward
                if (Utils.isDev()) {
                    window.alert("This is a Rewarded Ad!");
                    Bento.input.resetPointers();
                    Globals.attemptsSinceAd = 0;
                    getDouble();
                } else {
                    watchingAd = true;
                    doubleButton.setActive(false);
                    Ads.showRewarded(function () {
                        Globals.attemptsSinceAd = 0;
                        getDouble();
                        watchingAd = false;
                    }, function (e) {
                        doubleButton.setActive(Ads.canShowRewarded());
                        watchingAd = false;
                    }, "DoubleGoldRushCoins");
                }
            },
            components: [
                new SpriteContainer({
                    imageName: 'ui/icons/vid',
                    originRelative: new Vector2(0.5, 0.5),
                    position: new Vector2(-84, 0),
                    scale: new Vector2(1, 1).scalarMultiply(0.5)
                }),
                new Text({
                    position: new Vector2(68, 0),
                    scale: new Vector2(0.66, 0.66).scalarMultiply(1 / Globals.pixelScaleUI),
                    text: Localization.getText('getdouble'),
                    fontSettings: Utils.getTextStyle('respin'),
                    fontSize: 14,
                    fontColor: '#fbffbe',
                    align: 'center',
                    textBaseline: 'middle',
                    linebreaks: true,
                    maxWidth: 48,
                    maxHeight: 28
                })
            ]
        });
        doubleButton.setActive(Ads.canShowRewarded());

        var collectButton = new Entity({
            name: 'collectButton',
            position: new Vector2(0, 116),
            boundingBox: new Rectangle(-32, -15, 64, 40),
            components: [
                new Clickable({
                    onClick: function (data) {
                        if (canDouble && this.parent.active) {
                            if (watchingAd) {
                                return;
                            }
                            collectButton.setActive(false);
                            doubleButton.setActive(false);
                            getStandard();
                        }
                    }
                }),
                new Text({
                    position: new Vector2(0, 0),
                    scale: new Vector2(1.35, 1.35),
                    maxWidth: 54,
                    text: Localization.getText('collect'),
                    fontSettings: Utils.getTextStyle('respin'),
                    fontSize: 14,
                    fontColor: '#fbffbe',
                    align: 'center',
                    textBaseline: 'middle',
                    linebreaks: true,
                    maxHeight: 28
                })
            ]
        }).extend({
            active: true,
            setActive: function (newActive) {
                collectButton.active = newActive;
            }
        });

        var bloomHard = new SpriteContainer({
            imageName: 'ui/fx/bloom-hard',
            originRelative: new Vector2(0.5, 0.5),
            scale: new Vector2(0, 0)
        });
        var bloomHalo = new SpriteContainer({
            imageName: 'ui/fx/bloom-halo',
            originRelative: new Vector2(0.5, 0.5),
            scale: new Vector2(0, 0),
            alpha: 0.6
        });
        var mainRoot = new Entity({
            name: 'mainRoot',
            position: new Vector2(0, -12),
            components: [
                bloomHard,
                bloomHalo,
                counterBacker,
                coinSprite,
                coinCounter,
                doubleButton,
                collectButton
            ]
        });

        // --- ENTITY ---
        var dialog = new Dialog({
            titleText: Localization.getText('youGotCoins'),
            type: 'none',
            onYes: function (thisDialog) {
                if (!completed) {
                    completed = true;
                    burstAnimation();
                    dialog.hideButtons();
                }
            },
            onComplete: onComplete,
            components: [mainRoot],
            hidebuttons: true,
            attach: true
        });

        openAnimation();


        return dialog;
    };
});