/**
 * Module description
 * @moduleName LevelNode
 * @snippet LevelNode.snippet
LevelNode({
    position: new Vector2(0, 0)
})
 */
bento.define('ui/levelnode', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'modules/levelmanager',
    'components/spritecontainer',
    'globals',
    'components/sortedclickable',
    'modules/savemanager'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    LevelManager,
    SpriteContainer,
    Globals,
    SortedClickable,
    SaveManager
) {
    'use strict';
    // map node constructor
    return function (settings) {

        // --- PARAMETERS ---
        var pos = settings.position || new Vector2(0, 0);
        var levelNumber = settings.levelNumber || 1;
        var onSelect = settings.onSelect || function () {};
        var isInfinite = settings.isInfinite || false;
        var levelState = LevelManager.getLevelState(levelNumber);

        // --- COMPONENTS ---
        // 'stuffs'
        var lockedStuff = new Entity({
            name: 'lockedStuff',
            position: new Vector2(0, 0),
            components: [
                new SpriteContainer({
                    imageName: 'ui/sagamap/level_locked',
                    originRelative: new Vector2(0.5, 0.5),
                    scale: Globals.pixelScaleUIV
                }),
                new Text({
                    fontSettings: Utils.getTextStyle('levelNodeLocked'),
                    position: new Vector2(0, 0),
                    text: levelNumber.toString(),
                    maxWidth: 32,
                    maxHeight: 32
                }),
                new SpriteContainer({
                    imageName: 'ui/sagamap/levellock',
                    position: new Vector2(8, 12),
                    originRelative: new Vector2(0.5, 0.5),
                    scale: Globals.pixelScaleUIV
                })
            ]
        });

        // the most current level
        var unlockedStuff = new Entity({
            name: 'unlockedStuff',
            position: new Vector2(0, 0),
            boundingBox: new Rectangle(-12, -12, 24, 24),
            components: [
                new SpriteContainer({
                    imageName: 'ui/sagamap/level_locked',
                    originRelative: new Vector2(0.5, 0.5),
                    scale: Globals.pixelScaleUIV
                }),
                new Text({
                    fontSettings: Utils.getTextStyle('levelNodeLocked'),
                    position: new Vector2(0, 0),
                    text: levelNumber.toString(),
                    maxWidth: 32,
                    maxHeight: 32
                }),
                new SortedClickable({
                    onClick: onSelect
                })
            ]
        });

        //already played levels
        var beatenStuff = new Entity({
            name: 'beatenStuff',
            position: new Vector2(0, 0),
            boundingBox: new Rectangle(-12, -12, 24, 24),
            components: [
                new SpriteContainer({
                    imageName: 'ui/sagamap/level_unlocked',
                    originRelative: new Vector2(0.5, 0.5),
                    scale: Globals.pixelScaleUIV
                }),
                new Text({
                    fontSettings: Utils.getTextStyle('levelNodeUnlocked'),
                    position: new Vector2(0, 0),
                    text: levelNumber.toString(),
                    maxWidth: 32,
                    maxHeight: 32
                }),
                new SortedClickable({
                    onClick: onSelect
                })
            ]
        });

        //already played levels
        var infiniteStuffUnlocked = new Entity({
            name: 'beatenStuff',
            position: new Vector2(0, 0),
            boundingBox: new Rectangle(-12, -12, 24, 24),
            components: [
                new SpriteContainer({
                    imageName: 'ui/sagamap/level_unlocked',
                    originRelative: new Vector2(0.5, 0.5),
                    scale: Globals.pixelScaleUIV
                }),
                new SpriteContainer({
                    imageName: 'ui/icons/infinity',
                    originRelative: new Vector2(0.5, 0.5),
                    scale: Globals.pixelScaleUIV
                }),
                new SortedClickable({
                    onClick: onSelect
                })
            ]
        });
        var infiniteStuffLocked = new Entity({
            name: 'beatenStuff',
            position: new Vector2(0, 0),
            boundingBox: new Rectangle(-12, -12, 24, 24),
            components: [
                new SpriteContainer({
                    imageName: 'ui/sagamap/level_locked',
                    originRelative: new Vector2(0.5, 0.5),
                    scale: Globals.pixelScaleUIV
                }),
                new SpriteContainer({
                    imageName: 'ui/icons/infinity',
                    originRelative: new Vector2(0.5, 0.5),
                    scale: Globals.pixelScaleUIV
                }),
                new SpriteContainer({
                    imageName: 'ui/sagamap/levellock',
                    position: new Vector2(8, 12),
                    originRelative: new Vector2(0.5, 0.5),
                    scale: Globals.pixelScaleUIV
                })
            ]
        });

        // --- ENTITY ---
        var levelNode = new Entity({
            name: 'levelNode',
            position: pos,
            components: []
        }).extend({
            getLevelNumber: function () {
                return levelNumber;
            }
        });

        // --- SETUP ---
        // get stat

        // attach state stuff
        if (!isInfinite) {
            switch (levelState) {
            case 'locked':
                levelNode.attach(lockedStuff);
                break;
            case 'unlocked':
                levelNode.attach(unlockedStuff);
                break;
            case 'beaten':
                levelNode.attach(beatenStuff);
                break;
            }
        } else {
            if (SaveManager.load('infiniteModeUnlocked')) {
                levelNode.attach(infiniteStuffUnlocked);
            } else {
                levelNode.attach(infiniteStuffLocked);
            }
        }

        return levelNode;
    };
});