/**
 * Module description
 * @moduleName MainMenuScreen
 * @snippet MainMenuScreen.snippet
MainMenuScreen({
    position: new Vector2(0, 0)
})
 */
bento.define('ui/mainmenuscreen', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'components/spritecontainer',
    'ui/cheatscreen',
    'modules/gameanalytics',
    'modules/tenjin',
    'ui/potselector',
    'ui/coindialog',
    'ui/adforcoinbutton',
    'ui/plantselector',
    'modules/savemanager',
    'ui/beandialog',
    'components/sortedclickable',
    'ui/tutorialscreen',
    'modules/cloudsave',
    'modules/skinmanager',
    'modules/vipmanager',
    'ui/vipdialog',
    'ui/settings',
    'modules/crosspromo/crosspromo',
    'ui/askrating',
    'ui/viptab',
    'ui/iapoverlay',
    'ui/shoptab',
    'ui/noadsbutton',
    'ui/viptestdrivedialog',
    'modules/ads',
    'ui/superchestbutton',
    'ui/screentransition',
    'ui/worldselector',
    'modules/levelmanager',
    'modules/dailyrewardmanager',
    'ui/dailyrewarddialog',
    'modules/cloudsavemanager',
    'ui/dialog',
    'modules/localization',
    'modules/firebaseanalytics'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    SpriteContainer,
    CheatScreen,
    GameAnalytics,
    Tenjin,
    PotSelector,
    CoinDialog,
    AdForCoinbutton,
    PlantSelector,
    SaveManager,
    BeanDialog,
    SortedClickable,
    TutorialScreen,
    CloudSave,
    SkinManager,
    VipManager,
    VipDialog,
    SettingsScreen,
    CrossPromo,
    AskRating,
    VipTab,
    IapOverlay,
    ShopTab,
    NoAdsButton,
    VipTestDrive,
    Ads,
    SuperChestButton,
    ScreenTransition,
    WorldSelector,
    LevelManager,
    DailyRewardManager,
    DailyRewardDialog,
    CloudSaveManager,
    Dialog,
    Localization,
    FirebaseAnalytics
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var levelProgressCount = LevelManager.getLevelProgressCount();
        var plantSelectorStatusLast;
        var potSelectorStatusLast;
        var levelSelectorStatusLast;
        var isVipLast;

        // --- SETUP ---
        Utils.unlockLevelSkins();
        if (!SaveManager.load('canShowPlantSelector') && SkinManager.getAllUnlockablePlantSkins() > 0 && SkinManager.getAllUnlockedPlantSkins() > 1) {
            SaveManager.save('canShowPlantSelector', true);
        }

        // --- VARS ---
        var gc;
        var gameGUI;
        var hasStarted = false;
        var isPlantTutorial = !SaveManager.load('doneGrowTutorial') && SaveManager.load('canShowPlantSelector');

        // --- FUNCTIONS ---
        var playGame = function () {
            if (!gameGUI) {
                gameGUI = Bento.objects.get('gameGUI');
                if (!gameGUI) {
                    return;
                }
            }
            if (!hasStarted) {
                hasStarted = true;
                Globals.gameState = Globals.gameStates.game;
                gc.startLevel();
                if (gameGUI) {
                    gameGUI.showScore();
                }
                new Tween({
                    from: entity.alpha,
                    to: 0,
                    in: 15,
                    delay: 0,
                    applyOnDelay: 0,
                    ease: 'easeOutQuad',
                    onUpdate: function (v, t) {
                        entity.alpha = v;
                    },
                    onComplete: function () {
                        entity.removeSelf();
                    }
                });
            }
        };

        // --- COMPONENTS ---
        var playTutorial = new TutorialScreen({
            isMenu: true,
            doAttach: false
        });
        var playClickable = new Entity({
            name: 'playClickable',
            position: new Vector2(0, 0),
            boundingBox: new Rectangle(0, 0, viewport.width, viewport.height),
            components: [
                new SortedClickable({
                    onPressed: function (data) {
                        if (playClickable.active && entity.timer > 60) {
                            playGame();
                            playTutorial.hide();
                            ClickButton.currentlyPressing = null;
                        }
                    }
                })
            ]
        }).extend({
            active: true,
            setActive: function (newActive) {
                playClickable.active = newActive;
            }
        }).attach({
            name: 'activeBehaviour',
            update: function () {
                playTutorial.alpha = (playClickable.active) ? 1 : 0;
            }
        });

        var behaviour = {
            name: "behaviour",
            start: function () {
                if (!gameGUI) {
                    gameGUI = Bento.objects.get('gameGUI');
                    if (!gameGUI) {
                        return;
                    }
                }
            },
            update: function () {
                if (!gc) {
                    gc = Bento.objects.get('gameController');
                    if (!gc) {
                        return;
                    }
                }
                if (!gameGUI) {
                    gameGUI = Bento.objects.get('gameGUI');
                    if (!gameGUI) {
                        return;
                    }
                    if (!hasStarted) {
                        gameGUI.hideScore(true);
                    }
                } else {
                    if (!hasStarted) {
                        gameGUI.hideScore(true);
                    }
                }

                isPlantTutorial = !SaveManager.load('doneGrowTutorial') && SaveManager.load('canShowPlantSelector');

                var plantSelectorStatus = (plantSelector.getOpenStatus() !== "closed");
                var potSelectorStatus = (potSelector.getOpenStatus() !== "closed");
                var levelSelectorStatus = (levelSelector.getOpenStatus() !== "closed");
                var isVip = VipManager.isVip();

                if (plantSelectorStatus !== plantSelectorStatusLast || potSelectorStatus !== potSelectorStatusLast || levelSelectorStatus !== levelSelectorStatusLast || isVipLast !== isVip) {
                    if (isVip) {
                        vipTab.hideTab();
                    }
                    if (plantSelectorStatus || potSelectorStatus || levelSelectorStatus) {
                        vipTab.hideTab();
                        potSelector.hideTab();
                        plantSelector.hideTab();
                        levelSelector.hideTab();
                        shopTab.hideTab();
                    } else {
                        if (!isVip) {
                            vipTab.showTab();
                        }
                        potSelector.showTab();
                        plantSelector.showTab();
                        levelSelector.showTab();
                        shopTab.showTab();
                    }
                }

                plantSelectorStatusLast = plantSelectorStatus;
                potSelectorStatusLast = potSelectorStatus;
                levelSelectorStatusLast = levelSelectorStatus;
                isVipLast = isVip;
            },
            destroy: function () {
                entity.remove(crosspromo);
            }
        };

        // settings button
        var settingsButton = new ClickButton({
            name: 'settingsButton',
            sfx: 'sfx_clickbutton',
            imageName: 'ui/config',
            frameCountX: 1,
            frameCountY: 1,
            position: new Vector2(19, 21),
            scale: new Vector2(0.15, 0.15),
            updateWhenPaused: 0,
            float: false,
            sort: true,
            onButtonDown: function () {
                new Tween({
                    from: 0,
                    to: Math.PI,
                    in: 15,
                    ease: 'easeOutQuad',
                    onUpdate: function (v, t) {
                        settingsButton.rotation = -v;
                    }
                });
                new Tween({
                    from: settingsButton.scale.clone(),
                    to: new Vector2(0.2, 0.2),
                    in: 15,
                    ease: 'easeOutQuad',
                    onUpdate: function (v, t) {
                        settingsButton.scale = v;
                    }
                });
            },
            onClick: function () {
                settingsScreen.open();
            },
            onButtonUp: function () {
                new Tween({
                    from: settingsButton.scale.clone(),
                    to: new Vector2(0.15, 0.15),
                    in: 15,
                    ease: 'easeInOutBack',
                    onUpdate: function (v, t) {
                        settingsButton.scale = v;
                    }
                });
            }
        });

        var noAdsButton = new NoAdsButton({
            position: new Vector2(24, 72)
        });

        // Pot Selector
        var potSelector = new PotSelector({});

        // Plant Selector
        var plantSelector = new PlantSelector({});

        // VIP Tab
        var vipTab = new VipTab({
            onComplete: function (isNowVip) {
                if (isNowVip) {
                    plantSelector.open();
                }
            }
        });

        //Shop Tab
        var shopTab = new ShopTab({});

        // Level Selector
        var levelSelector = new WorldSelector({});

        var settingsScreen = new SettingsScreen({});

        var askForPassword = function () {
            var success = function (text) {
                if (text === 'thunderdogs4evar') {
                    SaveManager.save('isDev', true);
                    GameAnalytics.disable();
                    Tenjin.disable();
                    FirebaseAnalytics.setEnabled(false);
                    new CheatScreen({});
                }
            };
            Utils.promptText('???', '???', success);
        };
        var cheatTaps = 0;
        var cheatRegion = new Entity({
            name: 'cheatRegion',
            position: new Vector2(0, 0),
            boundingBox: new Rectangle(viewport.width - 48, 0, 48, 32),
            components: [
                new SortedClickable({
                    onClick: function (data) {
                        cheatTaps++;
                        if (cheatTaps === 8) {
                            if (SaveManager.load('isDev')) {
                                new CheatScreen({});
                            } else {
                                askForPassword();
                            }
                            cheatTaps = 0;
                        }
                    },
                    onClickMiss: function (data) {
                        cheatTaps = 0;
                    }
                })
            ]
        });

        var crosspromo;
        var crosspromoSize = (viewport.height / 320) * 70;
        if (!window.ISCHINA) {
            crosspromo = Globals.currentCrossPromo || new CrossPromo({
                z: Globals.layers.mainmenu - 1
            });
            crosspromo.setSize(new Rectangle(0, 0, crosspromoSize, crosspromoSize));
            crosspromo.position = new Vector2(viewport.width - (crosspromoSize * 0.5), (crosspromoSize * 0.5) + 55);
            crosspromo.rotation = -0.05;
            Globals.currentCrossPromo = crosspromo;
        }

        // --- ENTITY ---
        var entity = new Entity({
            z: Globals.layers.mainmenu,
            name: 'mainMenuScreen',
            family: [''],
            position: new Vector2(0, 0),
            updateWhenPaused: 0,
            float: true,
            components: [
                behaviour,
                playClickable,
                playTutorial,
                cheatRegion,
                settingsButton,
                (levelProgressCount >= 2 && !window.ISCHINA) ? noAdsButton : {
                    name: "blank"
                },
                SaveManager.load('canShowPlantSelector') ? vipTab : {
                    name: "blank"
                },
                (levelProgressCount >= 2) ? shopTab : {
                    name: "blank"
                },
                levelSelector,
                (levelProgressCount >= 2) ? potSelector : {
                    name: "blank"
                },
                (SaveManager.load('canShowPlantSelector') || levelProgressCount >= 5) ? plantSelector : {
                    name: "blank"
                },
                settingsScreen
            ]
        }).extend({
            potSelector: potSelector,
            plantSelector: plantSelector,
            playButton: playClickable
        });

        VipManager.checkVip(function (isVip) {
            if (isVip) {
                if (SaveManager.load('VipFirstTimeClaim') === false) {
                    SaveManager.save('VipFirstTimeClaim', true);
                }
            } else {
                if (SkinManager.isUsingVipSkin()) {
                    SkinManager.setCurrentPlantSkin('default');
                    plantSelector.focusSkin('default');
                }
            }
        });
        if (!VipManager.isVip() && SkinManager.isUsingVipSkin()) {
            SkinManager.setCurrentPlantSkin('default');
            plantSelector.focusSkin('default');
        }

        var postTransition = function () {

            EventSystem.fire("FirebaseAnalytics-setScreenName", "main_menu");

            //show crosspromo
            if (crosspromo && !window.ISCHINA && Globals.currentCrossPromo && CrossPromo.canShow()) {
                entity.attach(crosspromo);
                new Tween({
                    from: 0,
                    to: crosspromo.scale.x,
                    in: 15,
                    delay: 30,
                    ease: 'easeOutBack',
                    onUpdate: function (v, t) {
                        crosspromo.scale.x = v;
                        crosspromo.scale.y = crosspromo.scale.x;
                    }
                });
                crosspromo.scale.x = 0;
                crosspromo.scale.y = 0;
            }

            //check cloud saves
            if (Utils.isNativeIos()) {
                CloudSaveManager.checkCloudNewer(function (isNewer) {
                    if (isNewer) {
                        //show a popup!!!
                        new Dialog({
                            titleText: Localization.getText('cloud'),
                            bodyText: Localization.getText('loadFromCloud'),
                            bgScale: new Vector2(1, 0.66),
                            type: 'yesno',
                            onYes: function (dialog) {
                                Bento.saveState.save('hasAskedCloudSave', true);
                                CloudSaveManager.load(function (didLoad) {
                                    if (didLoad) {
                                        CloudSaveManager.resetup();
                                    }
                                }, true);
                            },
                            onNo: function (dialog) {
                                Bento.saveState.save('hasAskedCloudSave', true);
                                dialog.close();
                            },
                            components: [],
                            attach: true
                        });
                    } else {
                        //overwrite sicne the cloud is older
                        CloudSaveManager.save(function () {}, true);
                    }
                });
            }

            if (levelProgressCount > 4 && DailyRewardManager.canCollectReward()) {
                //show daily reward
                new DailyRewardDialog({
                    onComplete: function () {}
                });
                return;
            }

            // open plants
            if (Globals.doOpenPlants) {
                plantSelector.open();
                Globals.doOpenPlants = false;
                return;
            }

            //show vip
            if (levelProgressCount < 5 || VipManager.isVip()) {
                Globals.levelsSinceVIP = 0;
            } else {
                if (Globals.doOpenVIP || (!Globals.doOpenPlants && Globals.levelsSinceVIP > 2)) {
                    Globals.levelsSinceVIP = 0;
                    Globals.doOpenVIP = false;
                    if (SaveManager.load('TimesShownVIP') % 2 === 1 || !Ads.canShowRewarded()) {
                        new VipDialog({
                            onComplete: function (isNowVip) {
                                if (isNowVip) {
                                    plantSelector.open();
                                }
                            }
                        });
                        return;
                    } else {
                        new VipTestDrive({
                            onYes: function (newSkin) {
                                EventSystem.fire('onPlantChanged', newSkin);
                                Globals.doOpenVIP = true;
                                playGame();
                            },
                            onNo: function () {},
                        });
                    }
                    SaveManager.add('TimesShownVIP', 1);
                    return;
                }
            }

            if (levelProgressCount > 7 && AskRating.canAsk()) {
                // ask for a review
                if (Utils.isNativeIos()) {
                    AskRating.askIos10();
                    SaveManager.save('askedRating', true);
                } else if (Utils.isNativeAndroid() || Bento.isDev()) {
                    // use old review funnel
                    new AskRating({
                        z: Globals.layers.modals,
                        onComplete: function () {
                            SaveManager.save('askedRating', true);
                        }
                    });
                }
                return;
            }
        };

        if (!Bento.objects.get('screenTransition')) {
            postTransition();
        } else {
            ScreenTransition.onTransitionComplete = postTransition;
        }

        Bento.objects.attach(entity);
        return entity;
    };
});