/**
 * Module description
 * @moduleName ScreenTransition
 * @snippet ScreenTransition.snippet
ScreenTransition({
    destination: '',
    position: new Vector2(0, 0)
})
 */
bento.define('ui/screentransition', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'components/spritecontainer'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    SpriteContainer
) {
    'use strict';
    var ScreenTransition = function (settings) {
        var viewport = Bento.getViewport();

        if (Bento.objects.get('screenTransition')) {
            Utils.log("Warning: Attempting new transition with transition in progress, cancelling new transition!");
            return;
        }

        // --- PARAMETERS ---
        var destination = settings.destination;
        var imageName = settings.imageName || 'transition_ring';
        var position = settings.position || new Vector2(viewport.width * 0.5, viewport.height * 0.5);
        var onScreenChange = settings.onScreenChange || function (complete) {
            complete();
        };

        // --- COMPONENTS ---
        var ring = new SpriteContainer({
            imageName: imageName,
            originRelative: new Vector2(0.5, 0.5),
            position: position.clone(),
            scale: new Vector2(2, 2)
        });
        var borderBehaviour = {
            name: "borderBehaviour",
            draw: function (data) {
                var r = data.renderer;
                var c = [0, 0, 0, 1];
                r.fillRect(c, 0, 0, viewport.width, ring.position.y - (256 * ring.scale.y));
                r.fillRect(c, 0, ring.position.y + (256 * ring.scale.y), viewport.width, viewport.height - (ring.position.y + (256 * ring.scale.y)));

                r.fillRect(c, 0, ring.position.y - (256 * ring.scale.y), ring.position.x - (256 * ring.scale.x), 512 * ring.scale.y);
                r.fillRect(c, ring.position.x + (256 * ring.scale.x), ring.position.y - (256 * ring.scale.y), viewport.width - (ring.position.x + (256 * ring.scale.x)), 512 * ring.scale.y);
            }
        };

        // --- ENTITY ---
        var entity = new Entity({
            z: 10000,
            name: 'screenTransition',
            family: [''],
            position: new Vector2(0, 0),
            updateWhenPaused: 0,
            float: true,
            global: true,
            components: [
                borderBehaviour,
                ring
            ]
        });

        // --- ANIMATION ---
        var tweenIn = new Tween.TweenBehavior({
            from: ring.scale.clone(),
            to: new Vector2(0, 0),
            in: 25,
            ease: 'easeOutQuad',
            onUpdate: function (v, t) {
                ring.scale = v;
            },
            onComplete: function () {
                tweenInEntity.removeSelf();
                onScreenChange(function () {
                    console.log('--- SCREEN CHANGED ---');
                    Bento.screens.show(destination);
                    Bento.objects.attach(tweenOutEntity);
                    ring.position = new Vector2(viewport.width * 0.5, viewport.height * 0.5);
                });
            }
        });
        var tweenInEntity = new Entity({
            name: 'tweenIn',
            global: true,
            components: [
                tweenIn
            ]
        });
        var tweenOut = new Tween.TweenBehavior({
            delay: 0,
            from: new Vector2(0, 0),
            to: new Vector2(2, 2),
            in: 25,
            ease: 'easeInQuad',
            onUpdate: function (v, t) {
                ring.scale = v;
            },
            onComplete: function () {
                tweenOutEntity.removeSelf();
                entity.removeSelf();
                if (ScreenTransition.onTransitionComplete) {
                    ScreenTransition.onTransitionComplete();
                    ScreenTransition.onTransitionComplete = null;
                }
            }
        });
        var tweenOutEntity = new Entity({
            name: 'tweenOut',
            global: true,
            components: [{
                name: 'tweenOutOveride',
                update: function () {
                    if (ScreenTransition.canTweenOut) {
                        tweenOutEntity.attach(tweenOut);
                        tweenOutEntity.remove(this);
                    }
                }
            }]
        });

        Bento.objects.attach(tweenInEntity);
        Bento.objects.attach(entity);
        return entity;
    };
    ScreenTransition.canTweenOut = true;
    ScreenTransition.onTransitionComplete = null;
    return ScreenTransition;
});