/**
 * Module description
 * @moduleName ShopDialogChina
 * @snippet ShopDialogChina.snippet
ShopDialogChina({
    position: new Vector2(0, 0)
})
 */
bento.define('ui/shopdialogchina', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'components/spritecontainer',
    'modules/localization',
    'bento/components/fill',
    'ui/iapoverlay',
    'bento/components/modal',
    'ui/coindialog',
    'modules/savemanager',
    'components/sortedclickable',
    'ui/vipdialog',
    'components/gamecounter',
    'bento/components/nineslice',
    'entities/particle',
    'ui/coinsforadshop',
    'modules/vipmanager'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    SpriteContainer,
    Localization,
    Fill,
    IapOverlay,
    Modal,
    CoinDialog,
    SaveManager,
    SortedClickable,
    VipDialog,
    GameCounter,
    NineSlice,
    Particle,
    CoinsForAdShop,
    VipManager
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var isAnimating = false;
        var onComplete = settings.onComplete || function () {};

        var close = function () {
            if (isAnimating) {
                return;
            }
            isAnimating = true;
            new Tween({
                from: 0.5,
                to: 0,
                in: 15,
                ignoreGameSpeed: true,
                onUpdate: function (v, t) {
                    backgroundFill.alpha = v;
                }
            });
            new Tween({
                from: new Vector2(1, 1),
                to: new Vector2(0.001, 0.001),
                in: 15,
                ease: 'easeInBack',
                onUpdate: function (v, t) {
                    mainRoot.scale = v;
                },
                onComplete: function () {
                    isAnimating = false;
                    entity.removeSelf();
                    onComplete();
                }
            });
        };

        // --- COMPONENTS ---
        var backgroundFill = new Entity({
            name: 'backgroundFill',
            alpha: 0.5,
            components: [
                new Fill({
                    dimension: new Rectangle(-viewport.width * 0.5, -viewport.height * 0.5, viewport.width, viewport.height),
                    color: [0, 0, 0, 1]
                })
            ]
        });

        var backboard = new Entity({
            name: 'backboard',
            scale: Globals.pixelScaleUIV.multiply(new Vector2(1, 0.5)),
            components: [
                new SpriteContainer({
                    imageName: 'ui/backboard',
                    originRelative: new Vector2(0.999, 0.999),
                    scale: new Vector2(1, 1)
                }),
                new SpriteContainer({
                    imageName: 'ui/backboard',
                    originRelative: new Vector2(0.999, 0.999),
                    scale: new Vector2(-1, 1)
                }),
                new SpriteContainer({
                    imageName: 'ui/backboard',
                    originRelative: new Vector2(0.999, 0.999),
                    scale: new Vector2(-1, -1)
                }),
                new SpriteContainer({
                    imageName: 'ui/backboard',
                    originRelative: new Vector2(0.999, 0.999),
                    scale: new Vector2(1, -1)
                })
            ]
        });

        var ribbon = new SpriteContainer({
            imageName: 'ui/ribbon',
            originRelative: new Vector2(0.5, 0.25),
            position: new Vector2(0, 0),
            scale: Globals.pixelScaleUIV.scalarMultiply(0.9)
        });
        var titleBanner = new Entity({
            name: 'titleBanner',
            family: [''],
            position: new Vector2(0, -48),
            components: [
                ribbon,
                new Text({
                    fontSettings: Utils.getTextStyle('dialogTitle'),
                    position: new Vector2(0, 2),
                    maxWidth: 108,
                    maxHeight: 24,
                    text: Localization.getText('coinBonus')
                })
            ]
        });



        var closeButton = new ClickButton({
            name: 'closeButton',
            sfx: 'sfx_clickbutton',
            imageName: 'ui/greenbutton',
            frameCountX: 1,
            frameCountY: 3,
            position: new Vector2(64, -48),
            scale: new Vector2(Globals.pixelScaleUI * 0.5, Globals.pixelScaleUI * 1),
            updateWhenPaused: 0,
            sort: true,
            onClick: function () {
                close();
            },
        });
        closeButton.attach(new SpriteContainer({
            imageName: 'ui/icons/close',
            position: new Vector2(0, 0),
            originRelative: new Vector2(0.5, 0.5),
            scale: new Vector2(1 / closeButton.scale.x, 1 / closeButton.scale.y).multiply(Globals.pixelScaleUIV).scalarMultiply(0.8)
        }));

        // --- SHOP ITEMS ---
        var coinsForAdShop = new CoinsForAdShop({
            position: new Vector2(0, 16)
        });


        var mainRoot = new Entity({
            name: 'mainRoot',
            boundingBox: new Rectangle(-78, -140, 156, 280),
            components: [
                new SortedClickable({}),
                backboard,
                coinsForAdShop,
                titleBanner,
                closeButton
            ]
        });

        // --- ENTITY ---
        var entity = new Entity({
            z: Globals.layers.modals,
            name: 'shopDialog',
            position: new Vector2(viewport.width * 0.5, viewport.height * 0.5),
            boundingBox: new Rectangle(-viewport.width * 0.5, -viewport.height * 0.5, viewport.width, viewport.height),
            updateWhenPaused: 0,
            float: true,
            components: [
                new Modal({}),
                new SortedClickable({
                    onClick: function (data) {
                        close();
                    }
                }),
                backgroundFill,
                mainRoot
            ]
        });
        Bento.objects.attach(entity);

        // animate it all in
        isAnimating = true;
        new Tween({
            from: 0,
            to: 0.5,
            in: 15,
            ignoreGameSpeed: true,
            onUpdate: function (v, t) {
                backgroundFill.alpha = v;
            }
        });
        new Tween({
            from: new Vector2(0.001, 0.001),
            to: new Vector2(1, 1),
            in: 20,
            ease: 'easeOutBack',
            onUpdate: function (v, t) {
                mainRoot.scale = v;
            },
            onComplete: function () {
                isAnimating = false;
            }
        });

        new Tween({
            from: 0,
            to: Globals.pixelScaleUI * 0.9,
            in: 45,
            delay: 10,
            ease: 'easeOutBack',
            onUpdate: function (v, t) {
                ribbon.scale.x = v;
            }
        });
        ribbon.scale.x = 0;

        new Tween({
            from: new Vector2(0.001, 0.001),
            to: new Vector2(1, 1),
            in: 15,
            delay: 10,
            ease: 'easeOutQuad',
            onUpdate: function (v, t) {
                titleBanner.scale = v;
            }
        });
        titleBanner.scale = new Vector2(0.001, 0.001);


        return entity;
    };
});