/**     
 * Module description
 * @moduleName SuperChestScreen
 * @snippet SuperChestScreen.snippet
SuperChestScreen({})
 */
bento.define('ui/superchestscreen', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'components/spritecontainer',
    'globals',
    'bento/components/modal',
    'bento/components/fill',
    'modules/localization',
    'modules/savemanager',
    'modules/ads',
    'ui/bonuswheel',
    'ui/potunlockeddialog',
    'ui/coindialog',
    'modules/skinmanager',
    'modules/taptic',
    'modules/abtest'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    SpriteContainer,
    Globals,
    Modal,
    Fill,
    Localization,
    SaveManager,
    Ads,
    BonusWheel,
    PotUnlockedDialog,
    CoinDialog,
    SkinManager,
    TapticEngine,
    ABTest
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();

        // --- VARS ---
        var wobbleChest = true;
        var watchingAd = false;
        var animating = false;
        var canRespin = true;
        var discardButtonTween;
        var currentReward;
        var rewardHidden = true;

        // --- PARAMETERS ---
        var skipAd = settings.skipAd || false;
        var onComplete = settings.onComplete || function () {};
        var doPause = Utils.isDefined(settings.doPause) ? settings.doPause : true;

        // --- FUNCTIONS ---
        var open = function () {
            if (animating) {
                return;
            }
            animating = true;
            new Tween({
                from: 0.5,
                to: 1,
                in: 100,
                decay: 8,
                oscilations: 5,
                ease: 'easeOutElastic',
                onUpdate: function (v, t) {
                    ribbon.scale.x = Globals.pixelScaleUI * v;
                }
            });
            new Tween({
                from: 0,
                to: 0.66,
                in: 20,
                ease: 'linear',
                onUpdate: function (v, t) {
                    backgroundFill.alpha = v;
                }
            });
            new Tween({
                from: new Vector2(0, 0),
                to: new Vector2(1, 1),
                in: 20,
                ease: 'easeOutBack',
                onUpdate: function (v, t) {
                    mainBox.scale = v;
                },
                onComplete: function () {
                    animating = false;
                }
            });
            // bloom halo burst outwards
            new Tween({
                from: 0,
                to: 1,
                in: 30,
                ease: 'easeOutQuad',
                onUpdate: function (v, t) {
                    bloomHalo.scale.x = bloomHalo.scale.y = 0.75 * v;
                    bloomHalo.alpha = (1 - v) * 0.6;
                },
            });
            // bloom grow and shrink
            new Tween({
                from: 0,
                to: 0.45,
                in: 30,
                ease: 'easeOutQuad',
                onUpdate: function (v, t) {
                    bloomHard.scale.x = v;
                    bloomHard.scale.y = v;
                },
                onComplete: function () {
                    new Tween({
                        from: 0.45,
                        to: 0.375,
                        in: 60,
                        ease: 'easeInOutCubic',
                        onUpdate: function (v, t) {
                            bloomHard.scale.x = v;
                            bloomHard.scale.y = v;
                        }
                    });
                }
            });
            if (ABTest.getAB1() === 'A') { // AB TEST
                //animate discard button delayed
                discardButtonTween = new Tween({
                    from: 0,
                    to: 1,
                    in: 15,
                    ease: 'easeOutExpo',
                    delay: 150,
                    onUpdate: function (v, t) {
                        discardButton.alpha = v;
                    }
                });
                discardButton.alpha = 0;
            } else {
                //animate discard button delayed
                discardButtonTween = new Tween({
                    from: 0,
                    to: 1,
                    in: 15,
                    ease: 'easeOutExpo',
                    delay: 120,
                    onUpdate: function (v, t) {
                        noThanksText.alpha = v;
                    }
                });
                rewardIcons.scale.x = 0;
                rewardIcons.scale.y = 0;
                new Tween({
                    from: 0,
                    to: 1,
                    in: 90,
                    delay: 15,
                    ease: 'easeOutBack',
                    onUpdate: function (v, t) {
                        rewardIcons.scale.x = v;
                        rewardIcons.scale.y = v;
                    }
                });
                noThanksText.alpha = 0;
            }
        };
        var openToWheel = function () {
            if (animating) {
                return;
            }
            backing.visible = false;
            animating = true;
            wobbleChest = false;
            discardButton.setActive(false);
            discardButton.visible = false;
            bonusButton.setActive(false);
            bonusButton.visible = false;
            rewardIcons.visible = false;
            new Tween({
                from: 0,
                to: 0.66,
                in: 20,
                ease: 'linear',
                onUpdate: function (v, t) {
                    backgroundFill.alpha = v;
                }
            });
            new Tween({
                from: new Vector2(0, 0),
                to: new Vector2(1, 1),
                in: 20,
                ease: 'easeOutBack',
                onUpdate: function (v, t) {
                    mainBox.scale = v;
                },
                onComplete: function () {
                    animating = false;
                }
            });
            // movethe chest to position
            new Tween({
                from: chestSprite.rotation,
                to: Math.PI * 2,
                in: 20,
                delay: 5,
                ease: 'easeOutExpo',
                onUpdate: function (v, t) {
                    chestSprite.rotation = v;
                }
            });
            new Tween({
                from: chestSprite.position.y,
                to: -184,
                in: 20,
                delay: 5,
                ease: 'easeOutExpo',
                onUpdate: function (v, t) {
                    chestSprite.position.y = v;
                },
                onComplete: function () {
                    new Tween({
                        from: chestSprite.position.y,
                        to: -92,
                        in: 10,
                        ease: 'easeInExpo',
                        onUpdate: function (v, t) {
                            chestSprite.position.y = v;
                        },
                        onComplete: function () {
                            new Tween({
                                from: Math.PI * 0.1,
                                to: 0,
                                in: 20,
                                ease: 'easeOutBack',
                                onUpdate: function (v, t) {
                                    chestSprite.rotation = v;
                                }
                            });
                            //screenshake
                            var origPos = mainBox.position.clone();
                            new Tween({
                                from: 1,
                                to: 0,
                                in: 15,
                                ease: 'linear',
                                onUpdate: function (v, t) {
                                    mainBox.position = origPos.add(new Vector2(Utils.getRandomFloat(v) * 4, 0).rotateDegree(Utils.getRandomFloat(360)));
                                }
                            });
                            TapticEngine.impact({
                                style: 'heavy'
                            });
                        }
                    });
                }
            });
            titleText.setText(Localization.getText('superChest'));
            titleBanner.position.y = -68;
            new Tween({
                from: 0,
                to: 1,
                in: 20,
                delay: 15,
                ease: 'easeOutBack',
                onUpdate: function (v, t) {
                    titleBanner.scale.x = v;
                }
            });
            titleBanner.scale.x = 0;
            new Tween({ in: 15,
                onComplete: function () {
                    bonusWheel.visible = true;
                }
            });
            new Tween({
                from: 0,
                to: bonusWheel.scale.x,
                in: 20,
                delay: 15,
                ease: 'easeOutBack',
                onUpdate: function (v, t) {
                    bonusWheel.scale.x = v;
                    bonusWheel.scale.y = v;
                },
                onComplete: function () {
                    // spin the wheel, initiating the slowdown
                    spinWheel();
                }
            });
            bonusWheel.scale = new Vector2(0, 0);
        };

        var discard = function () {
            if (animating) {
                return;
            }
            animating = true;
            new Tween({
                from: 0.5,
                to: 0,
                in: 20,
                delay: 30,
                ease: 'linear',
                onUpdate: function (v, t) {
                    backgroundFill.alpha = v;
                }
            });
            new Tween({
                from: new Vector2(1, 1),
                to: new Vector2(0, 0),
                in: 20,
                delay: 30,
                ease: 'easeInBack',
                onUpdate: function (v, t) {
                    mainBox.scale = v;
                },
                onComplete: function () {
                    entity.removeSelf();
                    onComplete();
                    animating = false;
                }
            });

            if (ABTest.getAB1() === 'A') { // AB TEST
                discardButton.setActive(false);
                discardButtonTween = new Tween({
                    from: 1,
                    to: 0,
                    in: 15,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        discardButton.alpha = v;
                    }
                });
            } else {
                noThanksButton.setActive(false);
                discardButtonTween = new Tween({
                    from: 1,
                    to: 0,
                    in: 15,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        noThanksText.alpha = v;
                        rewardIcons.alpha = v;
                    }
                });
            }

            bonusButton.setActive(true);
            new Tween({
                from: 1,
                to: 0,
                in: 15,
                ease: 'linear',
                onUpdate: function (v, t) {
                    bonusButton.alpha = v;
                }
            });
            new Tween({
                from: new Vector2(0.5, 0.5),
                to: new Vector2(0, 0),
                in: 30,
                ease: 'easeInBack',
                onUpdate: function (v, t) {
                    chestSprite.scale = v;
                }
            });
        };

        var transitionToWheel = function () {
            if (animating) {
                return;
            }
            animating = true;
            wobbleChest = false;
            discardButton.setActive(false);
            discardButtonTween.stop();
            if (ABTest.getAB1() === 'A') { // AB TEST
                discardButton.setActive(false);
                discardButtonTween = new Tween({
                    from: 1,
                    to: 0,
                    in: 15,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        discardButton.alpha = v;
                    }
                });
            } else {
                noThanksButton.setActive(false);
                discardButtonTween = new Tween({
                    from: 1,
                    to: 0,
                    in: 15,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        noThanksText.alpha = v;
                        rewardIcons.alpha = v;
                    }
                });
            }
            new Tween({
                from: 1,
                to: 0,
                in: 15,
                ease: 'linear',
                onUpdate: function (v, t) {
                    bonusButton.alpha = v;
                }
            });
            new Tween({
                from: new Vector2(1, 1),
                to: new Vector2(0, 0),
                in: 30,
                ease: 'easeInBack',
                onUpdate: function (v, t) {
                    backing.scale = v;
                }
            });
            // bloom halo burst outwards
            new Tween({
                from: 1,
                to: 0,
                in: 15,
                ease: 'easeInQuad',
                onUpdate: function (v, t) {
                    bloomHalo.scale.x = bloomHalo.scale.y = 0.25 * v;
                    bloomHalo.alpha = (1 - v) * 0.6;
                },
            });
            // bloom grow and shrink
            new Tween({
                from: 0.15,
                to: 0,
                in: 15,
                ease: 'easeOutQuad',
                onUpdate: function (v, t) {
                    bloomHard.scale.x = v;
                    bloomHard.scale.y = v;
                }
            });
            // movethe chest to position
            new Tween({
                from: chestSprite.rotation,
                to: Math.PI * 2,
                in: 20,
                ease: 'easeOutExpo',
                onUpdate: function (v, t) {
                    chestSprite.rotation = v;
                }
            });
            Bento.audio.stopSound('sfx_chestlaunch');
            Bento.audio.playSound('sfx_chestlaunch');
            new Tween({
                from: chestSprite.position.y,
                to: -184,
                in: 20,
                ease: 'easeOutExpo',
                onUpdate: function (v, t) {
                    chestSprite.position.y = v;
                },
                onComplete: function () {
                    new Tween({
                        from: chestSprite.position.y,
                        to: -92,
                        in: 10,
                        ease: 'easeInExpo',
                        onUpdate: function (v, t) {
                            chestSprite.position.y = v;
                        },
                        onComplete: function () {
                            new Tween({
                                from: Math.PI * 0.1,
                                to: 0,
                                in: 20,
                                ease: 'easeOutBack',
                                onUpdate: function (v, t) {
                                    chestSprite.rotation = v;
                                }
                            });
                            //screenshake
                            var origPos = mainBox.position.clone();
                            new Tween({
                                from: 1,
                                to: 0,
                                in: 15,
                                ease: 'linear',
                                onUpdate: function (v, t) {
                                    mainBox.position = origPos.add(new Vector2(Utils.getRandomFloat(v) * 4, 0).rotateDegree(Utils.getRandomFloat(360)));
                                }
                            });
                            TapticEngine.impact({
                                style: 'heavy'
                            });
                            Bento.audio.stopSound('sfx_chestthud');
                            Bento.audio.playSound('sfx_chestthud');
                        }
                    });
                }
            });
            new Tween({
                from: 1,
                to: 0,
                in: 30,
                ease: 'easeInBack',
                onUpdate: function (v, t) {
                    titleBanner.scale.x = v;
                },
                onComplete: function () {
                    titleText.setText(Localization.getText('superChest'));
                    titleBanner.position.y = -68;
                    new Tween({
                        from: 0,
                        to: 1,
                        in: 20,
                        delay: 10,
                        ease: 'easeOutBack',
                        onUpdate: function (v, t) {
                            titleBanner.scale.x = v;
                        }
                    });
                }
            });
            new Tween({ in: 30,
                onComplete: function () {
                    bonusWheel.visible = true;
                }
            });
            new Tween({
                from: 0,
                to: bonusWheel.scale.x,
                in: 20,
                delay: 30,
                ease: 'easeOutBack',
                onUpdate: function (v, t) {
                    bonusWheel.scale.x = v;
                    bonusWheel.scale.y = v;
                },
                onComplete: function () {
                    // spin the wheel, initiating the slowdown
                    spinWheel();
                }
            });
            bonusWheel.scale = new Vector2(0, 0);
        };

        var spinWheel = function () {
            chestSprite.position.y = -92;
            chestSprite.sprite.setAnimation('closed');
            mainBox.moveComponentTo(chestSprite, 1);
            bonusWheel.spin(null, function (reward) {
                animating = false;
                currentReward = reward;
                if (ABTest.getAB1() === 'A') { // AB TEST
                    showReward();
                } else {
                    openChest();
                }
            });
        };

        var hideReward = function () {
            //show box
            if (rewardHidden) {
                return;
            }
            rewardHidden = true;
            rewardBox.visible = true;
            new Tween({
                from: rewardBox.position.y,
                to: rewardBox.position.y + 128,
                in: 20,
                ease: 'easeInQuad',
                onUpdate: function (v, t) {
                    rewardBox.position.y = v;
                },
                onComplete: function () {
                    rewardBox.position.y = rewardBox.position.y - 128;
                    rewardBox.visible = false;
                }
            });

        };

        var showReward = function () {
            if (animating) {
                return;
            }
            if (!rewardHidden) {
                return;
            }
            rewardHidden = false;
            animating = true;
            //change visuals
            if (canRespin && !Ads.canShowRewarded()) {
                canRespin = false;
            }
            collectButton.setActive(true);
            if (canRespin) {
                reSpinButton.setActive(true);
                reSpinButton.position.x = 50;
                if (ABTest.getAB1() === 'B') { // AB TEST
                    spinAgainButton.setActive(true);
                    collectTextButton.position.x = 0;
                    collectText.position.x = 0;
                }
            } else {
                reSpinButton.setActive(false);
                reSpinButton.position.x = 5000;
                collectButton.position.x = 0;
                if (ABTest.getAB1() === 'B') { // AB TEST
                    spinAgainButton.setActive(false);
                    spinAgainButton.position.x = 5000;
                    collectTextButton.position.x = 5000;
                    collectText.position.x = 5000;
                }
            }
            rewardText.setText(currentReward.name + "!");
            //show box
            rewardBox.visible = true;
            new Tween({
                from: rewardBox.position.y + 128,
                to: rewardBox.position.y,
                in: 20,
                ease: 'easeOutBack',
                onUpdate: function (v, t) {
                    rewardBox.position.y = v;
                },
                onComplete: function () {
                    animating = false;
                }
            });
        };

        var openChest = function () {
            if (animating) {
                return;
            }
            animating = true;
            //hide reward window
            hideReward();
            Bento.audio.stopSound('sfx_chestlaunch');
            Bento.audio.playSound('sfx_chestlaunch');
            new Tween({
                from: chestSprite.position.y,
                to: chestSprite.position.y - 32,
                in: 10,
                ease: 'easeOutQuad',
                onUpdate: function (v, t) {
                    chestSprite.position.y = v;
                },
                onComplete: function () {
                    mainBox.moveComponentTo(chestSprite, 5);
                    new Tween({
                        from: chestSprite.position.y,
                        to: 32,
                        in: 20,
                        ease: 'easeOutBack',
                        onUpdate: function (v, t) {
                            chestSprite.position.y = v;
                        },
                        onComplete: function () {
                            animating = false;
                        }
                    });
                    new Tween({ in: 15,
                        onComplete: function () {
                            new Tween({
                                from: 0,
                                to: 1,
                                in: 15,
                                ease: 'linear',
                                onUpdate: function (v, t) {
                                    chestSprite.scale = new Vector2(0.5, 0.5).scalarMultiply(1 - (v * 0.25));
                                },
                                onComplete: function () {
                                    chestSprite.sprite.setAnimation('opening');
                                    new Tween({
                                        from: 0.75,
                                        to: 1,
                                        in: 15,
                                        ease: 'easeOutBack',
                                        onUpdate: function (v, t) {
                                            chestSprite.scale = new Vector2(0.5, 0.5).scalarMultiply(v);
                                        },
                                        onComplete: function () {
                                            handleReward(currentReward);
                                        }
                                    });
                                }
                            });
                        }
                    });
                }
            });
        };

        var handleReward = function (reward) {

            var complete = function () {
                if (ABTest.getAB1() === 'A') { // AB TEST
                    close();
                } else {
                    if (canRespin) {
                        showReward();
                    } else {
                        close();
                    }
                }
            };

            if (!reward || !reward.id) {
                close();
                return;
            }

            var idSplit = reward.id.split('~');
            var rewardType = idSplit[0];
            var rewardValue = idSplit[1];

            SaveManager.save('openedSuperChest', true);
            SaveManager.save('chestsComplete', SaveManager.load('chestsComplete') + 1);
            if (rewardType === 'pot') {
                EventSystem.fire('GameAnalytics-addDesignEvent', {
                    eventId: "potsUnlocked:" + rewardValue
                });
                EventSystem.fire("FirebaseAnalytics-logEvent", {
                    name: "pot_unlocked",
                    properties: {
                        name: rewardValue
                    }
                });
                SkinManager.makePotSkinUnlocked(rewardValue);
                SkinManager.setCurrentPotSkin(rewardValue);
                new PotUnlockedDialog({
                    potSkin: rewardValue,
                    doAnother: false,
                    onComplete: complete
                });
                return;
            }
            if (rewardType === 'coins') {
                var coins = parseInt(rewardValue);
                new CoinDialog({
                    numberOfCoins: coins,
                    onComplete: function () {
                        Utils.giveCoins(coins);
                        complete();
                    }
                });
                return;
            }
            complete();
        };

        var close = function () {
            if (animating) {
                return;
            }
            animating = true;
            new Tween({
                from: 0.5,
                to: 0,
                in: 20,
                delay: 30,
                ease: 'linear',
                onUpdate: function (v, t) {
                    backgroundFill.alpha = v;
                }
            });
            new Tween({
                from: new Vector2(1, 1),
                to: new Vector2(0, 0),
                in: 20,
                delay: 30,
                ease: 'easeInBack',
                onUpdate: function (v, t) {
                    mainBox.scale = v;
                },
                onComplete: function () {
                    entity.removeSelf();
                    onComplete();
                    animating = false;
                }
            });
            new Tween({
                from: new Vector2(0.5, 0.5),
                to: new Vector2(0, 0),
                in: 30,
                ease: 'easeInBack',
                onUpdate: function (v, t) {
                    chestSprite.scale = v;
                }
            });
        };

        // --- COMPONENTS ---
        var backgroundFill = new Entity({
            name: 'backgroundFill',
            alpha: 0.5,
            components: [
                new Fill({
                    dimension: new Rectangle(0, 0, viewport.width, viewport.height),
                    color: [0, 0, 0, 1]
                })
            ]
        });

        var backing = new Entity({
            name: 'backing',
            components: [
                new SpriteContainer({
                    imageName: 'ui/backboard',
                    originRelative: new Vector2(0.999, 0.999),
                    scale: Globals.pixelScaleUIV.multiply(settings.bgScale || new Vector2(1, 1)).multiply(new Vector2(1, 1))
                }),
                new SpriteContainer({
                    imageName: 'ui/backboard',
                    originRelative: new Vector2(0.999, 0.999),
                    scale: Globals.pixelScaleUIV.multiply(settings.bgScale || new Vector2(1, 1)).multiply(new Vector2(-1, 1))
                }),
                new SpriteContainer({
                    imageName: 'ui/backboard',
                    originRelative: new Vector2(0.999, 0.999),
                    scale: Globals.pixelScaleUIV.multiply(settings.bgScale || new Vector2(1, 1)).multiply(new Vector2(-1, -1))
                }),
                new SpriteContainer({
                    imageName: 'ui/backboard',
                    originRelative: new Vector2(0.999, 0.999),
                    scale: Globals.pixelScaleUIV.multiply(settings.bgScale || new Vector2(1, 1)).multiply(new Vector2(1, -1))
                }),
            ]
        });

        var ribbon = new SpriteContainer({
            imageName: 'ui/ribbon',
            originRelative: new Vector2(0.5, 0.25),
            position: new Vector2(0, 0),
            scale: Globals.pixelScaleUIV.clone().multiply(new Vector2(1, -1))
        });
        var titleText = new Text({
            fontSettings: Utils.getTextStyle('dialogTitle'),
            position: new Vector2(0, -3),
            text: Localization.getText('youGotSuperChest'),
            maxWidth: 136,
            maxHeight: 24,
            linebreaks: true
        });
        var titleBanner = new Entity({
            name: 'titleBanner',
            family: [''],
            position: new Vector2(0, -60),
            components: [
                ribbon,
                titleText
            ]
        });

        var discardButton = new ClickButton({
            name: 'closeButton',
            sfx: 'sfx_clickbutton',
            imageName: 'ui/greenbutton',
            frameCountX: 1,
            frameCountY: 3,
            position: new Vector2(-72, -4),
            scale: new Vector2(Globals.pixelScaleUI * 0.5, Globals.pixelScaleUI * 1),
            updateWhenPaused: 0,
            sort: true,
            onClick: function () {
                if (animating) {
                    return;
                }
                EventSystem.fire('GameAnalytics-addDesignEvent', {
                    eventId: "ads:superChestAd",
                    value: 0
                });
                discard();
            },
        });
        discardButton.attach(new SpriteContainer({
            imageName: 'ui/icons/discard',
            position: new Vector2(0, -32),
            originRelative: new Vector2(0.5, 0.5),
            scale: new Vector2(1 / discardButton.scale.x, 1 / discardButton.scale.y).multiply(Globals.pixelScaleUIV).scalarMultiply(0.8)
        }));
        discardButton.attach(new Text({
            position: new Vector2(0, 48),
            scale: new Vector2(1 / discardButton.scale.x, 1 / discardButton.scale.y).scalarMultiply(0.5),
            maxWidth: 48,
            maxHeight: 12,
            text: Localization.getText('discard'),
            fontSettings: Utils.getTextStyle('dialogTitle'),
            fontSize: 10,
            fontColor: '#fbffbe',
            align: 'center',
            textBaseline: 'middle',
            linebreaks: true
        }));

        var noThanksButton = new ClickButton({
            name: 'noThanksButton',
            sfx: 'sfx_clickbutton',
            imageName: 'ui/greenbutton',
            frameCountX: 1,
            frameCountY: 3,
            position: new Vector2(0, 108),
            scale: new Vector2(Globals.pixelScaleUI * 1.5, Globals.pixelScaleUI * 1),
            updateWhenPaused: 0,
            alpha: 0,
            sort: true,
            onClick: function () {
                if (animating) {
                    return;
                }
                EventSystem.fire('GameAnalytics-addDesignEvent', {
                    eventId: "ads:superChestAd",
                    value: 0
                });
                discard();
            },
        });
        var noThanksText = new Text({
            position: new Vector2(0, 108),
            scale: new Vector2(1, 1).scalarMultiply(1),
            maxWidth: 64,
            maxHeight: 12,
            text: Localization.getText('nothanks'),
            fontSettings: Utils.getTextStyle('dialogTitle'),
            fontSize: 16,
            fontColor: '#fbffbe',
            align: 'center',
            textBaseline: 'middle',
            linebreaks: true
        });

        var buttonChestSprite = new SpriteContainer({
            spriteSheet: 'ui/small-chest',
            position: new Vector2(-60, 55),
            alpha: 1,
            scale: new Vector2(0.5, 0.5)
        });
        buttonChestSprite.sprite.setAnimation('open');
        var bonusButton = new ClickButton({
            name: 'bonusButtonChest',
            alpha: 1,
            sfx: 'sfx_clickbutton',
            imageName: 'ui/bluebutton',
            frameCountX: 1,
            frameCountY: 3,
            position: new Vector2(0, 64),
            scale: Globals.pixelScaleUIV.clone().scalarMultiply(2),
            updateWhenPaused: 0,
            float: false,
            onClick: function () {
                if (watchingAd || animating) {
                    return;
                }
                bonusButton.setActive(false);
                //Bonus Chest
                if (Utils.isDev()) {
                    window.alert("This is a Rewarded Ad!");
                    Bento.input.resetPointers();
                    SaveManager.save('levelsSinceSkinAd', 0);
                    Globals.attemptsSinceAd = 0;
                    transitionToWheel();
                } else {
                    watchingAd = true;
                    Ads.showRewarded(function () {
                        EventSystem.fire('GameAnalytics-addDesignEvent', {
                            eventId: "ads:superChestAd",
                            value: 1
                        });
                        SaveManager.save('levelsSinceSkinAd', 0);
                        Globals.attemptsSinceAd = 0;
                        transitionToWheel();
                        watchingAd = false;
                    }, function (e) {
                        bonusButton.setActive(true);
                        watchingAd = false;
                    }, (ABTest.getAB1() === 'A') ? "SpinSuperChest_A" : "SpinSuperChest_B"); // AB TEST
                }
            },
            components: [
                new Text({
                    position: new Vector2(70, 0),
                    scale: new Vector2(0.5, 0.5).scalarMultiply(1 / Globals.pixelScaleUI),
                    text: Localization.getText('openIt'),
                    fontSettings: Utils.getTextStyle('dialogTitle'),
                    fontSize: 14,
                    fontColor: '#fbffbe',
                    align: 'center',
                    textBaseline: 'middle',
                    linebreaks: true,
                    maxWidth: 56,
                    maxHeight: 14
                }),
                new SpriteContainer({
                    imageName: 'ui/icons/vid-plant',
                    originRelative: new Vector2(0.5, 0.5),
                    position: new Vector2(-100, -12.5),
                    scale: new Vector2(0.5, 0.5)
                }),
                buttonChestSprite
            ]
        });

        var chestSprite = new SpriteContainer({
            spriteSheet: 'ui/small-chest',
            position: new Vector2(0, 0),
            scale: new Vector2(0.5, 0.5)
        });

        var coin1Icon = new Entity({
            name: 'coin1Icon',
            position: new Vector2(60, 18),
            scale: Globals.pixelScaleUIV,
            components: [
                new SpriteContainer({
                    imageName: 'ui/circle_bg',
                    originRelative: new Vector2(0.5, 0.5),
                    scale: new Vector2(1.5, 1.5)
                }),
                new SpriteContainer({
                    imageName: 'ui/shop/coinpack-mid',
                    originRelative: new Vector2(0.5, 0.5),
                    scale: new Vector2(0.85, 0.85)
                }), {
                    name: 'wobbleBehaviour',
                    startY: 18,
                    update: function () {
                        this.parent.position.y = this.startY + Math.sin((this.parent.ticker + 678678) * 0.01) * 3;
                        this.parent.rotation = Math.sin((this.parent.ticker + 1) * 0.035) * 0.1;
                    }
                }
            ]
        });

        var coin2Icon = new Entity({
            name: 'coin2Icon',
            position: new Vector2(-60, -18),
            scale: Globals.pixelScaleUIV,
            components: [
                new SpriteContainer({
                    imageName: 'ui/circle_bg',
                    originRelative: new Vector2(0.5, 0.5),
                    scale: new Vector2(1.5, 1.5)
                }),
                new SpriteContainer({
                    imageName: 'ui/shop/coinpack-small',
                    originRelative: new Vector2(0.5, 0.5),
                    scale: new Vector2(0.85, 0.85)
                }), {
                    name: 'wobbleBehaviour',
                    startY: -18,
                    update: function () {
                        this.parent.position.y = this.startY + Math.sin((this.parent.ticker + 5675675) * 0.01) * 3;
                        this.parent.rotation = Math.sin((this.parent.ticker + 23424) * 0.045) * 0.1;
                    }
                }
            ]
        });

        var pots = SkinManager.getWheelPots().slice();
        var pot1 = pots[Utils.getRandom(pots.length)];
        pots.splice(pots.indexOf(pot1), 1);
        var pot1Icon = new Entity({
            name: 'pot1Icon',
            position: new Vector2(-60, 18),
            scale: Globals.pixelScaleUIV,
            components: [
                new SpriteContainer({
                    imageName: 'ui/circle_bg',
                    originRelative: new Vector2(0.5, 0.5),
                    scale: new Vector2(1.5, 1.5)
                }),
                pot1 ? new SpriteContainer({
                    spriteSheet: ('boxes/' + pot1),
                    originRelative: new Vector2(0.5, 0.5),
                    scale: new Vector2(1.5, 1.5)
                }) : new SpriteContainer({
                    imageName: 'ui/shop/coinpack-huge',
                    originRelative: new Vector2(0.5, 0.5),
                    scale: new Vector2(0.85, 0.85)
                }), {
                    name: 'wobbleBehaviour',
                    startY: 18,
                    update: function () {
                        this.parent.position.y = this.startY + Math.sin((this.parent.ticker + 345575664) * 0.01) * 3;
                        this.parent.rotation = Math.sin((this.parent.ticker + 345575664) * 0.038) * 0.15;
                    }
                }
            ]
        });

        var pot2 = pots[Utils.getRandom(pots.length)];
        var pot2Icon = new Entity({
            name: 'pot2Icon',
            position: new Vector2(60, -18),
            scale: Globals.pixelScaleUIV,
            components: [
                new SpriteContainer({
                    imageName: 'ui/circle_bg',
                    originRelative: new Vector2(0.5, 0.5),
                    scale: new Vector2(1.5, 1.5)
                }),
                pot2 ? new SpriteContainer({
                    spriteSheet: ('boxes/' + pot2),
                    originRelative: new Vector2(0.5, 0.5),
                    scale: new Vector2(1.5, 1.5)
                }) : new SpriteContainer({
                    imageName: 'ui/shop/coinpack-small',
                    originRelative: new Vector2(0.5, 0.5),
                    scale: new Vector2(0.85, 0.85)
                }), {
                    name: 'wobbleBehaviour',
                    startY: -18,
                    update: function () {
                        this.parent.position.y = this.startY + Math.sin((this.parent.ticker + 3434) * 0.01) * 3;
                        this.parent.rotation = Math.sin((this.parent.ticker + 222) * 0.046) * 0.1;
                    }
                }
            ]
        });

        var rewardIcons = new Entity({
            name: 'rewardIcons',
            components: [
                pot1Icon,
                pot2Icon,
                coin1Icon,
                coin2Icon
            ]
        });

        var bloomHard = new SpriteContainer({
            imageName: 'ui/fx/bloom-hard',
            originRelative: new Vector2(0.5, 0.5),
            position: new Vector2(0, -8),
            scale: new Vector2(0, 0)
        });
        var bloomHalo = new SpriteContainer({
            imageName: 'ui/fx/bloom-halo',
            originRelative: new Vector2(0.5, 0.5),
            position: new Vector2(0, -8),
            scale: new Vector2(0, 0),
            alpha: 0.6
        });

        var bonusWheel = new BonusWheel({
            position: new Vector2(0, 36)
        });
        bonusWheel.visible = false;

        var rewardText = new Text({
            position: new Vector2(0, -112),
            scale: new Vector2(0.75, 0.75),
            maxWidth: 128,
            text: '',
            fontSettings: Utils.getTextStyle('wheelReward'),
            align: 'center',
            textBaseline: 'middle',
            linebreaks: true,
            maxHeight: 24
        });
        var collectButton = new ClickButton({
            name: 'collectButton',
            alpha: 1,
            sfx: 'sfx_clickbutton',
            imageName: 'ui/greenbutton',
            frameCountX: 1,
            frameCountY: 3,
            position: new Vector2(-26, -72),
            scale: Globals.pixelScaleUIV.clone().scalarMultiply(1.5),
            updateWhenPaused: 0,
            float: false,
            onClick: function () {
                openChest();
            },
            components: [
                new Text({
                    position: new Vector2(0, 4),
                    scale: new Vector2(0.75, 0.75).scalarMultiply(1 / Globals.pixelScaleUI),
                    maxWidth: 60,
                    text: Localization.getText('collect'),
                    fontSettings: Utils.getTextStyle('collectButton'),
                    fontColor: '#fbffbe',
                    align: 'center',
                    textBaseline: 'middle',
                    linebreaks: true,
                    maxHeight: 14
                })
            ]
        });
        collectButton.setActive(false);

        var collectTextButton = new ClickButton({
            name: 'collectTextButton',
            sfx: 'sfx_clickbutton',
            imageName: 'ui/greenbutton',
            frameCountX: 1,
            frameCountY: 3,
            position: new Vector2(0, -40),
            scale: new Vector2(Globals.pixelScaleUI * 1.1, Globals.pixelScaleUI * 0.7),
            updateWhenPaused: 0,
            alpha: 0,
            sort: true,
            onClick: function () {
                close();
            },
        });
        var collectText = new Text({
            position: new Vector2(0, -40),
            scale: new Vector2(1, 1).scalarMultiply(1),
            maxWidth: 64,
            maxHeight: 12,
            text: Localization.getText('nothanks'),
            fontSettings: Utils.getTextStyle('dialogTitle'),
            fontSize: 16,
            fontColor: '#fbffbe',
            align: 'center',
            textBaseline: 'middle',
            linebreaks: true
        });

        var reSpinButton = new ClickButton({
            name: 'reSpinButton',
            alpha: 1,
            sfx: 'sfx_clickbutton',
            imageName: 'ui/orangebutton',
            frameCountX: 1,
            frameCountY: 3,
            position: new Vector2(50, -72),
            scale: Globals.pixelScaleUIV.multiply(new Vector2(0.5, 1)).scalarMultiply(1.5),
            updateWhenPaused: 0,
            float: false,
            onClick: function () {
                if (watchingAd || animating) {
                    return;
                }
                //Res Spin Chest
                if (Utils.isDev()) {
                    window.alert("This is a Rewarded Ad!");
                    Bento.input.resetPointers();
                    SaveManager.save('levelsSinceSkinAd', 0);
                    Globals.attemptsSinceAd = 0;
                    hideReward();
                    spinWheel();
                    canRespin = false;
                } else {
                    watchingAd = true;
                    reSpinButton.setActive(false);
                    Ads.showRewarded(function () {
                        EventSystem.fire('GameAnalytics-addDesignEvent', {
                            eventId: "ads:superChestRespinAd",
                            value: 1
                        });
                        SaveManager.save('levelsSinceSkinAd', 0);
                        Globals.attemptsSinceAd = 0;
                        spinWheel();
                        hideReward();
                        watchingAd = false;
                        canRespin = false;
                    }, function (e) {
                        reSpinButton.setActive(true);
                        watchingAd = false;
                        canRespin = true;
                    }, (ABTest.getAB1() === 'A') ? "ReSpinSuperChest_A" : "ReSpinSuperChest_B"); // AB TEST
                }
            },
            components: [
                new SpriteContainer({
                    imageName: 'ui/icons/vid',
                    originRelative: new Vector2(0.5, 0.5),
                    position: new Vector2(0, -16),
                    scale: new Vector2(2, 1).scalarMultiply(0.5)
                }),
                new Text({
                    position: new Vector2(0, 60),
                    scale: new Vector2(0.5, 0.5).multiply(new Vector2(2, 1)).scalarMultiply(1 / Globals.pixelScaleUI),
                    maxWidth: 60,
                    text: Localization.getText('respin'),
                    fontSettings: Utils.getTextStyle('respin'),
                    fontSize: 14,
                    fontColor: '#fbffbe',
                    align: 'center',
                    textBaseline: 'middle',
                    linebreaks: true,
                    maxHeight: 14
                })
            ]
        });
        reSpinButton.setActive(false);

        var spinAgainButton = new ClickButton({
            name: 'spinAgainButton',
            alpha: 1,
            sfx: 'sfx_clickbutton',
            imageName: 'ui/orangebutton',
            frameCountX: 1,
            frameCountY: 3,
            position: new Vector2(0, -76),
            scale: Globals.pixelScaleUIV.multiply(new Vector2(1.33, 1)).scalarMultiply(1.4),
            updateWhenPaused: 0,
            float: false,
            onClick: function () {
                if (watchingAd || animating) {
                    return;
                }
                //Res Spin Chest
                if (Utils.isDev()) {
                    window.alert("This is a Rewarded Ad!");
                    Bento.input.resetPointers();
                    SaveManager.save('levelsSinceSkinAd', 0);
                    Globals.attemptsSinceAd = 0;
                    hideReward();
                    spinWheel();
                    canRespin = false;
                } else {
                    watchingAd = true;
                    spinAgainButton.setActive(false);
                    Ads.showRewarded(function () {
                        EventSystem.fire('GameAnalytics-addDesignEvent', {
                            eventId: "ads:superChestRespinAd",
                            value: 1
                        });
                        SaveManager.save('levelsSinceSkinAd', 0);
                        Globals.attemptsSinceAd = 0;
                        spinWheel();
                        hideReward();
                        watchingAd = false;
                        canRespin = false;
                    }, function (e) {
                        spinAgainButton.setActive(true);
                        watchingAd = false;
                        canRespin = true;
                    }, "ReSpinSuperChest_B");
                }
            },
            components: [
                new SpriteContainer({
                    imageName: 'ui/icons/vid',
                    originRelative: new Vector2(0.5, 0.5),
                    position: new Vector2(-84, 0),
                    scale: new Vector2(1 / 1.33, 1).scalarMultiply(0.66)
                }),
                new Text({
                    position: new Vector2(72, 0),
                    scale: new Vector2(0.8, 0.8).multiply(new Vector2(1 / 1.33, 1)).scalarMultiply(1 / Globals.pixelScaleUI),
                    text: Localization.getText('spinAgain'),
                    fontSettings: Utils.getTextStyle('respin'),
                    fontSize: 14,
                    fontColor: '#fbffbe',
                    align: 'center',
                    textBaseline: 'middle',
                    linebreaks: true,
                    maxWidth: 43,
                    maxHeight: 28
                })
            ]
        });
        spinAgainButton.setActive(false);

        var rewardBacking = new Entity({
            name: 'rewardBacking',
            components: [
                new SpriteContainer({
                    imageName: 'ui/backboard',
                    originRelative: new Vector2(0.999, 0.999),
                    scale: Globals.pixelScaleUIV.multiply(new Vector2(1.15, 1.5)).multiply(new Vector2(1, 1))
                }),
                new SpriteContainer({
                    imageName: 'ui/backboard',
                    originRelative: new Vector2(0.999, 0.999),
                    scale: Globals.pixelScaleUIV.multiply(new Vector2(1.15, 1.5)).multiply(new Vector2(-1, 1))
                }),
            ]
        });
        var rewardBox = new Entity({
            name: 'rewardBox',
            position: new Vector2(0, (viewport.height * 0.5) + ((ABTest.getAB1() === 'A') ? 40 : 20)), // AB TEST
            visible: false,
            components: [
                rewardBacking,
                rewardText,
                (ABTest.getAB1() === 'A') ? collectButton : collectTextButton, // AB TEST
                (ABTest.getAB1() === 'A') ? {
                    name: 'blank'
                } : collectText, // AB TEST
                (ABTest.getAB1() === 'A') ? reSpinButton : spinAgainButton, // AB TEST
            ]
        });

        var mainBox = new Entity({
            name: 'mainBackboard',
            position: new Vector2(viewport.width * 0.5, viewport.height * 0.5),
            components: [
                backing,
                bloomHalo,
                bloomHard,
                (ABTest.getAB1() === 'A') ? {
                    name: 'blank'
                } : rewardIcons, // AB TEST
                chestSprite,
                bonusWheel,
                titleBanner,
                (ABTest.getAB1() === 'A') ? discardButton : noThanksButton, // AB TEST
                (ABTest.getAB1() === 'A') ? {
                    name: 'blank'
                } : noThanksText, // AB TEST
                bonusButton,
                rewardBox
            ]
        });

        var behaviour = {
            name: "behaviour",
            update: function () {
                if (wobbleChest) {
                    chestSprite.rotation = Math.sin(entity.timer * 0.05) * 0.1;
                }
            },
            start: function () {
                if (skipAd) {
                    openToWheel();
                } else {
                    open();
                }
            }
        };

        // --- ENTITY ---
        var entity = new Entity({
            z: settings.z || Globals.layers.modals,
            name: 'dialog',
            position: new Vector2(0, 0),
            updateWhenPaused: 0,
            float: true,
            components: [
                (doPause) ? new Modal({}) : {
                    name: 'blank'
                },
                behaviour,
                backgroundFill,
                mainBox
            ]
        }).extend({
            discard: discard,
            open: open
        });

        //attach
        Bento.objects.attach(entity);

        return entity;
    };
});