/**
 * Module description
 * @moduleName WorldSelector
 * @snippet WorldSelector.snippet
 WorldSelector({
    position: new Vector2(0, 0)
})
 */
bento.define('ui/worldselector', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'bento/components/nineslice',
    'components/spritecontainer',
    'components/sortedclickable',
    'bento/components/fill',
    'ui/cointracker',
    'bento/gui/scrollinglist',
    'components/mask',
    'modules/localization',
    'modules/savemanager',
    'components/backbutton',
    'ui/levelselector',
    'modules/levelmanager',
    'modules/taptic'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    NineSlice,
    SpriteContainer,
    SortedClickable,
    Fill,
    CoinTracker,
    ScrollingList,
    Mask,
    Localization,
    SaveManager,
    BackButton,
    LevelSelector,
    LevelManager,
    TapticEngine
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();

        // --- VARS ---
        var scrollTween;
        var currentLevel = LevelManager.getCurrentLevel();
        var canClose = true;
        var openStatus = 'closed';
        var tabVisible = true;
        var panelPosition = new Vector2(viewport.width * 0.5, viewport.height + 160);
        var panelWidth = 160;
        var panelHeight = 240;

        // --- FUNCTIONS ---
        var open = function () {
            if (openStatus === 'closed') {
                // move to front
                if (entity && entity.parent) {
                    entity.parent.moveComponentTo(entity, entity.components.length + 128);
                }
                openStatus = 'moving';
                new Tween({
                    from: backgroundOverlay.alpha,
                    to: 0.4,
                    in: 15,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        backgroundOverlay.alpha = v;
                    }
                });
                hideTab();
                new Tween({
                    from: detailsPanel.position.y,
                    to: viewport.height + 48,
                    in: 15,
                    delay: 10,
                    ease: 'easeOutBack',
                    onUpdate: function (v, t) {
                        detailsPanel.position.y = v;
                    }
                });
                new Tween({
                    from: panelPosition.y,
                    to: viewport.height - (panelHeight * 0.5),
                    in: 15,
                    delay: 10,
                    ease: 'easeOutExpo',
                    onUpdate: function (v, t) {
                        panelPosition.y = v;
                    },
                    onComplete: function () {
                        openStatus = 'open';
                    }
                });
            }
        };

        var close = function () {
            if (!canClose) {
                return;
            }
            if (openStatus === 'open') {
                openStatus = 'moving';
                new Tween({
                    from: backgroundOverlay.alpha,
                    to: 0,
                    in: 15,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        backgroundOverlay.alpha = v;
                    }
                });
                showTab();
                new Tween({
                    from: detailsPanel.position.y,
                    to: viewport.height + 96,
                    in: 15,
                    ease: 'easeInBack',
                    onUpdate: function (v, t) {
                        detailsPanel.position.y = v;
                    }
                });
                new Tween({
                    from: panelPosition.y,
                    to: viewport.height + 160,
                    in: 15,
                    ease: 'easeOutQuad',
                    onUpdate: function (v, t) {
                        panelPosition.y = v;
                    },
                    onComplete: function () {
                        openStatus = 'closed';
                    }
                });
            }
        };

        var hideTab = function () {
            if (tabVisible) {
                tabVisible = false;
                new Tween({
                    from: tabEntity.position.y,
                    to: viewport.height + 28,
                    in: 15,
                    ease: 'easeInBack',
                    onUpdate: function (v, t) {
                        tabEntity.position.y = v;
                    }
                });
            }
        };

        var showTab = function () {
            if (!tabVisible) {
                tabVisible = true;
                new Tween({
                    from: tabEntity.position.y,
                    to: viewport.height - 28,
                    in: 15,
                    ease: 'easeOutBack',
                    onUpdate: function (v, t) {
                        tabEntity.position.y = v;
                    }
                });
            }
        };

        // --- COMPONENTS ---
        var tabFG = new Entity({
            name: 'tabFG',
            components: [
                new NineSlice({
                    imageName: 'ui/9slices/tabs_pressed',
                    originRelative: new Vector2(0.5, 0.5),
                    width: 640,
                    height: 260
                })
            ]
        });
        tabFG.visible = false;
        var tabEntity = new Entity({
            name: 'tabEntity',
            position: new Vector2(viewport.width * 0.5, viewport.height - 28),
            scale: Globals.pixelScaleUIV.scalarMultiply(0.8),
            components: [
                new NineSlice({
                    imageName: 'ui/9slices/tabs',
                    originRelative: new Vector2(0.5, 0.5),
                    width: 640,
                    height: 260
                }),
                tabFG, {
                    name: 'hideBehaviour',
                    update: function () {
                        this.parent.visible = (openStatus !== 'open');
                    }
                },
                new SpriteContainer({
                    imageName: 'ui/icons/sagamap',
                    originRelative: new Vector2(0.5, 0.5),
                    scale: new Vector2(1.25, 1.25),
                    position: new Vector2(0, -44)
                }),
                new Text({
                    fontSettings: Utils.getTextStyle('tabTitle'),
                    position: new Vector2(0, 56),
                    text: Localization.getText('levelSelect'),
                    scale: new Vector2(1 / Globals.pixelScaleUI, 1 / Globals.pixelScaleUI),
                    maxWidth: 72,
                    maxHeight: 18
                }),
                new SortedClickable({
                    onPressed: function () {
                        tabFG.visible = true;
                    },
                    onClick: function () {
                        open();
                        TapticEngine.impact({
                            style: 'light'
                        });
                        Bento.audio.playSound('sfx_clickbutton');
                    },
                    onRelease: function () {
                        tabFG.visible = false;
                    }
                })
            ]
        });

        // dark background
        var backgroundOverlay = new Entity({
            name: 'backgroundOverlay',
            alpha: 0,
            boundingBox: new Rectangle(0, 0, viewport.width, viewport.height),
            components: [
                new Fill({
                    dimension: new Rectangle(0, 0, viewport.width, viewport.height),
                    color: [25 / 255, 6 / 255, 51 / 255, 1]
                }), {
                    name: 'hideBehaviour',
                    update: function () {
                        this.parent.visible = (openStatus !== 'closed');
                    }
                },
                new SortedClickable({
                    onClick: function () {
                        close();
                    }
                })
            ]
        });


        // main panel
        var backgroundNineSlice = new Entity({
            name: 'backgroundNineSlice',
            position: panelPosition,
            scale: Globals.pixelScaleUIV,
            dimension: new Rectangle(0, 0, 1024, panelHeight),
            components: [
                new SortedClickable({}), {
                    name: 'hideBehaviour',
                    update: function () {
                        this.parent.visible = (openStatus !== 'closed');
                    }
                },
                new NineSlice({
                    imageName: 'ui/9slices/tabs',
                    originRelative: new Vector2(0.5, 0.5),
                    width: panelWidth / Globals.pixelScaleUI,
                    height: panelHeight / Globals.pixelScaleUI
                })
            ]
        });


        // make a holder for all the worlds
        var currentWorld = Math.min(Math.floor((currentLevel - 1) / 10), Globals.worldCount - 1);
        var worldOffset = new Vector2(0, -8);
        var worldSpacing = 160;
        var scrollToWorld = function (world) {
            currentWorld = world;
            worldTitle.setText(Localization.getText('world' + (currentWorld + 1)));
            worldList.snapTo(currentWorld);
        };

        var worldList = new ScrollingList({
            position: new Vector2(0, 0),
            area: new Rectangle(panelWidth * -0.5, panelHeight * -0.5, panelWidth, panelHeight * 0.8125), // area that user can grab
            direction: 'x',
            spacing: worldSpacing,
            snap: true,
            onSnap: function (item, index) {
                currentWorld = index;
                worldTitle.setText(Localization.getText('world' + (currentWorld + 1)));
                TapticEngine.impact({
                    style: 'light'
                });
            },
            snapMinSlideSpeed: 2,
            snapSpeed: 0.2,
            damping: 0.85
        });
        var isScrolling = function () {
            return (worldList.didMove(4));
        };
        // make all the worlds
        for (var i = 0; i < Globals.worldCount; i++) {
            var levelSelector = new LevelSelector({
                worldNumber: i,
                position: worldOffset.add(new Vector2(worldSpacing * i, 0)),
                addInfinite: (i === Globals.worldCount - 1),
                isScrolling: isScrolling
            });
            worldList.addItem(levelSelector);
        }

        var mainPanel = new Entity({
            name: 'mainPanel',
            position: panelPosition,
            components: [{
                    name: 'hideBehaviour',
                    update: function () {
                        this.parent.visible = (openStatus !== 'closed');
                    }
                },
                new Entity({
                    name: 'nineslice',
                    position: new Vector2(0, 11),
                    scale: Globals.pixelScaleUIV.scalarMultiply(2),
                    components: [
                        new NineSlice({
                            imageName: 'ui/9slices/tabs-inner',
                            originRelative: new Vector2(0.5, 0.5),
                            width: (panelWidth - 8) / (Globals.pixelScaleUI * 2),
                            height: (panelHeight - 11) / (Globals.pixelScaleUI * 2)
                        })
                    ]
                }),
                new Entity({
                    name: 'nineslicetop',
                    position: new Vector2(0, -panelHeight * 0.5),
                    scale: Globals.pixelScaleUIV,
                    components: [
                        new NineSlice({
                            imageName: 'ui/9slices/tabs-top',
                            originRelative: new Vector2(0.5, 0.5),
                            width: (panelWidth) / Globals.pixelScaleUI,
                            height: 200
                        })
                    ]
                }),
                new Entity({
                    name: 'titleText',
                    position: new Vector2(0, -panelHeight * 0.5),
                    components: [
                        new Text({
                            fontSettings: Utils.getTextStyle('levelSelectorTitle'),
                            position: new Vector2(0, 6),
                            text: Localization.getText('chooseStage'),
                            maxWidth: 96,
                            maxHeight: 12
                        })
                    ]
                }),
                worldList,
                new ClickButton({
                    name: 'closeButton',
                    sfx: 'sfx_clickbutton',
                    imageName: 'ui/greenbutton',
                    frameCountX: 1,
                    frameCountY: 3,
                    position: new Vector2((panelWidth * 0.5) - 16, (panelHeight * -0.5)),
                    scale: new Vector2(0.4, 0.8).scalarMultiply(Globals.pixelScaleUI),
                    sort: true,
                    onClick: function () {
                        close();
                    }
                }).attach(new SpriteContainer({
                    imageName: 'ui/icons/close',
                    originRelative: new Vector2(0.5, 0.5),
                    position: new Vector2(0, 0),
                    alpha: 1,
                    scale: new Vector2(0.5 / 0.3, 0.5 / 0.6),
                    rotation: 0
                }))
            ]
        });

        var prevButton = new ClickButton({
            name: 'prevButton',
            sfx: 'sfx_clickbutton',
            imageName: 'ui/greenbutton',
            frameCountX: 1,
            frameCountY: 3,
            position: new Vector2(-340, -64),
            scale: new Vector2(0.55, 1),
            sort: true,
            onClick: function () {
                if (currentWorld > 0) {
                    scrollToWorld(currentWorld - 1);
                }
            }
        }).attach(new SpriteContainer({
            imageName: 'ui/icons/play',
            originRelative: new Vector2(0.5, 0.5),
            position: new Vector2(0, 0),
            alpha: 1,
            scale: new Vector2(-0.4 / 0.55, 0.4 / 1),
            rotation: 0
        })).attach({
            name: 'activeBehaviour',
            update: function () {
                var thisActive = (currentWorld > 0);
                if (prevButton.active !== thisActive) {
                    prevButton.setActive(thisActive);
                    prevButton.alpha = (prevButton.active) ? 1 : 0.33;
                }
            }
        });

        var nextButton = new ClickButton({
            name: 'nextButton',
            sfx: 'sfx_clickbutton',
            imageName: 'ui/greenbutton',
            frameCountX: 1,
            frameCountY: 3,
            position: new Vector2(340, -64),
            scale: new Vector2(0.55, 1),
            sort: true,
            onClick: function () {
                if (currentWorld < (Globals.worldCount - 1)) {
                    scrollToWorld(currentWorld + 1);
                }
            }
        }).attach(new SpriteContainer({
            imageName: 'ui/icons/play',
            originRelative: new Vector2(0.5, 0.5),
            position: new Vector2(0, 0),
            alpha: 1,
            scale: new Vector2(0.4 / 0.55, 0.4 / 1),
            rotation: 0
        })).attach({
            name: 'activeBehaviour',
            update: function () {
                var thisActive = currentWorld < (Globals.worldCount - 1);
                if (nextButton.active !== thisActive) {
                    nextButton.setActive(thisActive);
                    nextButton.alpha = (nextButton.active) ? 1 : 0.33;
                }
            }
        });

        var worldTitle = new Text({
            fontSettings: Utils.getTextStyle('worldTitle'),
            position: new Vector2(0, -64),
            text: Localization.getText('world' + (currentWorld + 1)),
            scale: new Vector2(1 / Globals.pixelScaleUI, 1 / Globals.pixelScaleUI),
            maxWidth: 72,
            maxHeight: 18
        });

        var detailsMain = new Entity({
            name: 'detailsMain',
            position: new Vector2(0, -320),
            components: [
                prevButton,
                nextButton,
                worldTitle
            ]
        });

        var detailsPanel = new Entity({
            name: 'detailsPanel',
            position: new Vector2(viewport.width * 0.5, viewport.height + 96),
            scale: Globals.pixelScaleV.multiply(new Vector2(1, 1)),
            boundingBox: new Rectangle(-525, -493, 1050, 986),
            components: [{
                    name: 'hideBehaviour',
                    update: function () {
                        this.parent.visible = (openStatus !== 'closed');
                    }
                },
                new SpriteContainer({
                    imageName: 'ui/backboard',
                    originRelative: new Vector2(0.999, 0.999),
                    scale: new Vector2(1.05, 1)
                }),
                new SpriteContainer({
                    imageName: 'ui/backboard',
                    originRelative: new Vector2(0.999, 0.999),
                    scale: new Vector2(-1.05, 1)
                }),
                detailsMain
            ]
        });

        // --- ENTITY ---
        var entity = new Entity({
            z: Globals.layers.mainMenu,
            name: 'worldSelector',
            float: true,
            components: [
                backgroundOverlay,
                backgroundNineSlice,
                tabEntity,
                mainPanel,
                detailsPanel,
                new CoinTracker({
                    position: new Vector2(viewport.width - 12, 19),
                    playSound: false
                })

            ]
        }).extend({
            getOpenStatus: function () {
                return openStatus;
            },
            setCanClose: function (newCanClose) {
                canClose = newCanClose;
            },
            hideTab: hideTab,
            showTab: showTab
        });

        // --- SETUP ---
        scrollToWorld(currentWorld);

        return entity;
    };
});